﻿Type=Class
Version=6.5
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'Class module
'ClsConnection v1.0
Sub Class_Globals
	Public GPC As GPlayConnection
	Public NI As GPlayNetworkInfo
	Public Const ERROR_CONNECTION_FAILED As Byte = 0
	Public Const ERROR_CONNECTION_SUSPENDED As Byte = 1
	Public Const ERROR_NO_NETWORK As Byte = 2
	Public Const ERROR_SIGNIN_FAILED As Byte = 3
	Public Const ERROR_SIGNOUT As Byte = 4
	Public Const SUCCESS_SIGNIN As Byte = 0
	Public Const SUCCESS_SIGNOUT As Byte = 1
	Public Const SUCCESS_CONNECTED As Byte = 2
	Type typError(Code As Int, Msg As String, Error As Byte)
	Private clbkModule As Object
	Private clbkSuccess As String
	Private clbkFailure As String
End Sub

'Initializes the connection class.
Public Sub Initialize(GamesAPI As Boolean, SavedGamesAPI As Boolean, NearbyConnectionsAPI As Boolean, CallbackModule As Object, SuccessCallback As String, FailureCallback As String, ShowPopup As Boolean, DebugMode As Boolean)
	clbkModule = CallbackModule
	clbkSuccess = SuccessCallback
	clbkFailure = FailureCallback
	GPC.DEBUG = DebugMode
	Dim Options As GPlayConnectionOptions
	Options.GamesAPI = GamesAPI
	Options.NearbyConnectionsAPI = NearbyConnectionsAPI
	Options.SavedGamesAPI = SavedGamesAPI
	Options.ShowConnectionPopup = ShowPopup
	GPC.Initialize(Options, "GPC")
	NI.Initialize
	NI.RegisterConnectivityChangeListener
End Sub

'Auto-reconnects to Google Play services if disconnected.
Public Sub AutoConnect
	If NI.IsConnected Then
		If Not(IsAlreadyConnected) Then GPC.Connect(False)
	Else If GPC.ConnectionState <> GPC.STATE_DISCONNECTED Then
		'Not connected to a data network
		GPC.Disconnect(False)
		Dim Error As typError
		Error.Initialize
		Error.Code = GPC.RESULT_NETWORK_ERROR
		Error.Msg = "Not connected to a data network."
		Error.Error = ERROR_NO_NETWORK
		CallSub2(clbkModule, clbkFailure, Error)
	End If
End Sub

Public Sub IsConnected As Boolean
	Return GPC.IsConnected
End Sub

Private Sub IsAlreadyConnected As Boolean
	Dim AlreadyConnected As Boolean = GPC.IsConnected
	If AlreadyConnected Then
		If GPC.RequiresAuthentication Then
			AlreadyConnected = GPC.IsSignedIn
		End If
	End If
	Return AlreadyConnected
End Sub

'Connects to Google Play services.
Public Sub SignIn
	If NI.IsConnected Then
		If IsAlreadyConnected Then
			GPC.Reconnect
		Else
			GPC.Connect(True)
		End If
	Else
		'Not connected to a data network
		GPC.Disconnect(False)
		Dim Error As typError
		Error.Initialize
		Error.Code = GPC.RESULT_NETWORK_ERROR
		Error.Msg = "Not connected to a data network."
		Error.Error = ERROR_NO_NETWORK
		CallSub2(clbkModule, clbkFailure, Error)
	End If
End Sub

Public Sub IsSignedIn As Boolean
	Return GPC.IsSignedIn
End Sub

'Signs the user out but does not disconnect the Google API client.
Public Sub SignOut
	GPC.SignOut
End Sub

'Signs the user out and disconnects the Google API client.
Public Sub SignOutAndDisconnect
	GPC.Disconnect(GPC.IsSignedIn)
	NI.UnregisterConnectivityChangeListener
End Sub

Private Sub GPC_onClientConnected
	If GPC.DEBUG Then Log("GPC_onClientConnected")
	CallSub2(clbkModule, clbkSuccess, SUCCESS_CONNECTED)
End Sub

Private Sub GPC_onConnectionFailed(ErrorCode As Int, ErrorMsg As String, Resolvable As Boolean)
	If GPC.DEBUG Then Log("GPC_onConnectionFailed: ErrorMsg=""" & ErrorMsg & """ Resolvable=" & Resolvable)
	If Resolvable Then
		If GPC.ConnectionState = GPC.STATE_DISCONNECTED Then
			GPC.ShowErrorDialog(ErrorCode)
		End If
	Else
		Dim Error As typError
		Error.Initialize
		Error.Code = ErrorCode
		Error.Msg = ErrorMsg
		Error.Error = ERROR_CONNECTION_FAILED
		CallSub2(clbkModule, clbkFailure, Error)
	End If
End Sub

Private Sub GPC_onConnectionSuspended(Cause As Int, CauseMsg As String)
	If GPC.DEBUG Then Log("GPC_onConnectionSuspended: Cause=""" & CauseMsg & """")
	Dim Error As typError
	Error.Initialize
	Error.Code = Cause
	Error.Msg = CauseMsg
	Error.Error = ERROR_CONNECTION_SUSPENDED
	CallSub2(clbkModule, clbkFailure, Error)
End Sub

Private Sub GPC_onGivingUp(ErrorCode As Int, ErrorMsg As String, ResolveCode As Int, ResolveMsg As String)
	If GPC.DEBUG Then Log("GPC_onGivingUp: ErrorMsg=""" & ErrorMsg & """ ResolveMsg=""" & ResolveMsg & """")
End Sub

Private Sub GPC_onSignInFailed(ReasonCode As Int, ReasonMsg As String)
	If GPC.DEBUG Then Log("GPC_onSignInFailed: Reason=""" & ReasonMsg & """")
	Dim Error As typError
	Error.Initialize
	Error.Code = ReasonCode
	Error.Msg = ReasonMsg
	Error.Error = ERROR_SIGNIN_FAILED
	CallSub2(clbkModule, clbkFailure, Error)
End Sub

Private Sub GPC_onSignInSucceeded
	If GPC.DEBUG Then Log("GPC_onSignInSucceeded")
	CallSub2(clbkModule, clbkSuccess, SUCCESS_SIGNIN)
End Sub

Private Sub GPC_onConnectivityChange(NetworkType As Int, IsConnectedOrConnecting As Boolean)
	If GPC.DEBUG Then Log("GPC_onConnectivityChange: NetworkType=" & NetworkType & " IsConnectedOrConnecting=" & IsConnectedOrConnecting)
	If Not(IsConnectedOrConnecting) And Not(NI.IsFailover) And GPC.IsSignedIn Then
		'Connectivity is lost for a signed-in user and there's no failover
		GPC.Disconnect(False)
		Dim Error As typError
		Error.Initialize
		Error.Code = GPC.RESULT_NETWORK_ERROR
		Error.Msg = "Connection loss"
		Error.Error = ERROR_NO_NETWORK
		CallSub2(clbkModule, clbkFailure, Error)
	End If
End Sub

Private Sub GPC_onSignOutComplete(StatusCode As Int)
	If GPC.DEBUG Then Log("GPC_onSignOutComplete: StatusCode=" & StatusCode)
	If StatusCode = 0 Then
		CallSub2(clbkModule, clbkSuccess, SUCCESS_SIGNOUT)
	Else
		Dim Error As typError
		Error.Initialize
		Error.Code = StatusCode
		Error.Msg = "Error during sign-out"
		Error.Error = ERROR_SIGNOUT
		CallSub2(clbkModule, clbkFailure, Error)
	End If
End Sub
