﻿Type=StaticCode
Version=6.31
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'Code module
'CodTurnBasedMatch v1.1
Sub Process_Globals
	Public Invitations As GPlayInvitations
	Public Players As GPlayPlayers
	Public TBMulti As GPlayTurnBasedMultiplayer
	Public GPUtils As GPlayUtils
	Public Const PARTICIPANT_CURRENT As Byte = 0
	Public Const PARTICIPANT_NEXT As Byte = 1
	Private PlayerID As String
End Sub

'Initializes this helper module and registers the listeners
Public Sub Initialize(RegisterInvitationListener As Boolean, RegisterMatchUpdateListener As Boolean)
	'Initializes all classes
	Invitations.Initialize
	Players.Initialize
	TBMulti.Initialize

	'Registers the listeners and stops the status bar notifications
	If RegisterInvitationListener Then
		Invitations.RegisterInvitationListener
	End If
	If RegisterMatchUpdateListener Then
		TBMulti.RegisterMatchUpdateListener
	End If

	'Stores the player ID for further use
	PlayerID = Players.CurrentPlayerId
End Sub

'Creates a TB match with the given number of players (current player excluded) and variant
Public Sub QuickMatch(MinPlayers As Int, MaxPlayers As Int, Variant As Int, EventName As String)
	Dim TBMC As GPlayTurnBasedMatchConfig
	Dim Criteria As Object = TBMC.CreateAutoMatchCriteria(MinPlayers, MaxPlayers, 0)
	TBMC.InitializeWithAutoMatch2(Null, Criteria, Variant)
	TBMulti.CreateMatch(TBMC).SetResultEvent(EventName)
End Sub

'Creates a TB match with the given configuration
Public Sub CreateMatch(Config As GPlayTurnBasedMatchConfig, EventName As String)
	TBMulti.CreateMatch(Config).SetResultEvent(EventName)
End Sub

'Returns the position of the current player in the list of participants
Public Sub GetMyPosition(Match As GPlayTurnBasedMatch) As Int
	Dim MyParticipantId As String = Match.GetParticipantId(PlayerID)
	Dim lstParticipants As List = Match.GetParticipantIds
	For i = 0 To lstParticipants.Size - 1
		Dim PId As String = lstParticipants.Get(i)
		If PId = MyParticipantId Then
			Return i
		End If
	Next
	Return -1
End Sub

'Returns the participant ID of the current player
Public Sub GetMyParticipantId(Match As GPlayTurnBasedMatch) As String
	Return Match.GetParticipantId(PlayerID)
End Sub

'Returns the ID of the next participant
Public Sub GetNextParticipantId(Match As GPlayTurnBasedMatch) As String
	Dim MyParticipantId As String = Match.GetParticipantId(PlayerID)
	Dim lstParticipants As List = Match.GetParticipantIds
	For i = 0 To lstParticipants.Size - 1
		Dim PId As String = lstParticipants.Get(i)
		If PId = MyParticipantId Then
			If i < lstParticipants.Size - 1 Then
				Return lstParticipants.Get(i + 1)
			Else
				Return lstParticipants.Get(0)
			End If
		End If
	Next
	Return ""
End Sub

'Updates the data of the TB match on the Google's servers
'Participant: either PARTICIPANT_CURRENT to save the match data of the current player, or PARTICIPANT_NEXT to pass the turn to the next participant
Public Sub TakeTurn(Match As GPlayTurnBasedMatch, arrData() As Byte, Participant As Byte, EventName As String)
	Dim PendingParticipantId As String
	If Participant = PARTICIPANT_CURRENT Then
		PendingParticipantId = Match.GetParticipantId(PlayerID)
	Else If Participant = PARTICIPANT_NEXT And Match.AvailableAutoMatchSlots = 0 Then
		PendingParticipantId = GetNextParticipantId(Match)
	End If
	TBMulti.TakeTurn(Match.MatchId, arrData, PendingParticipantId).SetResultEvent(EventName)
End Sub

'Loads the specified TB match
Public Sub LoadMatch(MatchId As String, EventName As String)
	TBMulti.LoadMatch(MatchId).SetResultEvent(EventName)
End Sub

'Loads a list of TB matches with any of the specified status codes
Public Sub LoadMatches(TurnStatuses() As Int, EventName As String)
	TBMulti.LoadMatchesByStatus(TurnStatuses).SetResultEvent(EventName)
End Sub

'Cancels the specified TB match
Public Sub CancelMatch(MatchId As String, EventName As String)
	TBMulti.CancelMatch(MatchId).SetResultEvent(EventName)
End Sub

'Leaves the specified TB match
Public Sub LeaveMatch(Match As GPlayTurnBasedMatch, EventName As String)
	If Match.TurnStatus = Match.MATCH_TURN_STATUS_MY_TURN Then
		TBMulti.LeaveMatchDuringTurn(Match.MatchId, GetNextParticipantId(Match)).SetResultEvent(EventName)
	Else
		TBMulti.LeaveMatch(Match.MatchId).SetResultEvent(EventName)
	End If
End Sub

'Finishes the specified TB match
Public Sub FinishMatch(MatchId As String, EventName As String)
	TBMulti.FinishMatch(MatchId).SetResultEvent(EventName)
End Sub

'Finishes the specified TB match.
'If the match ends in a tie, WinnerID must be empty.
Public Sub FinishMatch2(Match As GPlayTurnBasedMatch, arrData() As Byte, WinnerID As String, EventName As String)
	Dim lstPR As List: lstPR.Initialize
	Dim lstParticipants As List = Match.GetParticipantIds
	For i = 0 To lstParticipants.Size - 1
		Dim PId As String = lstParticipants.Get(i)
		Dim PR As GPlayParticipantResult
		Dim Result As Int
		If WinnerID = "" Then
			Result = PR.MATCH_RESULT_TIE
		Else If WinnerID = PId Then
			Result = PR.MATCH_RESULT_WIN
		Else
			Result = PR.MATCH_RESULT_LOSS
		End If
		PR.Initialize(PId, Result, PR.PLACING_UNINITIALIZED)
		lstPR.Add(PR)
	Next
	TBMulti.FinishMatch2(Match.MatchId, arrData, lstPR).SetResultEvent(EventName)
End Sub

'Opens the "Select opponents" dialog
Public Sub OpenSelectOpponentsDialog(MinPlayers As Int, MaxPlayers As Int, AllowAutomatch As Boolean, DialogID As String)
	Dim i As Intent = TBMulti.GetSelectOpponentsIntent(MinPlayers, MaxPlayers, AllowAutomatch)
	GPUtils.StartActivityForResult(i, DialogID)
End Sub

'Opens the "Inbox" dialog
Public Sub OpenInboxDialog(DialogID As String)
	Dim i As Intent = TBMulti.GetInboxIntent
	GPUtils.StartActivityForResult(i, DialogID)
End Sub

'Opens the "Invitations" dialog
Public Sub OpenInvitationsDialog(DialogID As String)
	Dim i As Intent = Invitations.GetInvitationInboxIntent
	GPUtils.StartActivityForResult(i, DialogID)
End Sub

'Returns the match object, or null
Public Sub RetrieveMatchFrom(rIntent As Intent) As Object
	If rIntent.HasExtra(TBMulti.EXTRA_TURN_BASED_MATCH) Then
		Return TBMulti.GetMatchFromIntentExtra(rIntent)
	Else
		Return Null
	End If
End Sub

'Returns the list of invitees, or null
Public Sub RetrieveInviteesFrom(rIntent As Intent) As Object
	If rIntent.HasExtra(TBMulti.EXTRA_PLAYER_IDS) Then
		Return TBMulti.GetPlayerIDsFromIntentExtra(rIntent)
	Else
		Return Null
	End If
End Sub

'Returns the minimum number of additional auto-match players
Public Sub RetrieveMinAutomatchPlayersFrom(rIntent As Intent) As Int
	If rIntent.HasExtra(TBMulti.EXTRA_MIN_AUTOMATCH_PLAYERS) Then
		Return rIntent.GetExtra(TBMulti.EXTRA_MIN_AUTOMATCH_PLAYERS)
	End If
	Return 0
End Sub

'Returns the maximum number of additional auto-match players
Public Sub RetrieveMaxAutomatchPlayersFrom(rIntent As Intent) As Int
	If rIntent.HasExtra(TBMulti.EXTRA_MAX_AUTOMATCH_PLAYERS) Then
		Return rIntent.GetExtra(TBMulti.EXTRA_MAX_AUTOMATCH_PLAYERS)
	End If
	Return 0
End Sub

Public Sub GetStatusCode(Result As Map) As Int
	Return Result.Get(TBMulti.RESULTMAP_STATUS_CODE)
End Sub

Public Sub IsStatusOK(Status As Int) As Boolean
	Return (Status = TBMulti.STATUS_OK)
End Sub

Public Sub GetMatch(Result As Map) As GPlayTurnBasedMatch
	Return Result.Get(TBMulti.RESULTMAP_MATCH)
End Sub

Public Sub GetMatchesResponse(Result As Map) As GPlayLoadMatchesResponse
	Return Result.Get(TBMulti.RESULTMAP_MATCHES)
End Sub
