package flm.b4a.googleplay;

import android.net.Uri;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.objects.ImageViewWrapper;

import com.google.android.gms.games.achievement.Achievement;

@ShortName("GPlayAchievement")
@BA.Events(values = {
		"onImageLoaded(URI As String, Drawable As BitmapDrawable)"
})
public class AchievementWrapper
{
	protected Achievement _Achvmt;

	public AchievementWrapper()
	{
		_Achvmt = null;
	}

	public AchievementWrapper(Achievement achievement)
	{
		_Achvmt = achievement;
	}

	public boolean IsInitialized()
	{
		return _Achvmt != null;
	}

	public static final int STATE_HIDDEN = Achievement.STATE_HIDDEN;
	public static final int STATE_REVEALED = Achievement.STATE_REVEALED;
	public static final int STATE_UNLOCKED = Achievement.STATE_UNLOCKED;
	public static final int TYPE_INCREMENTAL = Achievement.TYPE_INCREMENTAL;
	public static final int TYPE_STANDARD = Achievement.TYPE_STANDARD;

	/**
	 * Retrieves the ID of this achievement.
	 */
	public String getAchievementId()
	{
		return Utils.StringOrEmpty(_Achvmt.getAchievementId());
	}

	/**
	 * Retrieves the number of steps this user has gone toward unlocking this achievement; only applicable for TYPE_INCREMENTAL achievement types.
	 */
	public int getCurrentSteps()
	{
		return _Achvmt.getCurrentSteps();
	}

	/**
	 * Retrieves the description for this achievement.
	 */
	public String getDescription()
	{
		return Utils.StringOrEmpty(_Achvmt.getDescription());
	}

	/**
	 * Retrieves the number of steps this user has gone toward unlocking this achievement (formatted for the user's locale); only applicable for TYPE_INCREMENTAL achievement types.
	 */
	public String getFormattedCurrentSteps()
	{
		return Utils.StringOrEmpty(_Achvmt.getFormattedCurrentSteps());
	}

	/**
	 * Retrieves the total number of steps necessary to unlock this achievement, formatted for the user's locale; only applicable for TYPE_INCREMENTAL achievement types.
	 */
	public String getFormattedTotalSteps()
	{
		return Utils.StringOrEmpty(_Achvmt.getFormattedTotalSteps());
	}

	/**
	 * Retrieves the timestamp (in milliseconds since epoch) at which this achievement was last updated. If the achievement has never been updated, this will return -1.
	 */
	public long getLastUpdatedTimestamp()
	{
		return _Achvmt.getLastUpdatedTimestamp();
	}

	/**
	 * Retrieves the name of this achievement.
	 */
	public String getName()
	{
		return Utils.StringOrEmpty(_Achvmt.getName());
	}

	/**
	 * Retrieves the player information associated with this achievement.
	 */
	public PlayerWrapper getPlayer()
	{
		return Utils.PlayerWrapperOrNull(_Achvmt.getPlayer());
	}

	/**
	 *Loads the achievement's revealed image icon and fills the given ImageView.
	 *Returns the URI of the image, or an empty string if the achievement has no revealed image.
	 */
	public String GetRevealedImage(final BA ba, ImageViewWrapper ImageView)
	{
		Uri uri = _Achvmt.getRevealedImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("GetRevealedImage: uri=" + uri);
		Utils.LoadImage(ba, uri, ImageView);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 *Loads the achievement's revealed image icon and raises an onImageLoaded event when the load is complete.
	 *Returns the URI of the image, or an empty string if the achievement has no revealed image.
	 */
	public String GetRevealedImage2(final BA ba)
	{
		Uri uri = _Achvmt.getRevealedImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("GetRevealedImage2: uri=" + uri);
		Utils.LoadImage(ba, uri, null);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 * Retrieves the state of the achievement - one of STATE_UNLOCKED, STATE_REVEALED, or STATE_HIDDEN.
	 */
	public int getState()
	{
		return _Achvmt.getState();
	}

	/**
	 * Retrieves the total number of steps necessary to unlock this achievement; only applicable for TYPE_INCREMENTAL achievement types.
	 */
	public int getTotalSteps()
	{
		return _Achvmt.getTotalSteps();
	}

	/**
	 * Retrieves the type of this achievement - one of TYPE_STANDARD or TYPE_INCREMENTAL.
	 */
	public int getType()
	{
		return _Achvmt.getType();
	}

	/**
	 *Loads the achievement's unlocked image icon and fills the given ImageView.
	 *Returns the URI of the image, or an empty string if the achievement has no unlocked image.
	 */
	public String GetUnlockedImage(final BA ba, ImageViewWrapper ImageView)
	{
		Uri uri = _Achvmt.getUnlockedImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("GetUnlockedImage: uri=" + uri);
		Utils.LoadImage(ba, uri, ImageView);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 *Loads the achievement's unlocked image icon and raises an onImageLoaded event when the load is complete.
	 *Returns the URI of the image, or an empty string if the achievement has no unlocked image.
	 */
	public String GetUnlockedImage2(final BA ba)
	{
		Uri uri = _Achvmt.getUnlockedImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("GetUnlockedImage2: uri=" + uri);
		Utils.LoadImage(ba, uri, null);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 * Retrieves the XP value given to players for unlocking this achievement.
	 */
	public long getXpValue()
	{
		return _Achvmt.getXpValue();
	}
}
