package flm.b4a.googleplay;

import android.content.Intent;
import anywheresoftware.b4a.BA;

import com.google.android.gms.common.api.PendingResult;
import com.google.android.gms.games.Games;
import com.google.android.gms.games.GamesStatusCodes;
import com.google.android.gms.games.achievement.Achievements;
import com.google.android.gms.games.achievement.Achievements.LoadAchievementsResult;
import com.google.android.gms.games.achievement.Achievements.UpdateAchievementResult;

@BA.ShortName("GPlayAchievements")
public class AchievementsWrapper
{
	protected Achievements _Ach;

	public AchievementsWrapper()
	{
		_Ach = null;
	}

	public void Initialize()
	{
		_Ach = Games.Achievements;
	}

	public boolean IsInitialized()
	{
		return _Ach != null;
	}

	public static final String RESULTMAP_ACHIEVEMENT_ID = PendingResultWrapper.RESULTMAP_ACHIEVEMENT_ID;
	public static final String RESULTMAP_ACHIEVEMENTS = PendingResultWrapper.RESULTMAP_ACHIEVEMENTS;
	public static final String RESULTMAP_STATUS_CODE = PendingResultWrapper.RESULTMAP_STATUS_CODE;
	public final static int STATUS_ACHIEVEMENT_NOT_INCREMENTAL = GamesStatusCodes.STATUS_ACHIEVEMENT_NOT_INCREMENTAL;
	public final static int STATUS_ACHIEVEMENT_UNKNOWN = GamesStatusCodes.STATUS_ACHIEVEMENT_UNKNOWN;
	public final static int STATUS_ACHIEVEMENT_UNLOCK_FAILURE = GamesStatusCodes.STATUS_ACHIEVEMENT_UNLOCK_FAILURE;
	public final static int STATUS_ACHIEVEMENT_UNLOCKED = GamesStatusCodes.STATUS_ACHIEVEMENT_UNLOCKED;
	public final static int STATUS_OK = GamesStatusCodes.STATUS_OK;

	/**
	 *Gets an intent to show the list of achievements for a game. This intent must be invoked with StartActivityForResult.
	 *A RESULT_RECONNECT_REQUIRED may be returned as the result code in the onActivityResult event if the Google API client ends up in an inconsistent state.
	 */
	public Intent GetAchievementsIntent()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Achievements.GetAchievementsIntent");
		return _Ach.getAchievementsIntent(GooglePlayConnection.getAPIclient());
	}

	/**
	 *Increments an achievement by the given number of steps. The achievement must be an incremental achievement. Once an achievement reaches at least the maximum number of steps, it will be unlocked automatically. Any further increments will be ignored.
	 *Use this function if you don't need to know the status of the operation immediately. For most applications, this will be the preferred function to use, though note that the update may not be sent to the server until the next sync. See IncrementImmediate if you need the operation to attempt to communicate to the server immediately or need to have the status code delivered to your application.
	 *@param Id The ID of the achievement to increment.
	 *@param NumSteps The number of steps to increment by. Must be greater than 0.
	 */
	public void Increment(String Id, int NumSteps)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Achievements.Increment: " + Id + " numsteps=" + NumSteps);
		_Ach.increment(GooglePlayConnection.getAPIclient(), Id, NumSteps);
	}

	/**
	 *Increments an achievement by the given number of steps. The achievement must be an incremental achievement. Once an achievement reaches at least the maximum number of steps, it will be unlocked automatically. Any further increments will be ignored.
	 *This function will attempt to update the user's achievement on the server immediately, and will return a PendingResult that can be used to retrieve the result.
	 *@param Id The ID of the achievement to increment.
	 *@param NumSteps The number of steps to increment by. Must be greater than 0.
	 */
	public PendingResultWrapper IncrementImmediate(String Id, int NumSteps)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Achievements.IncrementImmediate: " + Id + " numsteps=" + NumSteps);
		PendingResult<UpdateAchievementResult> PR = _Ach.incrementImmediate(GooglePlayConnection.getAPIclient(), Id, NumSteps);
		return new PendingResultWrapper(PR, UpdateAchievementResult.class);
	}

	/**
	 * Asynchronously loads achievement data for the currently signed in player.
	 * @param ForceReload If True, this call will clear any locally cached data and attempt to fetch the latest data from the server. This would commonly be used for something like a user-initiated refresh. Normally, this should be set to False to gain advantages of data caching.
	 */
	public PendingResultWrapper Load(boolean ForceReload)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Achievements.Load: " + ForceReload);
		PendingResult<LoadAchievementsResult> PR = _Ach.load(GooglePlayConnection.getAPIclient(), ForceReload);
		return new PendingResultWrapper(PR, LoadAchievementsResult.class);
	}

	/**
	 *Reveals a hidden achievement to the currently signed in player. If the achievement is already visible, this will have no effect.
	 *Use this function if you don't need to know the status of the operation immediately. For most applications, this will be the preferred function to use, though note that the update may not be sent to the server until the next sync. See RevealImmediate if you need the operation to attempt to communicate to the server immediately or need to have the status code delivered to your application.
	 *@param Id The achievement ID to reveal
	 */
	public void Reveal(String Id)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Achievements.Reveal: " + Id);
		_Ach.reveal(GooglePlayConnection.getAPIclient(), Id);
	}

	/**
	 *Reveals a hidden achievement to the currently signed in player. If the achievement is already visible, this will have no effect.
	 *This function will attempt to update the user's achievement on the server immediately, and will return a PendingResult that can be used to retrieve the result.
	 *@param Id The ID of the achievement to reveal.
	 */
	public PendingResultWrapper RevealImmediate(String Id)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Achievements.RevealImmediate: " + Id);
		PendingResult<UpdateAchievementResult> PR = _Ach.revealImmediate(GooglePlayConnection.getAPIclient(), Id);
		return new PendingResultWrapper(PR, UpdateAchievementResult.class);
	}

	/**
	 *Sets an achievement to have at least the given number of steps completed. Calling this function while the achievement already has more steps than the provided value is a no-op. Once the achievement reaches the maximum number of steps, the achievement will automatically be unlocked, and any further mutation operations will be ignored.
	 *Use this function if you don't need to know the status of the operation immediately. For most applications, this will be the preferred function to use, though note that the update may not be sent to the server until the next sync. See SetStepsImmediate if you need the operation to attempt to communicate to the server immediately or need to have the status code delivered to your application.
	 *@param Id The ID of the achievement to modify.
	 *@param NumSteps The number of steps to set the achievement to. Must be greater than 0.
	 */
	public void SetSteps(String Id, int NumSteps)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Achievements.SetSteps: " + Id + " numsteps=" + NumSteps);
		_Ach.setSteps(GooglePlayConnection.getAPIclient(), Id, NumSteps);
	}

	/**
	 *Sets an achievement to have at least the given number of steps completed. Calling this function while the achievement already has more steps than the provided value is a no-op. Once the achievement reaches the maximum number of steps, the achievement will automatically be unlocked, and any further mutation operations will be ignored.
	 *This function will attempt to update the user's achievement on the server immediately, and will return a PendingResult that can be used to retrieve the result.
	 *@param Id The ID of the achievement to modify.
	 *@param NumSteps The number of steps to set the achievement to. Must be greater than 0.
	 */
	public PendingResultWrapper SetStepsImmediate(String Id, int NumSteps)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Achievements.SetStepsImmediate: " + Id + " numsteps=" + NumSteps);
		PendingResult<UpdateAchievementResult> PR = _Ach.setStepsImmediate(GooglePlayConnection.getAPIclient(), Id, NumSteps);
		return new PendingResultWrapper(PR, UpdateAchievementResult.class);
	}

	/**
	 *Unlocks an achievement for the currently signed in player. If the achievement is hidden this will reveal it to the player.
	 *Use this function if you don't need to know the status of the operation immediately. For most applications, this will be the preferred function to use, though note that the update may not be sent to the server until the next sync. See UnlockImmediate if you need the operation to attempt to communicate to the server immediately or need to have the status code delivered to your application.
	 *@param Id The achievement ID to unlock
	 */
	public void Unlock(String Id)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Achievements.Unlock: " + Id);
		_Ach.unlock(GooglePlayConnection.getAPIclient(), Id);
	}

	/**
	 *Unlocks an achievement for the currently signed in player. If the achievement is hidden this will reveal it to the player.
	 *This function will attempt to update the user's achievement on the server immediately, and will return a PendingResult that can be used to retrieve the result.
	 *@param Id The ID of the achievement to unlock.
	 */
	public PendingResultWrapper UnlockImmediate(String Id)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Achievements.UnlockImmediate: " + Id);
		PendingResult<UpdateAchievementResult> PR = _Ach.unlockImmediate(GooglePlayConnection.getAPIclient(), Id);
		return new PendingResultWrapper(PR, UpdateAchievementResult.class);
	}
}