package flm.b4a.googleplay;

import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.collections.List;

import com.google.android.gms.games.request.GameRequest;

@BA.ShortName("GPlayGameRequest")
public class GameRequestWrapper
{
	protected GameRequest _GR;

	public GameRequestWrapper()
	{
		_GR = null;
	}

	public GameRequestWrapper(GameRequest request)
	{
		_GR = request;
	}

	public boolean IsInitialized()
	{
		return _GR != null;
	}

	public static final int RECIPIENT_STATUS_ACCEPTED = GameRequest.RECIPIENT_STATUS_ACCEPTED;
	public static final int RECIPIENT_STATUS_PENDING = GameRequest.RECIPIENT_STATUS_PENDING;
	public static final int STATUS_ACCEPTED = GameRequest.STATUS_ACCEPTED;
	public static final int STATUS_PENDING = GameRequest.STATUS_PENDING;
	public static final int TYPE_ALL = GameRequest.TYPE_ALL;
	public static final int TYPE_GIFT = GameRequest.TYPE_GIFT;
	public static final int TYPE_WISH = GameRequest.TYPE_WISH;

	/**
	 * Returns the server timestamp (in milliseconds since epoch) at which this request was created.
	 */
	public long getCreationTimestamp()
	{
		return _GR.getCreationTimestamp();
	}

	/**
	 * Retrieves the data associated with the request.
	 */
	public byte[] getData()
	{
		return _GR.getData();
	}

	/**
	 * Returns the server timestamp (in milliseconds since epoch) at which this request will expire.
	 */
	public long getExpirationTimestamp()
	{
		return _GR.getExpirationTimestamp();
	}

	/**
	 * Retrieves the game associated with this request.
	 */
	public GameWrapper getGame()
	{
		return Utils.GameWrapperOrNull(_GR.getGame());
	}

	/**
	 * Retrieves the status of the request for a given recipient.
	 */
	public int GetRecipientStatus(String PlayerId)
	{
		return _GR.getRecipientStatus(PlayerId);
	}

	/**
	 * Retrieves the information about all the players that the request was sent to.
	 */
	public List getRecipients()
	{
		return Utils.FromListToB4AList_Player(_GR.getRecipients());
	}

	/** Retrieves the ID of this request. */
	public String getRequestId()
	{
		return Utils.StringOrEmpty(_GR.getRequestId());
	}

	/** Retrieves the information about the player that sent the request. */
	public PlayerWrapper getSender()
	{
		return Utils.PlayerWrapperOrNull(_GR.getSender());
	}

	/** Retrieves the status of the request as an overall status depending on all recipients. */
	public int getStatus()
	{
		return _GR.getStatus();
	}

	/** Retrieves the type of this request. */
	public int getType()
	{
		return _GR.getType();
	}

	/**
	 * Retrieves whether the request was consumed by a specific recipient.
	 * @param PlayerId The player ID for which the consumed state should be queried.
	 */
	public boolean IsConsumed(String PlayerId)
	{
		return _GR.isConsumed(PlayerId);
	}
}
