package flm.b4a.googleplay;

import android.net.Uri;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.objects.ImageViewWrapper;

import com.google.android.gms.games.Game;

@ShortName("GPlayGame")
@BA.Events(values = {
		"onImageLoaded(URI As String, Drawable As BitmapDrawable)"
})
public class GameWrapper
{
	protected Game _Game;

	public GameWrapper()
	{
		_Game = null;
	}

	public GameWrapper(Game game)
	{
		_Game = game;
	}

	public boolean IsInitialized()
	{
		return _Game != null;
	}

	/**
	 * Indicates whether or not this game supports snapshots.
	 */
	public boolean AreSnapshotsEnabled()
	{
		return _Game.areSnapshotsEnabled(); 
	}

	/**
	 * Retrieves the number of achievements registered for this game.
	 */
	public int getAchievementTotalCount()
	{
		return _Game.getAchievementTotalCount();
	}

	/**
	 * Retrieves the application ID for this game.
	 */
	public String getApplicationId()
	{
		return Utils.StringOrEmpty(_Game.getApplicationId());
	}

	/**
	 * Retrieves the description of this game. 
	 */
	public String getDescription()
	{
		return Utils.StringOrEmpty(_Game.getDescription());		
	}

	/**
	 * Retrieves the name of the developer of this game.
	 */
	public String getDeveloperName()
	{
		return Utils.StringOrEmpty(_Game.getDeveloperName());		
	}

	/**
	 * Retrieves the display name for this game.
	 */
	public String getDisplayName()
	{
		return Utils.StringOrEmpty(_Game.getDisplayName());
	}

	/**
	 *Loads the game's featured (banner) image from Google Play and fills the given ImageView.
	 *Returns the URI or an empty string if the game has no featured image.
	 */
	public String GetFeaturedImage(final BA ba, ImageViewWrapper ImageView)
	{
		Uri uri = _Game.getFeaturedImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("Game.GetFeaturedImage: uri=" + uri);
		Utils.LoadImage(ba, uri, ImageView);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 *Loads the game's featured (banner) image from Google Play and raises an onImageLoaded event when the load is complete.
	 *Returns the URI or an empty string if the game has no featured image.
	 */
	public String GetFeaturedImage2(final BA ba)
	{
		Uri uri = _Game.getFeaturedImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("Game.GetFeaturedImage2: uri=" + uri);
		Utils.LoadImage(ba, uri, null);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 *Loads the game's hi-res image and fills the given ImageView.
	 *Returns the URI or an empty string if the game has no hi-res image.
	 */
	public String GetHiResImage(final BA ba, ImageViewWrapper ImageView)
	{
		Uri uri = _Game.getHiResImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("Game.GetHiResImage: uri=" + uri);
		Utils.LoadImage(ba, uri, ImageView);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 *Loads the game's hi-res image and raises an onImageLoaded event when the load is complete.
	 *Returns the URI or an empty string if the game has no hi-res image.
	 */
	public String GetHiResImage2(final BA ba)
	{
		Uri uri = _Game.getHiResImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("Game.GetHiResImage2: uri=" + uri);
		Utils.LoadImage(ba, uri, null);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 *Loads the game's icon and fills the given ImageView.
	 *Returns the URI or an empty string if the game has no icon.
	 */
	public String GetIconImage(final BA ba, ImageViewWrapper ImageView)
	{
		Uri uri = _Game.getIconImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("Game.GetIconImage: uri=" + uri);
		Utils.LoadImage(ba, uri, ImageView);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 *Loads the game's icon and raises an onImageLoaded event when the load is complete.
	 *Returns the URI or an empty string if the game has no icon.
	 */
	public String GetIconImage2(final BA ba)
	{
		Uri uri = _Game.getIconImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("Game.GetIconImage2: uri=" + uri);
		Utils.LoadImage(ba, uri, null);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 * Gets the number of leaderboards registered for this game.
	 */
	public int getLeaderboardCount()
	{
		return _Game.getLeaderboardCount();
	}

	/**
	 * Retrieves the primary category of the game - this may be empty.
	 */
	public String getPrimaryCategory()
	{
		return Utils.StringOrEmpty(_Game.getPrimaryCategory());
	}

	/**
	 * Retrieves the secondary category of the game - this may be empty.
	 */
	public String getSecondaryCategory()
	{
		return Utils.StringOrEmpty(_Game.getSecondaryCategory());
	}

	/**
	 * Retrieves the theme color for this game as an RGB hex triplet. The theme color is used to configure the appearance of Play Games UIs.
	 */
	public String getThemeColor()
	{
		return Utils.StringOrEmpty(_Game.getThemeColor());
	}

	/**
	 * Indicates whether or not this game is marked as supporting gamepads.
	 */
	public boolean HasGamepadSupport()
	{
		return _Game.hasGamepadSupport();
	}

	/**
	 * Indicates whether or not this game supports real-time multiplayer.
	 */
	public boolean IsRealTimeMultiplayerEnabled()
	{
		return _Game.isRealTimeMultiplayerEnabled();
	}

	/**
	 * Indicates whether or not this game supports turn-based multiplayer.
	 */
	public boolean IsTurnBasedMultiplayerEnabled()
	{
		return _Game.isTurnBasedMultiplayerEnabled();
	}
}
