package flm.b4a.googleplay;

import java.lang.reflect.Field;
import java.util.ArrayList;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GoogleApiAvailability;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.common.api.GoogleApiClient.ConnectionCallbacks;
import com.google.android.gms.common.api.GoogleApiClient.OnConnectionFailedListener;
import com.google.android.gms.common.api.ResultCallback;
import com.google.android.gms.common.api.Status;
import com.google.android.gms.drive.Drive;
import com.google.android.gms.games.Games;
import com.google.android.gms.games.Games.GamesOptions;
import com.google.android.gms.games.GamesActivityResultCodes;
import com.google.android.gms.games.multiplayer.Invitation;
import com.google.android.gms.games.multiplayer.Multiplayer;
import com.google.android.gms.games.multiplayer.realtime.Room;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMatch;
import com.google.android.gms.games.request.GameRequest;
import com.google.android.gms.nearby.Nearby;

import android.app.Activity;
import android.app.Dialog;
import android.content.Intent;
import android.content.IntentSender.SendIntentException;
import android.os.Bundle;
import android.view.Gravity;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ActivityObject;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.IOnActivityResult;
import anywheresoftware.b4a.objects.collections.List;

@ActivityObject
@ShortName("GPlayConnection")
@Events(values = {
		"onClientConnected", "onConnectionSuspended(Cause As Int, CauseMsg As String)",
		"onConnectionFailed(ErrorCode As Int, ErrorMsg As String, Resolvable As Boolean)",
		"onGivingUp(ErrorCode As Int, ErrorMsg As String, ResolveCode As Int, ResolveMsg As String)",
		"onSignInSucceeded", "onSignInFailed(ReasonCode As Int, ReasonMsg As String)",
		"onSignOutComplete(StatusCode As Int)"
})
@Version(37.2F)
@Author("F. Leneuf-Magaud")
@DependsOn(values = { "com.google.android.gms:play-services-games", "com.google.android.gms:play-services-nearby" })
public class GooglePlayConnection implements ConnectionCallbacks, OnConnectionFailedListener
{
	private static GoogleApiClient GAC = null;
	private GoogleApiAvailability Availability = null;

	protected static String _evtPrefix;
	private BA _ba;

	public static final int RESULT_API_UNAVAILABLE = ConnectionResult.API_UNAVAILABLE;
	public static final int RESULT_CANCELED = ConnectionResult.CANCELED;
	public static final int RESULT_DEVELOPER_ERROR = ConnectionResult.DEVELOPER_ERROR;
	public static final int RESULT_INTERNAL_ERROR = ConnectionResult.INTERNAL_ERROR;
	public static final int RESULT_INTERRUPTED = ConnectionResult.INTERRUPTED;
	public static final int RESULT_INVALID_ACCOUNT = ConnectionResult.INVALID_ACCOUNT;
	public static final int RESULT_LICENSE_CHECK_FAILED = ConnectionResult.LICENSE_CHECK_FAILED;
	public static final int RESULT_NETWORK_ERROR = ConnectionResult.NETWORK_ERROR;
	public static final int RESULT_RESOLUTION_REQUIRED = ConnectionResult.RESOLUTION_REQUIRED;
	public static final int RESULT_RESTRICTED_PROFILE = ConnectionResult.RESTRICTED_PROFILE;
	public static final int RESULT_SERVICE_DISABLED = ConnectionResult.SERVICE_DISABLED;
	public static final int RESULT_SERVICE_INVALID = ConnectionResult.SERVICE_INVALID;
	public static final int RESULT_SERVICE_MISSING = ConnectionResult.SERVICE_MISSING;
	public static final int RESULT_SERVICE_MISSING_PERMISSION = ConnectionResult.SERVICE_MISSING_PERMISSION;
	public static final int RESULT_SERVICE_UPDATING = ConnectionResult.SERVICE_UPDATING;
	public static final int RESULT_SERVICE_VERSION_UPDATE_REQUIRED = ConnectionResult.SERVICE_VERSION_UPDATE_REQUIRED;
	public static final int RESULT_SIGN_IN_FAILED = ConnectionResult.SIGN_IN_FAILED;
	public static final int RESULT_SIGN_IN_REQUIRED = ConnectionResult.SIGN_IN_REQUIRED;
	public static final int RESULT_SUCCESS = ConnectionResult.SUCCESS;
	public static final int RESULT_TIMEOUT = ConnectionResult.TIMEOUT;
	static String ErrorCodeToString(int ErrorCode) {
		switch (ErrorCode) {
		case ConnectionResult.API_UNAVAILABLE:
			return "API unavailable";
		case ConnectionResult.CANCELED:
			return "Canceled";
		case ConnectionResult.DEVELOPER_ERROR:
			return "Developer error";
		case ConnectionResult.INTERNAL_ERROR:
			return "Internal error";
		case ConnectionResult.INTERRUPTED:
			return "Interrupted";
		case ConnectionResult.INVALID_ACCOUNT:
			return "Invalid account";
		case ConnectionResult.LICENSE_CHECK_FAILED:
			return "License check failed";
		case ConnectionResult.NETWORK_ERROR:
			return "Network error";
		case ConnectionResult.RESOLUTION_REQUIRED:
			return "Resolution required";
		case ConnectionResult.RESTRICTED_PROFILE:
			return "Restricted profile";
		case ConnectionResult.SERVICE_DISABLED:
			return "Service disabled";
		case ConnectionResult.SERVICE_INVALID:
			return "Service invalid";
		case ConnectionResult.SERVICE_MISSING:
			return "Service missing";
		case ConnectionResult.SERVICE_MISSING_PERMISSION:
			return "Service missing permission";
		case ConnectionResult.SERVICE_UPDATING:
			return "Service updating";
		case ConnectionResult.SERVICE_VERSION_UPDATE_REQUIRED:
			return "Service version update required";
		case ConnectionResult.SIGN_IN_FAILED:
			return "Sign in failed";
		case ConnectionResult.SIGN_IN_REQUIRED:
			return "Sign in required";
		case ConnectionResult.SUCCESS:
			return "Success";
		case ConnectionResult.TIMEOUT:
			return "Timeout";
		default:
			return "Unknown error code";
		}
	}

	public static final byte STATE_UNCONFIGURED = 0;
	public static final byte STATE_DISCONNECTED = 1;
	public static final byte STATE_CONNECTING = 2;
	public static final byte STATE_CONNECTED = 3;
	public static final byte STATE_SIGNED_IN = 4;
	public static final String[] STATE_NAMES = {
			"Unconfigured", "Disconnected", "Connecting", "Connected", "Signed in"
	}; 
	byte _State = STATE_UNCONFIGURED;

	public static final int SUSPD_CAUSE_NETWORK_LOST = CAUSE_NETWORK_LOST; 
	public static final int SUSPD_CAUSE_SERVICE_DISCONNECTED = CAUSE_SERVICE_DISCONNECTED;

	private boolean _AutoSignIn = false;

	public static boolean DEBUG = false;

	@ShortName("GPlayConnectionOptions")
	public static class Options
	{
		/** Whether you want to access the Games API (default = True). This API requires the user to authenticate. */
		public boolean GamesAPI = true;
		/** Whether you want to access the Drive API to load and save games (default = False). This API requires the user to authenticate. */
		public boolean SavedGamesAPI = false;
		/** Whether you want to access the Nearby Connections API to communicate with nearby devices (default = False). This API does not require the user to authenticate. */
		public boolean NearbyConnectionsAPI = false;
		/** Whether a "connecting" popup should be displayed automatically at the start of the sign-in flow (default = True). */
		public boolean ShowConnectionPopup = true;
		/** Specifies the part of the screen at which games service popups (for example, "welcome back" or "achievement unlocked" popups) will be displayed. */
		public int GravityForPopup = Gravity.TOP | Gravity.CENTER_HORIZONTAL;
	}

	/**
	 * Creates an instance of Google API client and specifies the Play services and scopes to use. This function should be called from Activity_Create.
	 * @param EventPrefix Prefix of events.
	 */
	public void Initialize(BA ba, Options Options, String EventPrefix)
	{
		Initialize2(ba, "", Options, EventPrefix);
	}

	/**
	 * Creates an instance of Google API client and specifies the Play services and scopes to use. This function should be called from Activity_Create.
	 * @param AccountName Account name on the device that should be used. If this is empty, the client will use the current default account for Google Play services for this application.
	 * @param EventPrefix Prefix of events.
	 */
	public void Initialize2(BA ba, String AccountName, Options Options, String EventPrefix)
	{
		if (IsInitialized()) {
			throw new RuntimeException("GPlayConnection already initialized.");
		}
		_ba = ba;
		_evtPrefix = EventPrefix.toLowerCase(BA.cul);
		if (DEBUG) BA.Log("Initialize: creating a GoogleAPIClient instance");
		GoogleApiClient.Builder GACBuilder = new GoogleApiClient.Builder(ba.context, this, this);
		if (AccountName.length() > 0)
			GACBuilder.setAccountName(AccountName);
		else
			GACBuilder.useDefaultAccount();
		if (Options.GamesAPI) {
			GamesOptions GamesAPIOptions = GamesOptions.builder()
					.setShowConnectingPopup(Options.ShowConnectionPopup).build();
			GACBuilder.addApi(Games.API, GamesAPIOptions);
			GACBuilder.addScope(Games.SCOPE_GAMES);
		}
		if (Options.SavedGamesAPI) {
			GACBuilder.addApi(Drive.API);
			GACBuilder.addScope(Drive.SCOPE_APPFOLDER);
		}
		if (Options.NearbyConnectionsAPI) {
			GACBuilder.addApi(Nearby.CONNECTIONS_API);
		}
		GACBuilder.setGravityForPopups(Options.GravityForPopup);
		GAC = GACBuilder.build();
		setState(STATE_DISCONNECTED);
		if (DEBUG) BA.Log("Initialize: ready for connection");
	}

	public boolean IsInitialized()
	{
		return _State != STATE_UNCONFIGURED;
	}

	/** Does one of the connected APIs require the user to authenticate? */
	public boolean RequiresAuthentication()
	{
		return (GAC.hasConnectedApi(Games.API) || GAC.hasConnectedApi(Drive.API)); 
	}

	private void setState(byte NewState)
	{
		if (DEBUG) BA.Log("State change: " + getConnectionStateStr() + " -> " + STATE_NAMES[NewState]);
		_State = NewState;
	}

	/** Gets the connection state (byte). */
	public byte getConnectionState()
	{
		return _State;
	}

	/** Gets the connection state (string). */
	public String getConnectionStateStr()
	{
		return STATE_NAMES[_State];
	}

	/** Updates the connection state of the Google API client. */
	private void UpdateState()
	{
		if (!IsInitialized())
			return;
		if (GAC.isConnecting()) {
			if (_State != STATE_CONNECTING)
				setState(STATE_CONNECTING);
		}
		else if (GAC.isConnected()) {
			if (_State != STATE_CONNECTED && _State != STATE_SIGNED_IN)
				setState(STATE_CONNECTED);
		}
		else if (_State != STATE_DISCONNECTED)
			setState(STATE_DISCONNECTED);
	}

	/**
	 *Connects the client to Google Play services in the background. If successful, triggers the onClientConnected event, then signs the user in. Otherwise, triggers onConnectionFailed.
	 *@param FirstAttempt If True, checks whether the Google Play Services are available and up-to-date on the device, then tries to connect. If False, reconnects automatically the client if found disconnected.
	 *Should be called from the Click event of a Sign-in button (FirstAttempt = True) to initiate the connection and from Activity_Resume to re-establish a lost connection (FirstAttempt = False). You can also set FirstAttempt to True in Activity_Resume if you want to try to sign in without waiting for an user action.
	 */
	public void Connect(boolean FirstAttempt)
	{
		UpdateState();
		if (DEBUG) BA.Log("Connect: state = " + getConnectionStateStr());
		switch (_State) {
		case STATE_DISCONNECTED:
			if (_AutoSignIn || FirstAttempt) {
				if (DEBUG) {
					if (FirstAttempt)
						BA.Log("Connect: first attempt");
					else
						BA.Log("Connect: auto sign-in");
				}
				if (FirstAttempt) {
					Availability = GoogleApiAvailability.getInstance();
					int result = Availability.isGooglePlayServicesAvailable(_ba.context);
					if (DEBUG) BA.Log("Connect: isGooglePlayServicesAvailable returned " + result);
					if (result != ConnectionResult.SUCCESS) {
						_ba.raiseEvent(this, _evtPrefix + "_onconnectionfailed",
								new Object[] { result, ErrorCodeToString(result), Availability.isUserResolvableError(result) });
						return;
					}
				}
				_InvitationID = null;
				_Requests = null;
				_TurnBasedMatch = null;
				setState(STATE_CONNECTING);
				GAC.connect();
			}
			else
				if (DEBUG) BA.Log("Connect: no auto sign-in, FirstAttempt is false");
			break;
		case STATE_CONNECTING:
			if (DEBUG) BA.Log("Connect: connection in progress, no action required");
			break;
		case STATE_CONNECTED:
			if (FirstAttempt) {
				if (DEBUG) BA.Log("Connect: already connected, reconnecting");
				_InvitationID = null;
				_Requests = null;
				_TurnBasedMatch = null;
				setState(STATE_CONNECTING);
				GAC.reconnect();
			}
			else
				if (DEBUG) BA.Log("Connect: already connected, no action required");
			break;
		case STATE_SIGNED_IN:
			if (DEBUG) BA.Log("Connect: already connected and signed in");
			break;
		default:
			throw new IllegalStateException("Connect: error \"Unconfigured state\". Did you call Initialize?");
		}
	}

	/**
	 * Closes the current connection to Google Play services and creates a new connection in the background.
	 */
	public void Reconnect()
	{
		if (!IsInitialized())
			throw new IllegalStateException("Reconnect: error \"Unconfigured state\". Did you call Initialize?");
		UpdateState();
		if (DEBUG) BA.Log("Reconnect: state = " + getConnectionStateStr());
		_InvitationID = null;
		_Requests = null;
		_TurnBasedMatch = null;
		if (_State == STATE_DISCONNECTED) {
			if (DEBUG) BA.Log("Reconnect: disconnected, so connecting");
			setState(STATE_CONNECTING);
			GAC.connect();
		}
		else {
			if (DEBUG) BA.Log("Reconnect: reconnecting");
			setState(STATE_CONNECTING);
			GAC.reconnect();
		}
	}

	private String _InvitationID;
	private ArrayList<GameRequest> _Requests;
	private TurnBasedMatchWrapper _TurnBasedMatch;
	private RoomWrapper _Room;

	/**
	 * Returns the invitation ID received through an invitation notification.
	 */ 
	public String getInvitationID()
	{
		if (IsSignedIn())
			return _InvitationID;
		else
			return null;
	}

	/**
	 * Returns the list of GameRequests received when connecting the client.
	 */
	public List getGameRequests()
	{
		if (IsSignedIn())
			return Utils.FromArrayListToB4AList_Request(_Requests);
		else
			return null;
	}

	/**
	 * Returns the TurnBasedMatch object received when connecting the client.
	 */
	public TurnBasedMatchWrapper getTurnBasedMatch()
	{
		if (IsSignedIn())
			return _TurnBasedMatch;
		else
			return null;
	}

	/**
	 * Returns the Room object received when connecting the client.
	 */
	public RoomWrapper getRoom()
	{
		if (IsSignedIn())
			return _Room;
		else
			return null;
	}

	@BA.Hide
	@Override
	public void onConnected(Bundle connectionHint)
	{
		UpdateState();
		if (DEBUG) BA.Log("onConnected: " + getConnectionStateStr());
		_ba.raiseEvent(this, _evtPrefix + "_onclientconnected", new Object[0]);

		if (GAC.hasConnectedApi(Games.API) && connectionHint != null) {
			if (DEBUG) BA.Log("onConnected: connection hint provided. Checking for invite.");
			Invitation inv = connectionHint.getParcelable(Multiplayer.EXTRA_INVITATION);
			if (inv != null && inv.getInvitationId() != null) {
				_InvitationID = inv.getInvitationId();
				if (DEBUG) BA.Log("onConnected: Invitation ID = " + _InvitationID);
			}

			if (DEBUG) BA.Log("onConnected: connection hint provided. Checking for requests.");
			_Requests = Games.Requests.getGameRequestsFromBundle(connectionHint);
			if (DEBUG && !_Requests.isEmpty()) {
				BA.Log("onConnected: connection hint has " + _Requests.size() + " request(s)");
			}

			if (DEBUG) BA.Log("onConnected: connection hint provided. Checking for TurnBasedMatch game.");
			TurnBasedMatch match = connectionHint.getParcelable(Multiplayer.EXTRA_TURN_BASED_MATCH);
			if (match != null) {
				_TurnBasedMatch = new TurnBasedMatchWrapper(match);
				if (DEBUG) BA.Log("onConnected: TurnBasedMatch = " + _TurnBasedMatch);
			}

			if (DEBUG) BA.Log("onConnected: connection hint provided. Checking for Room.");
			Room room = connectionHint.getParcelable(Multiplayer.EXTRA_ROOM);
			if (room != null) {
				_Room = new RoomWrapper(room);
				if (DEBUG) BA.Log("onConnected: Room = " + _Room);
			}
		}

		if (RequiresAuthentication())
			SucceedSignIn();
	}

	/** Returns whether the client is connected. */
	public boolean IsConnected()
	{
		UpdateState();
		return _State == STATE_CONNECTED || _State == STATE_SIGNED_IN;
	}

	private void SucceedSignIn()
	{
		if (DEBUG) BA.Log("SucceedSignIn");
		setState(STATE_SIGNED_IN);
		_AutoSignIn = true;
		_ba.raiseEvent(this, _evtPrefix + "_onsigninsucceeded", new Object[0]);
	}

	/** Returns whether the client is connected and the user is signed in. */
	public boolean IsSignedIn()
	{
		UpdateState();
		return _State == STATE_SIGNED_IN;
	}

	@BA.Hide
	@Override
	public void onConnectionSuspended(int Cause)
	{
		if (DEBUG) BA.Log("onConnectionSuspended: " + Cause);
		UpdateState();
		String CauseMsg;
		if (Cause == CAUSE_NETWORK_LOST)
			CauseMsg = "A peer device connection was lost.";
		else if (Cause == CAUSE_SERVICE_DISCONNECTED)
			CauseMsg = "The service has been killed.";
		else
			CauseMsg = "Unknown error code";
		_ba.raiseEvent(this, _evtPrefix + "_onconnectionsuspended", new Object[] { Cause, CauseMsg });
		if (_State == STATE_DISCONNECTED) {
			if (DEBUG) BA.Log("onConnectionSuspended: already disconnected, so no action taken");
		}
		else {
			if (DEBUG) BA.Log("onConnectionSuspended: still connected -> killing the connection.");
			KillConnection();
			_ba.raiseEvent(this, _evtPrefix + "_onsigninfailed", new Object[] { 0, "Disconnected" });
		}
	}

	/**
	 *Displays a dialog to address the provided ErrorCode. The error has to be user-recoverable.
	 *This dialog displays a localized message about the error and upon user confirmation (by tapping on dialog) will direct them to the Play Store if Google Play services is out of date or missing, or to system settings if Google Play services is disabled on the device.
	 *@param ErrorCode The error code returned by onConnectionFailed.
	 */
	public void ShowErrorDialog(int ErrorCode)
	{
		if (Availability == null)
			Availability = GoogleApiAvailability.getInstance();
		Dialog dlg = Availability.getErrorDialog(_ba.activity, ErrorCode, 9002);
		if (dlg != null)
			dlg.show();
	}

	@BA.Hide
	@Override
	public void onConnectionFailed(ConnectionResult result)
	{
		int ErrorCode = result.getErrorCode();
		if (DEBUG) {
			BA.Log("onConnectionFailed: " + ErrorCodeToString(ErrorCode)); 
			BA.Log(" - code: " + ErrorCode);
			BA.Log(" - resolvable: " + result.hasResolution());
		}
		_ba.raiseEvent(this, _evtPrefix + "_onconnectionfailed",
				new Object[] { ErrorCode, ErrorCodeToString(ErrorCode), result.hasResolution() });
		ResolveConnectionResult(result);
	}

	private boolean _ExpectingResolution = false;
	private IOnActivityResult iOnActivityResult;

	static String ActivityResultCodeToString(int resultCode)
	{
		switch (resultCode) {
		case Activity.RESULT_OK:
			return "RESULT_OK";
		case Activity.RESULT_CANCELED:
			return "RESULT_CANCELED";
		case GamesActivityResultCodes.RESULT_APP_MISCONFIGURED:
			return "RESULT_APP_MISCONFIGURED";
		case GamesActivityResultCodes.RESULT_INVALID_ROOM:
			return "RESULT_INVALID_ROOM";
		case GamesActivityResultCodes.RESULT_LEFT_ROOM:
			return "RESULT_LEFT_ROOM";
		case GamesActivityResultCodes.RESULT_LICENSE_FAILED:
			return "RESULT_LICENSE_FAILED";
		case GamesActivityResultCodes.RESULT_NETWORK_FAILURE:
			return "RESULT_NETWORK_FAILURE";
		case GamesActivityResultCodes.RESULT_RECONNECT_REQUIRED:
			return "RESULT_RECONNECT_REQUIRED";
		case GamesActivityResultCodes.RESULT_SEND_REQUEST_FAILED:
			return "SEND_REQUEST_FAILED";
		case GamesActivityResultCodes.RESULT_SIGN_IN_FAILED:
			return "SIGN_IN_FAILED";
		default:
			return String.valueOf(resultCode);
		}
	}

	private void ResolveConnectionResult(final ConnectionResult result)
	{
		if (_ExpectingResolution) {
			if (DEBUG) BA.Log("ResolveConnectionResult: already expecting the result of a previous resolution.");
			return;
		}

		if (result.hasResolution()) {
			if (DEBUG) BA.Log("ResolveConnectionResult: result has resolution. Starting it.");

			iOnActivityResult = new IOnActivityResult() {
				@Override
				public void ResultArrived(int resultCode, Intent intent) {
					if (DEBUG) BA.Log("OnActivityResult: " + ActivityResultCodeToString(resultCode));

					_ExpectingResolution = false;

//					if (_State != STATE_CONNECTING) {
//						if (DEBUG) BA.Log("OnActivityResult: ignoring because state isn't STATE_CONNECTING (it's " + getConnectionStateStr() + ")");
//						return;
//					}

					if (resultCode == Activity.RESULT_OK) {
						if (DEBUG) BA.Log("OnActivityResult: resolution was RESULT_OK, so connecting client again.");
						GAC.connect();
					}
					else if (resultCode == GamesActivityResultCodes.RESULT_RECONNECT_REQUIRED) {
						if (DEBUG) BA.Log("OnActivityResult: resolution was RECONNECT_REQUIRED, so reconnecting.");
						GAC.connect();
					}
					else if (resultCode == Activity.RESULT_CANCELED) {
						if (DEBUG) BA.Log("OnActivityResult: got a cancellation result, so disconnecting.");
						_AutoSignIn = false;
						KillConnection();
						_ba.raiseEvent(this, _evtPrefix + "_onsigninfailed",
								new Object[] { resultCode, ActivityResultCodeToString(resultCode) });
					}
					else {
						if (DEBUG) BA.Log("OnActivityResult: not resolved, so giving up.");
						GiveUp(result.getErrorCode(), resultCode);
					}
				}
			};

			BA pBA = _ba.processBA;
			if (pBA == null) pBA = _ba;
			try {
				pBA.startActivityForResult(iOnActivityResult, null);
			} catch (NullPointerException npe) {
				//a NPE is expected
			}
			BA.SharedProcessBA sba = pBA.sharedProcessBA;
			int requestCode;
			try {
				Field f = BA.SharedProcessBA.class.getDeclaredField("onActivityResultCode");
				f.setAccessible(true);
				requestCode = f.getInt(sba) - 1;
			} catch (Exception e) {
				throw new RuntimeException(e);
			}

			try {
				_ExpectingResolution = true;
				result.startResolutionForResult(_ba.activity, requestCode);
			} catch (SendIntentException e) {
				if (DEBUG) BA.Log("ResolveConnectionResult: SendIntentException, so connecting again.");
				GAC.connect();
			}
		}
		else {
			if (DEBUG) BA.Log("ResolveConnectionResult: result has no resolution. Giving up.");
			GiveUp(result.getErrorCode(), 0);
		}
	}

	private void GiveUp(int ErrorCode, int ResolveCode)
	{
		String ResolveStr = "";
		if (ResolveCode == 0)
			ResolveStr = ErrorCodeToString(ErrorCode) + ", not resolved";
		else
			ResolveStr = ActivityResultCodeToString(ResolveCode);
		_ba.raiseEvent(this, _evtPrefix + "_ongivingup",
				new Object[] { ErrorCode, ErrorCodeToString(ErrorCode), ResolveCode, ResolveStr });

		_AutoSignIn = false;
		KillConnection();
		_ba.raiseEvent(this, _evtPrefix + "_onsigninfailed", new Object[] { ResolveCode, ResolveStr });
	}

	private void KillConnection()
	{
		if (DEBUG) BA.Log("KillConnection: killing connection.");
		UpdateState();
		if (_State != STATE_DISCONNECTED) {
			GAC.disconnect();
			setState(STATE_DISCONNECTED);
			_InvitationID = null;
			_Requests = null;
			_TurnBasedMatch = null;
		}
		if (DEBUG) BA.Log("KillConnection: Google API client disconnected.");
	}

	/**
	 * Disconnects the Google API client. Should be called from Activity_Pause or from the Click event of a Sign-out button. 
	 * @param SignOut Whether the user is signed out before disconnecting. 
	 */
	public void Disconnect(boolean SignOut)
	{
		UpdateState();
		if (_State == STATE_DISCONNECTED) {
			if (DEBUG) BA.Log("Disconnect: already disconnected, ignoring.");
			return;
		}

		if (SignOut) {
			if (DEBUG) BA.Log("Disconnect: signing out from the Google API client.");
			SignOut();
		}

		if (DEBUG) BA.Log("Disconnect: proceeding with disconnection.");
		KillConnection();
	}

	private class SignOutEvent implements ResultCallback<Status>
	{
		@Override
		public void onResult(Status result)
		{
			if (GooglePlayConnection.DEBUG) BA.Log("SignOut: complete");
			if (GAC.isConnected())
				setState(STATE_CONNECTED);
			else
				UpdateState();
			_ba.raiseEvent2(this, true, _evtPrefix + "_onsignoutcomplete", false,
					new Object[] { result.getStatusCode() });
		}
	}

	/**
	 *Asynchronously signs the current user out and raises an onSignOutComplete event when done.
	 *This call doesn't disconnect the Google API Client. As no user is signed in after this call is completed, subsequent calls to this client will very likely fail. You should either call Disconnect or Activity.Finish after this call.
	 */
	public void SignOut()
	{
		if (DEBUG) BA.Log("SignOut");
		ResultCallback<Status> RC = new SignOutEvent();
		Games.signOut(GAC).setResultCallback(RC);
	}

	protected static GoogleApiClient getAPIclient()
	{
		return GAC;
	}
}
