package flm.b4a.googleplay;

import com.google.android.gms.games.multiplayer.Invitation;
import com.google.android.gms.games.multiplayer.realtime.Room;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMatch;

import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.objects.collections.List;

@ShortName("GPlayInvitation")
public class InvitationWrapper
{
	protected Invitation _Invitation;

	public InvitationWrapper()
	{
		_Invitation = null;
	}

	public InvitationWrapper(Invitation inv)
	{
		_Invitation = inv;
	}

	public boolean IsInitialized()
	{
		return _Invitation != null;
	}

	public final static int INVITATION_TYPE_REAL_TIME = Invitation.INVITATION_TYPE_REAL_TIME;
	public final static int INVITATION_TYPE_TURN_BASED = Invitation.INVITATION_TYPE_TURN_BASED;
	public final static int MATCH_VARIANT_DEFAULT = TurnBasedMatch.MATCH_VARIANT_DEFAULT;
	public final static int ROOM_VARIANT_DEFAULT = Room.ROOM_VARIANT_DEFAULT;

	/**
	 * Returns the maximum number of available automatch slots for this invitation. If automatch criteria were not specified during creation, or if all slots have been filled, this will return 0.
	 */
	public int getAvailableAutoMatchSlots()
	{
		return _Invitation.getAvailableAutoMatchSlots();
	}

	/**
	 * Retrieves the server timestamp at which this invitation was created.
	 */
	public long getCreationTimestamp()
	{
		return _Invitation.getCreationTimestamp();
	}

	/**
	 * Retrieves the Game object that this invitation is associated with.
	 */
	public GameWrapper getGame()
	{
		return Utils.GameWrapperOrNull(_Invitation.getGame());
	}

	/**
	 * Retrieves the ID of this invitation.
	 */
	public String getInvitationId()
	{
		return Utils.StringOrEmpty(_Invitation.getInvitationId());
	}

	/**
	 * Retrieves the type of this invitation. May be either INVITATION_TYPE_REAL_TIME or INVITATION_TYPE_TURN_BASED.
	 */
	public int getInvitationType()
	{
		return _Invitation.getInvitationType();
	}

	/**
	 * Returns the list of participants for this invitation.
	 */
	public List getParticipants()
	{
		return Utils.FromArrayListToB4AList_Participant(_Invitation.getParticipants());
	}

	/**
	 * Retrieves the participant who created this invitation.
	 */
	public ParticipantWrapper getInviter()
	{
		return Utils.ParticipantWrapperOrNull(_Invitation.getInviter());
	}

	/**
	 * Retrieves the variant specified for this invitation, if any. A variant is an optional developer-controlled parameter describing the type of game to play. If specified, this value will be a positive integer. If this invitation had no variant specified, returns ROOM_VARIANT_DEFAULT or MATCH_VARIANT_DEFAULT.
	 */
	public int getVariant()
	{
		return _Invitation.getVariant();
	}
}
