package flm.b4a.googleplay;

import android.content.Intent;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ShortName;

import com.google.android.gms.common.api.PendingResult;
import com.google.android.gms.games.Games;
import com.google.android.gms.games.multiplayer.Invitation;
import com.google.android.gms.games.multiplayer.Invitations;
import com.google.android.gms.games.multiplayer.Invitations.LoadInvitationsResult;
import com.google.android.gms.games.multiplayer.Multiplayer;
import com.google.android.gms.games.multiplayer.OnInvitationReceivedListener;

@ShortName("GPlayInvitations")
@BA.Events(values = {
		"onInvitationReceived(Invitation As GPlayInvitation)",
		"onInvitationRemoved(InvitationID As String)"
})
public class InvitationsWrapper
{
	protected Invitations _Inv = null;

	public void Initialize()
	{
		_Inv = Games.Invitations;
	}

	public boolean IsInitialized()
	{
		return (_Inv != null);
	}

	public final static String EXTRA_INVITATION = Multiplayer.EXTRA_INVITATION;
	public final static String RESULTMAP_INVITATIONS = PendingResultWrapper.RESULTMAP_INVITATIONS;
	public final static String RESULTMAP_STATUS_CODE = PendingResultWrapper.RESULTMAP_STATUS_CODE;
	public final static int SORT_ORDER_MOST_RECENT_FIRST = Multiplayer.SORT_ORDER_MOST_RECENT_FIRST;
	public final static int SORT_ORDER_SOCIAL_AGGREGATION = Multiplayer.SORT_ORDER_SOCIAL_AGGREGATION;

	/**
	 *Returns an intent that will let the user see and manage any outstanding invitations. This intent must be invoked with StartActivityForResult.
	 *If the user canceled, the result code will be ACTIVITY_RESULT_CANCELED. If the user selected an invitation to accept, the result code will be ACTIVITY_RESULT_OK and the data intent will contain the selected invitation in the extras. Based on the type of the match (TBMP/RTMP), the result will include either EXTRA_TURN_BASED_MATCH or EXTRA_INVITATION.
	 */
	public Intent GetInvitationInboxIntent()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Invitations.GetInvitationInboxIntent");
		return _Inv.getInvitationInboxIntent(GooglePlayConnection.getAPIclient());
	}

	/**
	 * Asynchronously loads the list of invitations for the current game. Invitations are returned sorted by most recent first.
	 */
	public PendingResultWrapper LoadInvitations()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Invitations.LoadInvitations");
		PendingResult<LoadInvitationsResult> PR = _Inv.loadInvitations(GooglePlayConnection.getAPIclient());
		return new PendingResultWrapper(PR, LoadInvitationsResult.class);
	}

	/**
	 * Asynchronously loads the list of invitations for the current game.
	 * @param SortOrder How to sort the returned invitations. Must be either SORT_ORDER_MOST_RECENT_FIRST or SORT_ORDER_SOCIAL_AGGREGATION.
	 */
	public PendingResultWrapper LoadInvitations2(int SortOrder)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Invitations.LoadInvitations2");
		PendingResult<LoadInvitationsResult> PR = _Inv.loadInvitations(GooglePlayConnection.getAPIclient(), SortOrder);
		return new PendingResultWrapper(PR, LoadInvitationsResult.class);
	}

	private OnInvitationReceivedListener _onInvitationReceivedListener = null;
	private class InvitationListener implements OnInvitationReceivedListener
	{
		private BA _ba;
		private String _evtPrefix;

		public InvitationListener(BA ba)
		{
			_ba = ba;
			_evtPrefix = GooglePlayConnection._evtPrefix; 
		}

		@Override
		public void onInvitationReceived(Invitation invit) {
			if (GooglePlayConnection.DEBUG) BA.Log("Invitations.onInvitationReceived: " + invit);
			_ba.raiseEvent2(this, true, _evtPrefix + "_oninvitationreceived",
					false, new Object[] { Utils.InvitationWrapperOrNull(invit) });
		}

		@Override
		public void onInvitationRemoved(String invitID) {
			if (GooglePlayConnection.DEBUG) BA.Log("Invitations.onInvitationRemoved: " + invitID);
			_ba.raiseEvent2(this, true, _evtPrefix + "_oninvitationremoved",
					false, new Object[] { Utils.StringOrEmpty(invitID) });
		}
	}

	/**
	 *Registers a listener to intercept incoming invitations for the currently signed-in user. If a listener is registered by this function, the incoming invitation will not generate a status bar notification as long as this client remains connected.
	 *Note that only one invitation listener may be active at a time. Calling this function while another invitation listener was previously registered will replace the original listener with the new one.
	 *The listener will raise the onInvitationReceived and onInvitationRemoved events.
	 */
	public void RegisterInvitationListener(BA ba)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Invitations.RegisterInvitationListener");
		if (_onInvitationReceivedListener == null) {
			_onInvitationReceivedListener = new InvitationListener(ba);
		}
		_Inv.registerInvitationListener(GooglePlayConnection.getAPIclient(), _onInvitationReceivedListener);
	}

	/**
	 * Unregisters this client's invitation listener, if any. Any new invitations will generate status bar notifications as normal.
	 */
	public void UnregisterInvitationListener()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Invitations.UnregisterInvitationListener");
		_Inv.unregisterInvitationListener(GooglePlayConnection.getAPIclient());
	}
}
