package flm.b4a.googleplay;

import android.net.Uri;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.ImageViewWrapper;

import com.google.android.gms.games.leaderboard.LeaderboardScore;

@BA.ShortName("GPlayLeaderboardScore")
@BA.Events(values = {
		"onImageLoaded(URI As String, Drawable As BitmapDrawable)"
})
public class LeaderboardScoreWrapper
{
	protected LeaderboardScore _LbS = null;

	public LeaderboardScoreWrapper(LeaderboardScore score)
	{
		_LbS = score;
	}

	public boolean IsInitialized()
	{
		return _LbS != null;
	}

	public static final int LEADERBOARD_RANK_UNKNOWN = LeaderboardScore.LEADERBOARD_RANK_UNKNOWN;

	/**
	 * Retrieves a formatted string to display for this rank. This handles appropriate localization and formatting.
	 */
	public String getDisplayRank()
	{
		return Utils.StringOrEmpty(_LbS.getDisplayRank());
	}

	/**
	 * Retrieves a formatted string to display for this score. The details of the formatting are specified by the developer in their dev console.
	 */
	public String getDisplayScore()
	{
		return Utils.StringOrEmpty(_LbS.getDisplayScore());
	}

	/**
	 *Retrieves the rank returned from the server for this score. Note that this may not be exact and that multiple scores can have identical ranks. Lower ranks indicate a better score, with rank 1 being the best score on the board.
	 *If the score holder's rank cannot be determined, this will return LEADERBOARD_RANK_UNKNOWN.
	 */
	public long getRank()
	{
		return _LbS.getRank();
	}

	/**
	 * Retrieves the raw score value.
	 */
	public long getRawScore()
	{
		return _LbS.getRawScore();
	}

	/**
	 * Retrieves the player that scored this particular score. The return value here may be Null if the current player is not authorized to see information about the holder of this score.
	 */
	public PlayerWrapper getScoreHolder()
	{
		return Utils.PlayerWrapperOrNull(_LbS.getScoreHolder());
	}

	/**
	 * Retrieves the name to display for the player who scored this score. If the identity of the player is unknown, this will return an anonymous name to display.
	 */
	public String getScoreHolderDisplayName()
	{
		return Utils.StringOrEmpty(_LbS.getScoreHolderDisplayName());
	}

	/**
	 *Loads the hi-res image to display for the player who scored this score and fills the given ImageView.
	 *Returns the URI of the image. If the identity of the player is unknown, this will be an empty string. It may also be an empty string if the player simply has no image.
	 */
	public String GetScoreHolderHiResImage(final BA ba, ImageViewWrapper ImageView)
	{
		Uri uri = _LbS.getScoreHolderHiResImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("GetScoreHolderHiResImage: uri=" + uri);
		Utils.LoadImage(ba, uri, ImageView);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 *Loads the hi-res image to display for the player who scored this score and raises an onImageLoaded event when the load is complete.
	 *Returns the URI of the image. If the identity of the player is unknown, this will be an empty string. It may also be an empty string if the player simply has no image.
	 */
	public String GetScoreHolderHiResImage2(final BA ba)
	{
		Uri uri = _LbS.getScoreHolderHiResImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("GetScoreHolderHiResImage2: uri=" + uri);
		Utils.LoadImage(ba, uri, null);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 *Loads the icon image to display for the player who scored this score and fills the given ImageView. If the identity of the player is unknown, this will load an anonymous image for the player.
	 *Returns the URI of the image or an empty string if the player simply has no image.
	 */
	public String GetScoreHolderIconImage(final BA ba, ImageViewWrapper ImageView)
	{
		Uri uri = _LbS.getScoreHolderIconImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("GetScoreHolderIconImage: uri=" + uri);
		Utils.LoadImage(ba, uri, ImageView);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 *Loads the icon image to display for the player who scored this score and raises an onImageLoaded event when the load is complete. If the identity of the player is unknown, this will load an anonymous image for the player.
	 *Returns the URI of the image or an empty string if the player simply has no image.
	 */
	public String GetScoreHolderIconImage2(final BA ba)
	{
		Uri uri = _LbS.getScoreHolderIconImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("GetScoreHolderIconImage2: uri=" + uri);
		Utils.LoadImage(ba, uri, null);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 * Retrieves the optional score tag associated with this score, if any.
	 */
	public String getScoreTag()
	{
		return Utils.StringOrEmpty(_LbS.getScoreTag());
	}

	/**
	 * Retrieves the timestamp (in milliseconds from epoch) at which this score was achieved.
	 */
	public long getTimestampMillis()
	{
		return _LbS.getTimestampMillis();
	}
}
