package flm.b4a.googleplay;

import anywheresoftware.b4a.BA;

import com.google.android.gms.games.leaderboard.LeaderboardVariant;

@BA.ShortName("GPlayLeaderboardVariant")
public class LeaderboardVariantWrapper
{
	protected LeaderboardVariant _LbV;

	public LeaderboardVariantWrapper()
	{
		_LbV = null;
	}

	public LeaderboardVariantWrapper(LeaderboardVariant variant)
	{
		_LbV = variant;
	}

	public boolean IsInitialized()
	{
		return _LbV != null;
	}

	public final static int COLLECTION_PUBLIC = LeaderboardVariant.COLLECTION_PUBLIC;
	public final static int COLLECTION_SOCIAL = LeaderboardVariant.COLLECTION_SOCIAL;
	public final static int NUM_SCORES_UNKNOWN = LeaderboardVariant.NUM_SCORES_UNKNOWN;
	public final static int NUM_TIME_SPANS = LeaderboardVariant.NUM_TIME_SPANS;
	public final static int PLAYER_RANK_UNKNOWN = LeaderboardVariant.PLAYER_RANK_UNKNOWN;
	public final static int PLAYER_SCORE_UNKNOWN = LeaderboardVariant.PLAYER_SCORE_UNKNOWN;
	public final static int TIME_SPAN_ALL_TIME = LeaderboardVariant.TIME_SPAN_ALL_TIME;
	public final static int TIME_SPAN_DAILY = LeaderboardVariant.TIME_SPAN_DAILY;
	public final static int TIME_SPAN_WEEKLY = LeaderboardVariant.TIME_SPAN_WEEKLY;

	/**
	 * Retrieves the collection of scores contained by this variant. Possible values are COLLECTION_PUBLIC or COLLECTION_SOCIAL.
	 */
	public int getCollection()
	{
		return _LbV.getCollection();		
	}

	/**
	 * Retrieves the viewing player's formatted rank for this variant, if any. Note that this value is only accurate if HasPlayerInfo returns True.
	 */
	public String getDisplayPlayerRank()
	{
		return Utils.StringOrEmpty(_LbV.getDisplayPlayerRank());
	}

	/**
	 * Retrieves the viewing player's score for this variant, if any. Note that this value is only accurate if HasPlayerInfo returns True.
	 */
	public String getDisplayPlayerScore()
	{
		return Utils.StringOrEmpty(_LbV.getDisplayPlayerScore());
	}

	/**
	 * Retrieves the total number of scores for this variant. Not all of these scores will always be present on the local device. Note that if scores for this variant have not been loaded, this function will return NUM_SCORES_UNKNOWN.
	 */
	public long getNumScores()
	{
		return _LbV.getNumScores();		
	}

	/**
	 *Retrieves the viewing player's rank for this variant, if any. Note that this value is only accurate if HasPlayerInfo returns True.
	 *This function returns PLAYER_RANK_UNKNOWN if the player has no rank for this variant.
	 */
	public long getPlayerRank()
	{
		return _LbV.getPlayerRank();		
	}

	/**
	 * Retrieves the viewing player's score tag for this variant, if any. Note that this value is only accurate if HasPlayerInfo returns True.
	 */
	public String getPlayerScoreTag()
	{
		return Utils.StringOrEmpty(_LbV.getPlayerScoreTag());
	}

	/**
	 * Retrieves the viewing player's score for this variant, if any. Note that this value is only accurate if HasPlayerInfo returns True.
	 */
	public long getRawPlayerScore()
	{
		return _LbV.getRawPlayerScore();		
	}

	/**
	 * Retrieves the time span that the scores for this variant are drawn from. Possible values are TIME_SPAN_ALL_TIME, TIME_SPAN_WEEKLY, or TIME_SPAN_DAILY.
	 */
	public int getTimeSpan()
	{
		return _LbV.getTimeSpan();
	}

	/**
	 *Gets whether or not this variant contains score information for the viewing player. There are several possible reasons why this might be False. If the scores for this variant have never been loaded, we won't know if the player has a score or not. Similarly, if the player has not submitted a score for this variant, this will return False.
	 *It is possible to have a score but no rank. For instance, on leaderboard variants of COLLECTION_PUBLIC, players who are not sharing their scores publicly will never have a rank.
	 */
	public boolean HasPlayerInfo()
	{
		return _LbV.hasPlayerInfo();
	}
}
