package flm.b4a.googleplay;

import android.net.Uri;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.ImageViewWrapper;
import anywheresoftware.b4a.objects.collections.List;

import com.google.android.gms.games.leaderboard.Leaderboard;

@BA.ShortName("GPlayLeaderboard")
@BA.Events(values = {
		"onImageLoaded(URI As String, Drawable As BitmapDrawable)"
})
public class LeaderboardWrapper
{
	protected Leaderboard _Lb;

	public LeaderboardWrapper()
	{
		_Lb = null;
	}

	public LeaderboardWrapper(Leaderboard lb)
	{
		_Lb = lb;
	}

	public boolean IsInitialized()
	{
		return _Lb != null;
	}

	public static final int SCORE_ORDER_LARGER_IS_BETTER = Leaderboard.SCORE_ORDER_LARGER_IS_BETTER;
	public static final int SCORE_ORDER_SMALLER_IS_BETTER = Leaderboard.SCORE_ORDER_SMALLER_IS_BETTER;

	/**
	 * Retrieves the display name of this leaderboard.
	 */
	public String getDisplayName()
	{
		return Utils.StringOrEmpty(_Lb.getDisplayName());
	}

	/**
	 *Loads this leaderboard's icon and fills the given ImageView.
	 *Returns the URI of the image, or an empty string if there was a problem retrieving the icon.
	 */
	public String GetIconImage(final BA ba, ImageViewWrapper ImageView)
	{
		Uri uri = _Lb.getIconImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("Leaderboard.GetIconImage: uri=" + uri);
		Utils.LoadImage(ba, uri, ImageView);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 *Loads this leaderboard's icon and raises an onImageLoaded event when the load is complete.
	 *Returns the URI of the image, or an empty string if there was a problem retrieving the icon.
	 */
	public String GetIconImage2(final BA ba)
	{
		Uri uri = _Lb.getIconImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("Leaderboard.GetIconImage2: uri=" + uri);
		Utils.LoadImage(ba, uri, null);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 * Retrieves the ID of this leaderboard.
	 */
	public String getLeaderboardId()
	{
		return Utils.StringOrEmpty(_Lb.getLeaderboardId());
	}

	/**
	 * Retrieves the sort order of scores for this leaderboard. Possible values are SCORE_ORDER_LARGER_IS_BETTER or SCORE_ORDER_SMALLER_IS_BETTER.
	 */
	public int getScoreOrder()
	{
		return _Lb.getScoreOrder();
	}

	/**
	 * Retrieves the variants (GPlayLeaderboardVariant) for this leaderboard. These will be returned sorted by time span first, then by variant type.
	 */
	public List getVariants()
	{
		return Utils.FromArrayListToB4AList_LeaderboardVariant(_Lb.getVariants());
	}
}
