package flm.b4a.googleplay;

import com.google.android.gms.common.api.PendingResult;
import com.google.android.gms.games.Games;
import com.google.android.gms.games.GamesStatusCodes;
import com.google.android.gms.games.PageDirection;
import com.google.android.gms.games.leaderboard.LeaderboardScoreBuffer;
import com.google.android.gms.games.leaderboard.LeaderboardVariant;
import com.google.android.gms.games.leaderboard.Leaderboards;
import com.google.android.gms.games.leaderboard.Leaderboards.LeaderboardMetadataResult;
import com.google.android.gms.games.leaderboard.Leaderboards.LoadPlayerScoreResult;
import com.google.android.gms.games.leaderboard.Leaderboards.LoadScoresResult;
import com.google.android.gms.games.leaderboard.Leaderboards.SubmitScoreResult;

import android.content.Intent;
import anywheresoftware.b4a.BA;

@BA.ShortName("GPlayLeaderboards")
public class LeaderboardsWrapper
{
	protected Leaderboards _Lb;

	public LeaderboardsWrapper()
	{
		_Lb = null;
	}

	public void Initialize()
	{
		_Lb = Games.Leaderboards;
	}

	public boolean IsInitialized()
	{
		return _Lb != null;
	}

	public final static int COLLECTION_PUBLIC = LeaderboardVariant.COLLECTION_PUBLIC;
	public final static int COLLECTION_SOCIAL = LeaderboardVariant.COLLECTION_SOCIAL;
	public final static int PAGE_DIRECTION_NEXT = PageDirection.NEXT;
	public final static int PAGE_DIRECTION_PREV = PageDirection.PREV;
	public static final String RESULTMAP_LEADERBOARD = PendingResultWrapper.RESULTMAP_LEADERBOARD;
	public final static String RESULTMAP_LEADERBOARDS = PendingResultWrapper.RESULTMAP_LEADERBOARDS;
	public final static String RESULTMAP_PLAYER_SCORE = PendingResultWrapper.RESULTMAP_PLAYER_SCORE;
	public static final String RESULTMAP_SCORE_BUFFER = PendingResultWrapper.RESULTMAP_SCORE_BUFFER;
	public static final String RESULTMAP_SCORE_DATA = PendingResultWrapper.RESULTMAP_SCORE_DATA;
	public static final String RESULTMAP_SCORES = PendingResultWrapper.RESULTMAP_SCORES;
	public final static String RESULTMAP_STATUS_CODE = PendingResultWrapper.RESULTMAP_STATUS_CODE;
	public final static int STATUS_OK = GamesStatusCodes.STATUS_OK;
	public final static int TIME_SPAN_ALL_TIME = LeaderboardVariant.TIME_SPAN_ALL_TIME;
	public final static int TIME_SPAN_DAILY = LeaderboardVariant.TIME_SPAN_DAILY;
	public final static int TIME_SPAN_WEEKLY = LeaderboardVariant.TIME_SPAN_WEEKLY;

	/**
	 *Gets an intent to show the list of leaderboards for a game. This intent must be invoked with StartActivityForResult.
	 *A RESULT_RECONNECT_REQUIRED may be returned as the result code in the onActivityResult event if the Google API client ends up in an inconsistent state.
	 */
	public Intent GetAllLeaderboardsIntent()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Leaderboards.GetAllLeaderboardsIntent");
		return _Lb.getAllLeaderboardsIntent(GooglePlayConnection.getAPIclient());
	}

	/**
	 *Gets an intent to show a leaderboard for a game. This intent must be invoked with StartActivityForResult.
	 *A RESULT_RECONNECT_REQUIRED may be returned as the result code in the onActivityResult event if the Google API client ends up in an inconsistent state.
	 * @param LeaderboardId The ID of the leaderboard to view.
	 */
	public Intent GetLeaderboardIntent(String LeaderboardId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Leaderboards.GetLeaderboardIntent: " + LeaderboardId);
		return _Lb.getLeaderboardIntent(GooglePlayConnection.getAPIclient(), LeaderboardId);
	}

	/**
	 * Asynchronously loads the currently signed in player's score for a given leaderboard.
	 * @param LeaderboardId ID of the leaderboard to load the score from.
	 * @param Span Time span to retrieve data for. Valid values are TIME_SPAN_DAILY, TIME_SPAN_WEEKLY, or TIME_SPAN_ALL_TIME.
	 * @param LeaderboardCollection	The leaderboard collection to retrieve scores for. Valid values are either COLLECTION_PUBLIC or COLLECTION_SOCIAL.
	 */
	public PendingResultWrapper LoadCurrentPlayerLeaderboardScore(String LeaderboardId, int Span, int LeaderboardCollection)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Leaderboards.LoadCurrentPlayerLeaderboardScore: " + LeaderboardId + " span=" + Span + " collection=" + LeaderboardCollection);
		PendingResult<LoadPlayerScoreResult> PR = _Lb.loadCurrentPlayerLeaderboardScore(GooglePlayConnection.getAPIclient(), LeaderboardId, Span, LeaderboardCollection);
		return new PendingResultWrapper(PR, LoadPlayerScoreResult.class);
	}

	/**
	 * Asynchronously loads the list of leaderboard metadata for this game.
	 * @param ForceReload If True, this call will clear any locally cached data and attempt to fetch the latest data from the server. This would commonly be used for something like a user-initiated refresh. Normally, this should be set to False to gain advantages of data caching.
	 */
	public PendingResultWrapper LoadLeaderboardMetadata(boolean ForceReload)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Leaderboards.LoadLeaderboardMetadata: forcereload=" + ForceReload);
		PendingResult<LeaderboardMetadataResult> PR = _Lb.loadLeaderboardMetadata(GooglePlayConnection.getAPIclient(), ForceReload);
		return new PendingResultWrapper(PR, LeaderboardMetadataResult.class);
	}

	/**
	 * Asynchronously loads a specific leaderboard's metadata for this game.
	 * @param LeaderboardId ID of the leaderboard to load metadata for.
	 * @param ForceReload If True, this call will clear any locally cached data and attempt to fetch the latest data from the server. This would commonly be used for something like a user-initiated refresh. Normally, this should be set to False to gain advantages of data caching.
	 */
	public PendingResultWrapper LoadLeaderboardMetadata2(String LeaderboardId, boolean ForceReload)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Leaderboards.LoadLeaderboardMetadata2: id=" + LeaderboardId + " forcereload=" + ForceReload);
		PendingResult<LeaderboardMetadataResult> PR = _Lb.loadLeaderboardMetadata(GooglePlayConnection.getAPIclient(), LeaderboardId, ForceReload);
		return new PendingResultWrapper(PR, LeaderboardMetadataResult.class);
	}

	/**
	 *Asynchronously loads an additional page of score data for the given score buffer.
	 *@param ScoreBuffer A score buffer delivered by a call to LoadPlayerCenteredScores or LoadTopScores in the result map.
	 *@param MaxResults The maximum number of scores to fetch per page. Must be between 1 and 25. Note that the number of scores returned here may be greater than this value, depending on how much data is cached on the device.
	 *@param PageDirection The direction to expand the list (one of the PAGE_DIRECTION constants).
	 */
	public PendingResultWrapper LoadMoreScores(Object ScoreBuffer, int MaxResults, int PageDirection)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Leaderboards.LoadMoreScores: " + ScoreBuffer.getClass().getName() + " maxresults=" + MaxResults + " pagedir=" + PageDirection);
		if (ScoreBuffer == null)
			throw new NullPointerException("No page of score data was loaded previously.");
		PendingResult<LoadScoresResult> PR = _Lb.loadMoreScores(GooglePlayConnection.getAPIclient(), (LeaderboardScoreBuffer) ScoreBuffer, MaxResults, PageDirection);
		return new PendingResultWrapper(PR, LoadScoresResult.class);
	}

	/**
	 *Asynchronously loads the player-centered page of scores for a given leaderboard. If the player does not have a score on this leaderboard, this call will return the top page instead.
	 *@param LeaderboardId ID of the leaderboard.
	 *@param Span Time span to retrieve data for. Valid values are TIME_SPAN_DAILY, TIME_SPAN_WEEKLY, or TIME_SPAN_ALL_TIME.
	 *@param LeaderboardCollection The leaderboard collection to retrieve scores for. Valid values are either COLLECTION_PUBLIC or COLLECTION_SOCIAL.
	 *@param MaxResults The maximum number of scores to fetch per page. Must be between 1 and 25.
	 */
	public PendingResultWrapper LoadPlayerCenteredScores(String LeaderboardId, int Span, int LeaderboardCollection, int MaxResults)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Leaderboards.LoadPlayerCenteredScores: id=" + LeaderboardId + " span=" + Span + " collection=" + LeaderboardCollection + " maxresults=" + MaxResults);
		PendingResult<LoadScoresResult> PR = _Lb.loadPlayerCenteredScores(GooglePlayConnection.getAPIclient(), LeaderboardId, Span, LeaderboardCollection, MaxResults);
		return new PendingResultWrapper(PR, LoadScoresResult.class);
	}

	/**
	 *Asynchronously loads the player-centered page of scores for a given leaderboard. If the player does not have a score on this leaderboard, this call will return the top page instead.
	 *@param LeaderboardId ID of the leaderboard.
	 *@param Span Time span to retrieve data for. Valid values are TIME_SPAN_DAILY, TIME_SPAN_WEEKLY, or TIME_SPAN_ALL_TIME.
	 *@param LeaderboardCollection The leaderboard collection to retrieve scores for. Valid values are either COLLECTION_PUBLIC or COLLECTION_SOCIAL.
	 *@param MaxResults The maximum number of scores to fetch per page. Must be between 1 and 25.
	 *@param ForceReload If True, this call will clear any locally cached data and attempt to fetch the latest data from the server. This would commonly be used for something like a user-initiated refresh. Normally, this should be set to False to gain advantages of data caching.
	 */
	public PendingResultWrapper LoadPlayerCenteredScores2(String LeaderboardId, int Span, int LeaderboardCollection, int MaxResults, boolean ForceReload)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Leaderboards.LoadPlayerCenteredScores2: id=" + LeaderboardId + " span=" + Span + " collection=" + LeaderboardCollection + " maxresults=" + MaxResults + " forcereload=" + ForceReload);
		PendingResult<LoadScoresResult> PR = _Lb.loadPlayerCenteredScores(GooglePlayConnection.getAPIclient(), LeaderboardId, Span, LeaderboardCollection, MaxResults, ForceReload);
		return new PendingResultWrapper(PR, LoadScoresResult.class);
	}

	/**
	 * Asynchronously loads the top page of scores for a given leaderboard.
	 * @param LeaderboardId ID of the leaderboard.
	 * @param Span Time span to retrieve data for. Valid values are TIME_SPAN_DAILY, TIME_SPAN_WEEKLY, or TIME_SPAN_ALL_TIME.
	 * @param LeaderboardCollection The leaderboard collection to retrieve scores for. Valid values are either COLLECTION_PUBLIC or COLLECTION_SOCIAL.
	 * @param MaxResults The maximum number of scores to fetch per page. Must be between 1 and 25.
	 */
	public PendingResultWrapper LoadTopScores(String LeaderboardId, int Span, int LeaderboardCollection, int MaxResults)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Leaderboards.LoadTopScores: id=" + LeaderboardId + " span=" + Span + " collection=" + LeaderboardCollection + " maxresults=" + MaxResults);
		PendingResult<LoadScoresResult> PR = _Lb.loadTopScores(GooglePlayConnection.getAPIclient(), LeaderboardId, Span, LeaderboardCollection, MaxResults);
		return new PendingResultWrapper(PR, LoadScoresResult.class);
	}

	/**
	 * Asynchronously loads the top page of scores for a given leaderboard.
	 * @param LeaderboardId ID of the leaderboard.
	 * @param Span Time span to retrieve data for. Valid values are TIME_SPAN_DAILY, TIME_SPAN_WEEKLY, or TIME_SPAN_ALL_TIME.
	 * @param LeaderboardCollection The leaderboard collection to retrieve scores for. Valid values are either COLLECTION_PUBLIC or COLLECTION_SOCIAL.
	 * @param MaxResults The maximum number of scores to fetch per page. Must be between 1 and 25.
	 * @param ForceReload If True, this call will clear any locally cached data and attempt to fetch the latest data from the server. This would commonly be used for something like a user-initiated refresh. Normally, this should be set to False to gain advantages of data caching.
	 */
	public PendingResultWrapper LoadTopScores2(String LeaderboardId, int Span, int LeaderboardCollection, int MaxResults, boolean ForceReload)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Leaderboards.LoadTopScores2: id=" + LeaderboardId + " span=" + Span + " collection=" + LeaderboardCollection + " maxresults=" + MaxResults + " forcereload=" + ForceReload);
		PendingResult<LoadScoresResult> PR = _Lb.loadTopScores(GooglePlayConnection.getAPIclient(), LeaderboardId, Span, LeaderboardCollection, MaxResults, ForceReload);
		return new PendingResultWrapper(PR, LoadScoresResult.class);
	}

	/**
	 *Submits a score to a leaderboard for the currently signed in player. The score is ignored if it is worse (as defined by the leaderboard configuration) than a previously submitted score for the same player.
	 *Use this function if you do not need to be notified of the results of submitting the score, though note that the update may not be sent to the server until the next sync.
	 *The meaning of the score value depends on the formatting of the leaderboard established in the developer console. Leaderboards support the following score formats:
	 *- Fixed-point: <I>Score</I> represents a raw value, and will be formatted based on the number of decimal places configured. A score of 1000 would be formatted as 1000, 100.0, or 10.00 for 0, 1, or 2 decimal places.
	 *- Time: <I>Score</I> represents an elapsed time in milliseconds. The value will be formatted as an appropriate time value.
	 *- Currency: <I>Score</I> represents a value in micro units. For example, in USD, a score of 100 would display as $0.0001, while a score of 1000000 would display as $1.00<BR/>
	 *@param LeaderboardId The leaderboard to submit the score to.
	 *@param Score The raw score value.
	 */
	public void SubmitScore(String LeaderboardId, long Score)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Leaderboards.SubmitScore: " + Score + " id=" + LeaderboardId);
		_Lb.submitScore(GooglePlayConnection.getAPIclient(), LeaderboardId, Score);
	}

	/**
	 *Submits a score to a leaderboard for the currently signed in player. The score is ignored if it is worse (as defined by the leaderboard configuration) than a previously submitted score for the same player.
	 *Use this function if you do not need to be notified of the results of submitting the score, though note that the update may not be sent to the server until the next sync.
	 *The meaning of the score value depends on the formatting of the leaderboard established in the developer console. Leaderboards support the following score formats:
	 *- Fixed-point: <I>Score</I> represents a raw value, and will be formatted based on the number of decimal places configured. A score of 1000 would be formatted as 1000, 100.0, or 10.00 for 0, 1, or 2 decimal places.
	 *- Time: <I>Score</I> represents an elapsed time in milliseconds. The value will be formatted as an appropriate time value.
	 *- Currency: <I>Score</I> represents a value in micro units. For example, in USD, a score of 100 would display as $0.0001, while a score of 1000000 would display as $1.00<BR/>
	 *@param LeaderboardId The leaderboard to submit the score to.
	 *@param Score The raw score value.
	 *@param ScoreTag Metadata about this score. The value may contain no more than 64 URI-safe characters as defined by section 2.3 of RFC 3986.
	 */
	public void SubmitScore2(String LeaderboardId, long Score, String ScoreTag)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Leaderboards.SubmitScore2: " + Score + " id=" + LeaderboardId + " tag=" + ScoreTag);
		_Lb.submitScore(GooglePlayConnection.getAPIclient(), LeaderboardId, Score, ScoreTag);
	}

	/**
	 *Submits a score to a leaderboard for the currently signed in player. The score is ignored if it is worse (as defined by the leaderboard configuration) than a previously submitted score for the same player.
	 *This function will attempt to submit the score to the server immediately, and will return a PendingResult with information about the submission.
	 *The meaning of the score value depends on the formatting of the leaderboard established in the developer console. Leaderboards support the following score formats:
	 *- Fixed-point: <I>Score</I> represents a raw value, and will be formatted based on the number of decimal places configured. A score of 1000 would be formatted as 1000, 100.0, or 10.00 for 0, 1, or 2 decimal places.
	 *- Time: <I>Score</I> represents an elapsed time in milliseconds. The value will be formatted as an appropriate time value.
	 *- Currency: <I>Score</I> represents a value in micro units. For example, in USD, a score of 100 would display as $0.0001, while a score of 1000000 would display as $1.00<BR/>
	 *@param LeaderboardId The leaderboard to submit the score to.
	 *@param Score The raw score value.
	 */
	public PendingResultWrapper SubmitScoreImmediate(String LeaderboardId, long Score)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Leaderboards.SubmitScoreImmediate: id=" + LeaderboardId + " score=" + Score);
		PendingResult<SubmitScoreResult> PR = _Lb.submitScoreImmediate(GooglePlayConnection.getAPIclient(), LeaderboardId, Score);
		return new PendingResultWrapper(PR, SubmitScoreResult.class);
	}

	/**
	 *Submits a score to a leaderboard for the currently signed in player. The score is ignored if it is worse (as defined by the leaderboard configuration) than a previously submitted score for the same player.
	 *This function will attempt to submit the score to the server immediately, and will return a PendingResult with information about the submission.
	 *The meaning of the score value depends on the formatting of the leaderboard established in the developer console. Leaderboards support the following score formats:
	 *- Fixed-point: <I>Score</I> represents a raw value, and will be formatted based on the number of decimal places configured. A score of 1000 would be formatted as 1000, 100.0, or 10.00 for 0, 1, or 2 decimal places.
	 *- Time: <I>Score</I> represents an elapsed time in milliseconds. The value will be formatted as an appropriate time value.
	 *- Currency: <I>Score</I> represents a value in micro units. For example, in USD, a score of 100 would display as $0.0001, while a score of 1000000 would display as $1.00<BR/>
	 *@param LeaderboardId The leaderboard to submit the score to.
	 *@param Score The raw score value.
	 *@param ScoreTag Metadata about this score. The value may contain no more than 64 URI-safe characters as defined by section 2.3 of RFC 3986.
	 */
	public PendingResultWrapper SubmitScoreImmediate2(String LeaderboardId, long Score, String ScoreTag)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("Leaderboards.SubmitScoreImmediate2: id=" + LeaderboardId + " score=" + Score + " tag=" + ScoreTag);
		PendingResult<SubmitScoreResult> PR = _Lb.submitScoreImmediate(GooglePlayConnection.getAPIclient(), LeaderboardId, Score, ScoreTag);
		return new PendingResultWrapper(PR, SubmitScoreResult.class);
	}
}
