package flm.b4a.googleplay;

import android.net.Uri;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.objects.ImageViewWrapper;

import com.google.android.gms.games.multiplayer.Participant;
import com.google.android.gms.games.multiplayer.ParticipantResult;

@ShortName("GPlayParticipant")
@BA.Events(values = {
		"onImageLoaded(URI As String, Drawable As BitmapDrawable)"
})
public class ParticipantWrapper
{
	protected Participant _Participant;

	public ParticipantWrapper()
	{
		_Participant = null;
	}

	public ParticipantWrapper(Participant participant)
	{
		_Participant = participant;
	}

	public boolean IsInitialized()
	{
		return _Participant != null;
	}

	public final static int STATUS_DECLINED = Participant.STATUS_DECLINED;
	public final static int STATUS_FINISHED = Participant.STATUS_FINISHED;
	public final static int STATUS_INVITED = Participant.STATUS_INVITED;
	public final static int STATUS_JOINED = Participant.STATUS_JOINED;
	public final static int STATUS_LEFT = Participant.STATUS_LEFT;
	public final static int STATUS_NOT_INVITED_YET = Participant.STATUS_NOT_INVITED_YET;
	public final static int STATUS_UNRESPONSIVE = Participant.STATUS_UNRESPONSIVE;

	/**
	 * Returns the name to display for this participant. If the identity of the player is unknown, this will be a generic handle to describe the player.
	 */
	public String getDisplayName()
	{
		return Utils.StringOrEmpty(_Participant.getDisplayName());	
	}

	private static String GetImage(final BA ba, final Uri uri, ImageViewWrapper ImageView, String DbgText)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Participant." + DbgText + ": uri=" + uri);
		Utils.LoadImage(ba, uri, ImageView);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 *Loads the hi-res image to display for this participant and fills the given ImageView.
	 *Returns the URI of the image. If the identity of the player is unknown, this will be an empty string. It may also be an empty string if the player simply has no image.
	 */
	public String GetHiResImage(final BA ba, ImageViewWrapper ImageView)
	{
		return GetImage(ba, _Participant.getHiResImageUri(), ImageView, "GetHiResImage");
	}

	/**
	 *Loads the hi-res image to display for this participant and raises an onImageLoaded event when the load is complete.
	 *Returns the URI of the image. If the identity of the player is unknown, this will be an empty string. It may also be an empty string if the player simply has no image.
	 */
	public String GetHiResImage2(final BA ba)
	{
		return GetImage(ba, _Participant.getHiResImageUri(), null, "GetHiResImage2");
	}

	/**
	 *Loads the icon-sized image to display for this participant and fills the given ImageView.
	 *Returns the URI of the image. If the identity of the player is unknown, this will be the automatch avatar icon image for the player. It may also be an empty string if the player simply has no image.
	 */
	public String GetIconImage(final BA ba, ImageViewWrapper ImageView)
	{
		return GetImage(ba, _Participant.getIconImageUri(), ImageView, "GetIconImage");
	}

	/**
	 *Loads the icon-sized image to display for this participant and raises an onImageLoaded event when the load is complete.
	 *Returns the URI of the image. If the identity of the player is unknown, this will be the automatch avatar icon image for the player. It may also be an empty string if the player simply has no image.
	 */
	public String GetIconImage2(final BA ba)
	{
		return GetImage(ba, _Participant.getIconImageUri(), null, "GetIconImage2");
	}

	/**
	 * Returns the ID of this participant. Note that this is only valid for use in the current multiplayer room or match: a participant will not have the same ID across multiple rooms or matches.
	 */
	public String getParticipantId()
	{
		return Utils.StringOrEmpty(_Participant.getParticipantId());
	}

	/**
	 * Returns the Player object corresponding to this participant. Note that this may be Null if the identity of the player is unknown. This occurs in automatching scenarios where some players are not permitted to see the real identity of others.
	 */
	public PlayerWrapper getPlayer()
	{
		return Utils.PlayerWrapperOrNull(_Participant.getPlayer());
	}

	/**
	 * Returns the ParticipantResult associated with this participant, if any. Only applies to turn-based match participants.
	 */
	public ParticipantResultWrapper getResult()
	{
		ParticipantResult result = _Participant.getResult(); 
		if (result == null)
			return null;
		else
			return new ParticipantResultWrapper(result);
	}

	/**
	 *Retrieves the status of this participant.
	 *Possible status values for room participants are STATUS_INVITED, STATUS_JOINED, STATUS_DECLINED, and STATUS_LEFT.
	 *Possible status values for turn-based match participants are all of the above, STATUS_NOT_INVITED_YET, STATUS_FINISHED, and STATUS_UNRESPONSIVE.
	 */
	public int getStatus ()
	{
		return _Participant.getStatus();
	}

	/**
	 * Retrieves the connected status of the participant. If True indicates that participant is in the connected set of the room. Only applies to room participants.
	 */
	public boolean IsConnectedToRoom ()
	{
		return _Participant.isConnectedToRoom();
	}
}
