package flm.b4a.googleplay;

import java.util.concurrent.TimeUnit;

import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.collections.Map;

import com.google.android.gms.common.api.PendingResult;
import com.google.android.gms.common.api.Result;
import com.google.android.gms.common.api.ResultCallback;
import com.google.android.gms.games.PlayerBuffer;
import com.google.android.gms.games.Players.LoadPlayersResult;
import com.google.android.gms.games.achievement.AchievementBuffer;
import com.google.android.gms.games.achievement.Achievements.LoadAchievementsResult;
import com.google.android.gms.games.achievement.Achievements.UpdateAchievementResult;
import com.google.android.gms.games.leaderboard.Leaderboard;
import com.google.android.gms.games.leaderboard.LeaderboardBuffer;
import com.google.android.gms.games.leaderboard.LeaderboardScore;
import com.google.android.gms.games.leaderboard.LeaderboardScoreBuffer;
import com.google.android.gms.games.leaderboard.Leaderboards.LeaderboardMetadataResult;
import com.google.android.gms.games.leaderboard.Leaderboards.LoadPlayerScoreResult;
import com.google.android.gms.games.leaderboard.Leaderboards.LoadScoresResult;
import com.google.android.gms.games.leaderboard.Leaderboards.SubmitScoreResult;
import com.google.android.gms.games.leaderboard.ScoreSubmissionData;
import com.google.android.gms.games.multiplayer.InvitationBuffer;
import com.google.android.gms.games.multiplayer.Invitations.LoadInvitationsResult;
import com.google.android.gms.games.multiplayer.turnbased.LoadMatchesResponse;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMultiplayer.CancelMatchResult;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMultiplayer.InitiateMatchResult;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMultiplayer.LeaveMatchResult;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMultiplayer.LoadMatchResult;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMultiplayer.LoadMatchesResult;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMultiplayer.UpdateMatchResult;
import com.google.android.gms.games.snapshot.Snapshot;
import com.google.android.gms.games.snapshot.SnapshotMetadata;
import com.google.android.gms.games.snapshot.SnapshotMetadataBuffer;
import com.google.android.gms.games.snapshot.Snapshots.CommitSnapshotResult;
import com.google.android.gms.games.snapshot.Snapshots.DeleteSnapshotResult;
import com.google.android.gms.games.snapshot.Snapshots.LoadSnapshotsResult;
import com.google.android.gms.games.snapshot.Snapshots.OpenSnapshotResult;
import com.google.android.gms.games.stats.PlayerStats;
import com.google.android.gms.games.stats.Stats.LoadPlayerStatsResult;

@BA.ShortName("GPlayPendingResult")
public class PendingResultWrapper
{
	protected PendingResult<?> _PR;
	protected Class<?> _ResultClass;

	public PendingResultWrapper(PendingResult<?> PR, Class<?> Class)
	{
		_PR = PR;
		_ResultClass = Class;
	}

	public static final String RESULTMAP_ACHIEVEMENT_ID = "AchievementID";
	public static final String RESULTMAP_ACHIEVEMENTS = "Achievements";
	public static final String RESULTMAP_INVITATIONS = "Invitations";
	public static final String RESULTMAP_LEADERBOARD = "Leaderboard";
	public static final String RESULTMAP_LEADERBOARDS = "Leaderboards";
	public static final String RESULTMAP_MATCH = "Match";
	public static final String RESULTMAP_MATCHES = "Matches";
	public static final String RESULTMAP_MATCH_ID = "MatchId";
	public static final String RESULTMAP_PLAYER_SCORE = "PlayerScore";
	public static final String RESULTMAP_PLAYER_STATS = "PlayerStats";
	public static final String RESULTMAP_PLAYERS = "Players";
	public static final String RESULTMAP_SCORE_BUFFER = "ScoreBuffer";
	public static final String RESULTMAP_SCORE_DATA = "ScoreData";
	public static final String RESULTMAP_SCORES = "Scores";
	public static final String RESULTMAP_SNAPSHOT = "Snapshot";
	public static final String RESULTMAP_SNAPSHOT_CONFLICT_ID = "ConflictId";
	public static final String RESULTMAP_SNAPSHOT_CONFLICTING_WITH = "ConflictingWith";
	public static final String RESULTMAP_SNAPSHOT_ID = "SnapshotId";
	public static final String RESULTMAP_SNAPSHOT_METADATA = "SnapshotMetadata";
	public static final String RESULTMAP_SNAPSHOTS = "Snapshots";
	public static final String RESULTMAP_STATUS_CODE = "StatusCode";

	private Map CreateResultMap(Object Result)
	{
		Map mResult = new Map();
		mResult.Initialize();
		// CancelMatchResult
		if (_ResultClass == CancelMatchResult.class) {
			mResult.Put(RESULTMAP_MATCH_ID, Utils.StringOrEmpty(((CancelMatchResult)Result).getMatchId()));
			mResult.Put(RESULTMAP_STATUS_CODE, ((CancelMatchResult)Result).getStatus().getStatusCode());
		}
		// CommitSnapshotResult
		else if (_ResultClass == CommitSnapshotResult.class) {
			SnapshotMetadata Metadata = ((CommitSnapshotResult)Result).getSnapshotMetadata();
			mResult.Put(RESULTMAP_SNAPSHOT_METADATA, Utils.MetadataWrapperOrNull(Metadata));
			mResult.Put(RESULTMAP_STATUS_CODE, ((CommitSnapshotResult)Result).getStatus().getStatusCode());
		}
		// DeleteSnapshotResult
		else if (_ResultClass == DeleteSnapshotResult.class) {
			mResult.Put(RESULTMAP_SNAPSHOT_ID, Utils.StringOrEmpty(((DeleteSnapshotResult)Result).getSnapshotId()));
			mResult.Put(RESULTMAP_STATUS_CODE, ((DeleteSnapshotResult)Result).getStatus().getStatusCode());
		}
		// InitiateMatchResult
		else if (_ResultClass == InitiateMatchResult.class) {
			mResult.Put(RESULTMAP_MATCH, Utils.MatchWrapperOrNull(((InitiateMatchResult)Result).getMatch()));
			mResult.Put(RESULTMAP_STATUS_CODE, ((InitiateMatchResult)Result).getStatus().getStatusCode());
		}
		// LeaderboardMetadataResult
		else if (_ResultClass == LeaderboardMetadataResult.class) {
			LeaderboardBuffer Buffer = ((LeaderboardMetadataResult)Result).getLeaderboards();
			mResult.Put(RESULTMAP_LEADERBOARDS, Utils.FromBufferToArray_Leaderboard(Buffer));
			mResult.Put(RESULTMAP_STATUS_CODE, ((LeaderboardMetadataResult)Result).getStatus().getStatusCode());
		}
		// LeaveMatchResult
		else if (_ResultClass == LeaveMatchResult.class) {
			mResult.Put(RESULTMAP_MATCH, Utils.MatchWrapperOrNull(((LeaveMatchResult)Result).getMatch()));
			mResult.Put(RESULTMAP_STATUS_CODE, ((LeaveMatchResult)Result).getStatus().getStatusCode());
		}
		// LoadAchievementsResult
		else if (_ResultClass == LoadAchievementsResult.class) {
			AchievementBuffer Achievements = ((LoadAchievementsResult)Result).getAchievements();
			mResult.Put(RESULTMAP_ACHIEVEMENTS, Utils.FromBufferToArray_Achievement(Achievements));
			mResult.Put(RESULTMAP_STATUS_CODE, ((LoadAchievementsResult)Result).getStatus().getStatusCode());
		}
		// LoadInvitationsResult
		else if (_ResultClass == LoadInvitationsResult.class) {
			InvitationBuffer invitations = ((LoadInvitationsResult)Result).getInvitations();
			mResult.Put(RESULTMAP_INVITATIONS, Utils.FromBufferToArray_Invitation(invitations));
			mResult.Put(RESULTMAP_STATUS_CODE, ((LoadInvitationsResult)Result).getStatus().getStatusCode());
		}
		// LoadMatchResult
		else if (_ResultClass == LoadMatchResult.class) {
			mResult.Put(RESULTMAP_MATCH, Utils.MatchWrapperOrNull(((LoadMatchResult)Result).getMatch()));
			mResult.Put(RESULTMAP_STATUS_CODE, ((LoadMatchResult)Result).getStatus().getStatusCode());
		}
		// LoadMatchesResult
		else if (_ResultClass == LoadMatchesResult.class) {
			LoadMatchesResponse Matches = ((LoadMatchesResult)Result).getMatches();
			if (Matches == null || !Matches.hasData())
				mResult.Put(RESULTMAP_MATCHES, null);
			else
				mResult.Put(RESULTMAP_MATCHES, new LoadMatchesResponseWrapper(Matches));
			mResult.Put(RESULTMAP_STATUS_CODE, ((LoadMatchesResult)Result).getStatus().getStatusCode());
		}
		// LoadPlayersResult
		else if (_ResultClass == LoadPlayersResult.class) {
			PlayerBuffer Players = ((LoadPlayersResult)Result).getPlayers();
			mResult.Put(RESULTMAP_PLAYERS, Utils.FromBufferToArray_Player(Players));
			mResult.Put(RESULTMAP_STATUS_CODE, ((LoadPlayersResult)Result).getStatus().getStatusCode());
		}
		// LoadPlayerScoreResult
		else if (_ResultClass == LoadPlayerScoreResult.class) {
			LeaderboardScore PlayerScore = ((LoadPlayerScoreResult)Result).getScore();
			mResult.Put(RESULTMAP_PLAYER_SCORE, Utils.LeaderboardScoreWrapperOrNull(PlayerScore));
			mResult.Put(RESULTMAP_STATUS_CODE, ((LoadPlayerScoreResult)Result).getStatus().getStatusCode());
		}
		// LoadPlayerStatsResult
		else if (_ResultClass == LoadPlayerStatsResult.class) {
			PlayerStats PlayerStats = ((LoadPlayerStatsResult)Result).getPlayerStats();
			mResult.Put(RESULTMAP_PLAYER_STATS, Utils.PlayerStatsWrapperOrNull(PlayerStats));
			mResult.Put(RESULTMAP_STATUS_CODE, ((LoadPlayerStatsResult)Result).getStatus().getStatusCode());
		}
		// LoadScoresResult
		else if (_ResultClass == LoadScoresResult.class) {
			Leaderboard leaderboard = ((LoadScoresResult)Result).getLeaderboard();
			LeaderboardScoreBuffer buffer = ((LoadScoresResult)Result).getScores();
			mResult.Put(RESULTMAP_LEADERBOARD, Utils.LeaderboardWrapperOrNull(leaderboard));
			mResult.Put(RESULTMAP_SCORE_BUFFER, buffer);
			mResult.Put(RESULTMAP_SCORES, Utils.FromBufferToArray_LeaderboardScore(buffer));
			mResult.Put(RESULTMAP_STATUS_CODE, ((LoadScoresResult)Result).getStatus().getStatusCode());
		}
		// LoadSnapshotsResult
		else if (_ResultClass == LoadSnapshotsResult.class) {
			SnapshotMetadataBuffer SM_Buffer = ((LoadSnapshotsResult)Result).getSnapshots();
			mResult.Put(RESULTMAP_SNAPSHOTS, Utils.FromBufferToArray_SnapshotMetadata(SM_Buffer));
			mResult.Put(RESULTMAP_STATUS_CODE, ((LoadSnapshotsResult)Result).getStatus().getStatusCode());
		}
		// OpenSnapshotResult
		else if (_ResultClass == OpenSnapshotResult.class) {
			Snapshot snapshot = ((OpenSnapshotResult)Result).getSnapshot();
			Snapshot conflictingSnapshot = ((OpenSnapshotResult)Result).getConflictingSnapshot();
			mResult.Put(RESULTMAP_SNAPSHOT_CONFLICT_ID, Utils.StringOrEmpty(((OpenSnapshotResult)Result).getConflictId()));
			mResult.Put(RESULTMAP_SNAPSHOT_CONFLICTING_WITH, Utils.SnapshotWrapperOrNull(conflictingSnapshot));
			mResult.Put(RESULTMAP_SNAPSHOT, Utils.SnapshotWrapperOrNull(snapshot));
			mResult.Put(RESULTMAP_STATUS_CODE, ((OpenSnapshotResult)Result).getStatus().getStatusCode());
		}
		// SubmitScoreResult
		else if (_ResultClass == SubmitScoreResult.class) {
			ScoreSubmissionData SSD = ((SubmitScoreResult)Result).getScoreData();
			mResult.Put(RESULTMAP_SCORE_DATA, Utils.ScoreSubmissionDataWrapperOrNull(SSD));
			mResult.Put(RESULTMAP_STATUS_CODE, ((SubmitScoreResult)Result).getStatus().getStatusCode());
		}
		// UpdateAchievementResult
		else if (_ResultClass == UpdateAchievementResult.class) {
			mResult.Put(RESULTMAP_ACHIEVEMENT_ID, Utils.StringOrEmpty(((UpdateAchievementResult)Result).getAchievementId()));
			mResult.Put(RESULTMAP_STATUS_CODE, ((UpdateAchievementResult)Result).getStatus().getStatusCode());
		}
		// UpdateMatchResult
		else if (_ResultClass == UpdateMatchResult.class) {
			mResult.Put(RESULTMAP_MATCH, Utils.MatchWrapperOrNull(((UpdateMatchResult)Result).getMatch()));
			mResult.Put(RESULTMAP_STATUS_CODE, ((UpdateMatchResult)Result).getStatus().getStatusCode());
		}
		return mResult;
	}

	/** Blocks until the task is completed. This is not allowed on the UI thread. */
	public Map Await()
	{
		return CreateResultMap(_PR.await());
	}

	/** Blocks until the task is completed or has timed out waiting for the result. This is not allowed on the UI thread.
	 * @param TimeOut In milliseconds. */
	public Map Await2(long TimeOut)
	{
		return CreateResultMap(_PR.await(TimeOut, TimeUnit.MILLISECONDS));
	}

	/** Requests that the pending result be canceled. If the result is available, but not consumed it will be released. If the result is set after cancellation was requested it is immediately released. */
	public void Cancel()
	{
		_PR.cancel();
	}

	/** Indicates whether the pending result has been canceled either due to calling Disconnect or calling Cancel directly on the pending result. */
	public boolean IsCanceled()
	{
		return _PR.isCanceled();
	}

	private void RaiseEvent(BA ba, String evtName, Result result, String strLog)
	{
		// Called by the ...Event functions below to notify a result to the user through an event
		if (GooglePlayConnection.DEBUG) BA.Log(strLog + " -> " + evtName);
		ba.raiseEvent2(this, true, evtName, false, new Object[] { CreateResultMap(result) });
	}

	private class InitiateMatchEvent implements ResultCallback<InitiateMatchResult>
	{
		private BA _ba;
		private String _evtName;

		public InitiateMatchEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(InitiateMatchResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "InitiateMatchResult");
		}
	}

	private class CancelMatchEvent implements ResultCallback<CancelMatchResult>
	{
		private BA _ba;
		private String _evtName;

		public CancelMatchEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(CancelMatchResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "CancelMatchResult");
		}
	}

	private class CommitSnapshotEvent implements ResultCallback<CommitSnapshotResult>
	{
		private BA _ba;
		private String _evtName;

		public CommitSnapshotEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(CommitSnapshotResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "CommitSnapshotResult");
		}
	}

	private class DeleteSnapshotEvent implements ResultCallback<DeleteSnapshotResult>
	{
		private BA _ba;
		private String _evtName;

		public DeleteSnapshotEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(DeleteSnapshotResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "DeleteSnapshotResult");
		}
	}

	private class LeaderboardMetadataEvent implements ResultCallback<LeaderboardMetadataResult>
	{
		private BA _ba;
		private String _evtName;

		public LeaderboardMetadataEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(LeaderboardMetadataResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "LeaderboardMetadataResult");
		}
	}

	private class LeaveMatchEvent implements ResultCallback<LeaveMatchResult>
	{
		private BA _ba;
		private String _evtName;

		public LeaveMatchEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(LeaveMatchResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "LeaveMatchResult");
		}
	}

	private class LoadAchievementsEvent implements ResultCallback<LoadAchievementsResult>
	{
		private BA _ba;
		private String _evtName;

		public LoadAchievementsEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(LoadAchievementsResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "LoadAchievementsResult");
		}
	}

	private class LoadInvitationsEvent implements ResultCallback<LoadInvitationsResult>
	{
		private BA _ba;
		private String _evtName;

		public LoadInvitationsEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(LoadInvitationsResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "LoadInvitationsResult");
		}
	}

	private class LoadMatchEvent implements ResultCallback<LoadMatchResult>
	{
		private BA _ba;
		private String _evtName;

		public LoadMatchEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(LoadMatchResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "LoadMatchResult");
		}
	}

	private class LoadMatchesEvent implements ResultCallback<LoadMatchesResult>
	{
		private BA _ba;
		private String _evtName;

		public LoadMatchesEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(LoadMatchesResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "LoadMatchesResult");
		}
	}

	private class LoadPlayersEvent implements ResultCallback<LoadPlayersResult>
	{
		private BA _ba;
		private String _evtName;

		public LoadPlayersEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(LoadPlayersResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "LoadPlayersResult");
		}
	}

	private class LoadPlayerScoreEvent implements ResultCallback<LoadPlayerScoreResult>
	{
		private BA _ba;
		private String _evtName;

		public LoadPlayerScoreEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(LoadPlayerScoreResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "LoadPlayerScoreResult");
		}
	}

	private class LoadPlayerStatsEvent implements ResultCallback<LoadPlayerStatsResult>
	{
		private BA _ba;
		private String _evtName;

		public LoadPlayerStatsEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(LoadPlayerStatsResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "LoadPlayerStatsResult");
		}
	}

	private class LoadScoresEvent implements ResultCallback<LoadScoresResult>
	{
		private BA _ba;
		private String _evtName;

		public LoadScoresEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(LoadScoresResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "LoadScoresResult");
		}
	}

	private class LoadSnapshotsEvent implements ResultCallback<LoadSnapshotsResult>
	{
		private BA _ba;
		private String _evtName;

		public LoadSnapshotsEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(LoadSnapshotsResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "LoadSnapshotsResult");
		}
	}

	private class OpenSnapshotEvent implements ResultCallback<OpenSnapshotResult>
	{
		private BA _ba;
		private String _evtName;

		public OpenSnapshotEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(OpenSnapshotResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "OpenSnapshotResult");
		}
	}

	private class SubmitScoreEvent implements ResultCallback<SubmitScoreResult>
	{
		private BA _ba;
		private String _evtName;

		public SubmitScoreEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(SubmitScoreResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "SubmitScoreResult");
		}
	}

	private class UpdateAchievementEvent implements ResultCallback<UpdateAchievementResult>
	{
		private BA _ba;
		private String _evtName;

		public UpdateAchievementEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(UpdateAchievementResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "UpdateAchievementResult");
		}
	}

	private class UpdateMatchEvent implements ResultCallback<UpdateMatchResult>
	{
		private BA _ba;
		private String _evtName;

		public UpdateMatchEvent(BA ba, String evtName)
		{
			_ba = ba;
			_evtName = evtName;
		}

		@Override
		public void onResult(UpdateMatchResult Result)
		{
			RaiseEvent(_ba, _evtName, Result, "UpdateMatchResult");
		}
	}

	@SuppressWarnings("unchecked")
	private void SetResultCallback(final BA ba, final String EventName, long Time)
	{
		// CancelMatchResult
		if (_ResultClass == CancelMatchResult.class) {
			ResultCallback<CancelMatchResult> RC_Cancel = new CancelMatchEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<CancelMatchResult>)_PR).setResultCallback(RC_Cancel);
			else
				((PendingResult<CancelMatchResult>)_PR).setResultCallback(RC_Cancel, Time, TimeUnit.MILLISECONDS);
		}
		// CommitSnapshotResult
		else if (_ResultClass == CommitSnapshotResult.class) {
			ResultCallback<CommitSnapshotResult> RC_Commit = new CommitSnapshotEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<CommitSnapshotResult>)_PR).setResultCallback(RC_Commit);
			else
				((PendingResult<CommitSnapshotResult>)_PR).setResultCallback(RC_Commit, Time, TimeUnit.MILLISECONDS);
		}
		// DeleteSnapshotResult
		else if (_ResultClass == DeleteSnapshotResult.class) {
			ResultCallback<DeleteSnapshotResult> RC_Delete = new DeleteSnapshotEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<DeleteSnapshotResult>)_PR).setResultCallback(RC_Delete);
			else
				((PendingResult<DeleteSnapshotResult>)_PR).setResultCallback(RC_Delete, Time, TimeUnit.MILLISECONDS);
		}
		// InitiateMatchResult
		else if (_ResultClass == InitiateMatchResult.class) {
			ResultCallback<InitiateMatchResult> RC_Initiate = new InitiateMatchEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<InitiateMatchResult>)_PR).setResultCallback(RC_Initiate);
			else
				((PendingResult<InitiateMatchResult>)_PR).setResultCallback(RC_Initiate, Time, TimeUnit.MILLISECONDS);
		}
		// LeaderboardMetadataResult
		else if (_ResultClass == LeaderboardMetadataResult.class) {
			ResultCallback<LeaderboardMetadataResult> RC_Leave = new LeaderboardMetadataEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<LeaderboardMetadataResult>)_PR).setResultCallback(RC_Leave);
			else
				((PendingResult<LeaderboardMetadataResult>)_PR).setResultCallback(RC_Leave, Time, TimeUnit.MILLISECONDS);
		}
		// LeaveMatchResult
		else if (_ResultClass == LeaveMatchResult.class) {
			ResultCallback<LeaveMatchResult> RC_Leave = new LeaveMatchEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<LeaveMatchResult>)_PR).setResultCallback(RC_Leave);
			else
				((PendingResult<LeaveMatchResult>)_PR).setResultCallback(RC_Leave, Time, TimeUnit.MILLISECONDS);
		}
		// LoadAchievementsResult
		else if (_ResultClass == LoadAchievementsResult.class) {
			ResultCallback<LoadAchievementsResult> RC = new LoadAchievementsEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<LoadAchievementsResult>)_PR).setResultCallback(RC);
			else
				((PendingResult<LoadAchievementsResult>)_PR).setResultCallback(RC, Time, TimeUnit.MILLISECONDS);
		}
		// LoadInvitationsResult
		else if (_ResultClass == LoadInvitationsResult.class) {
			ResultCallback<LoadInvitationsResult> RC = new LoadInvitationsEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<LoadInvitationsResult>)_PR).setResultCallback(RC);
			else
				((PendingResult<LoadInvitationsResult>)_PR).setResultCallback(RC, Time, TimeUnit.MILLISECONDS);
		}
		// LoadMatchResult
		else if (_ResultClass == LoadMatchResult.class) {
			ResultCallback<LoadMatchResult> RC_LoadM = new LoadMatchEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<LoadMatchResult>)_PR).setResultCallback(RC_LoadM);
			else
				((PendingResult<LoadMatchResult>)_PR).setResultCallback(RC_LoadM, Time, TimeUnit.MILLISECONDS);
		}
		// LoadMatchesResult
		else if (_ResultClass == LoadMatchesResult.class) {
			ResultCallback<LoadMatchesResult> RC_LoadMs = new LoadMatchesEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<LoadMatchesResult>)_PR).setResultCallback(RC_LoadMs);
			else
				((PendingResult<LoadMatchesResult>)_PR).setResultCallback(RC_LoadMs, Time, TimeUnit.MILLISECONDS);
		}
		// LoadPlayersResult
		else if (_ResultClass == LoadPlayersResult.class) {
			ResultCallback<LoadPlayersResult> RC = new LoadPlayersEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<LoadPlayersResult>)_PR).setResultCallback(RC);
			else
				((PendingResult<LoadPlayersResult>)_PR).setResultCallback(RC, Time, TimeUnit.MILLISECONDS);
		}
		// LoadPlayerScoreResult
		else if (_ResultClass == LoadPlayerScoreResult.class) {
			ResultCallback<LoadPlayerScoreResult> RC = new LoadPlayerScoreEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<LoadPlayerScoreResult>)_PR).setResultCallback(RC);
			else
				((PendingResult<LoadPlayerScoreResult>)_PR).setResultCallback(RC, Time, TimeUnit.MILLISECONDS);
		}
		// LoadPlayerStatsResult
		else if (_ResultClass == LoadPlayerStatsResult.class) {
			ResultCallback<LoadPlayerStatsResult> RC = new LoadPlayerStatsEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<LoadPlayerStatsResult>)_PR).setResultCallback(RC);
			else
				((PendingResult<LoadPlayerStatsResult>)_PR).setResultCallback(RC, Time, TimeUnit.MILLISECONDS);
		}
		// LoadScoresResult
		else if (_ResultClass == LoadScoresResult.class) {
			ResultCallback<LoadScoresResult> RC = new LoadScoresEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<LoadScoresResult>)_PR).setResultCallback(RC);
			else
				((PendingResult<LoadScoresResult>)_PR).setResultCallback(RC, Time, TimeUnit.MILLISECONDS);
		}
		// LoadSnapshotsResult
		else if (_ResultClass == LoadSnapshotsResult.class) {
			ResultCallback<LoadSnapshotsResult> RC = new LoadSnapshotsEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<LoadSnapshotsResult>)_PR).setResultCallback(RC);
			else
				((PendingResult<LoadSnapshotsResult>)_PR).setResultCallback(RC, Time, TimeUnit.MILLISECONDS);
		}
		// OpenSnapshotResult
		else if (_ResultClass == OpenSnapshotResult.class) {
			ResultCallback<OpenSnapshotResult> RC = new OpenSnapshotEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<OpenSnapshotResult>)_PR).setResultCallback(RC);
			else
				((PendingResult<OpenSnapshotResult>)_PR).setResultCallback(RC, Time, TimeUnit.MILLISECONDS);
		}
		// SubmitScoreResult
		else if (_ResultClass == SubmitScoreResult.class) {
			ResultCallback<SubmitScoreResult> RC_Update = new SubmitScoreEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<SubmitScoreResult>)_PR).setResultCallback(RC_Update);
			else
				((PendingResult<SubmitScoreResult>)_PR).setResultCallback(RC_Update, Time, TimeUnit.MILLISECONDS);
		}
		// UpdateAchievementResult
		else if (_ResultClass == UpdateAchievementResult.class) {
			ResultCallback<UpdateAchievementResult> RC_Update = new UpdateAchievementEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<UpdateAchievementResult>)_PR).setResultCallback(RC_Update);
			else
				((PendingResult<UpdateAchievementResult>)_PR).setResultCallback(RC_Update, Time, TimeUnit.MILLISECONDS);
		}
		// UpdateMatchResult
		else if (_ResultClass == UpdateMatchResult.class) {
			ResultCallback<UpdateMatchResult> RC_Update = new UpdateMatchEvent(ba, EventName);
			if (Time < 0)
				((PendingResult<UpdateMatchResult>)_PR).setResultCallback(RC_Update);
			else
				((PendingResult<UpdateMatchResult>)_PR).setResultCallback(RC_Update, Time, TimeUnit.MILLISECONDS);
		}
	}

	/**
	 *Sets the name of the event that will receive the result.
	 *The event must be declared like this: Sub &lt;EventName&gt;(Result As Map) 
	 * */
	public void SetResultEvent(BA ba, String EventName)
	{
		SetResultCallback(ba, EventName.toLowerCase(BA.cul), -1);
	}

	/**
	 *Sets the name of the event that will receive the result.
	 *The event must be declared like this: Sub &lt;EventName&gt;(Result As Map) 
	 *@param TimeOut In milliseconds. */
	public void SetResultEvent2(BA ba, String EventName, long TimeOut)
	{
		SetResultCallback(ba, EventName.toLowerCase(BA.cul), TimeOut);
	}
}
