package flm.b4a.googleplay;

import android.net.Uri;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.objects.ImageViewWrapper;

import com.google.android.gms.games.Player;
import com.google.android.gms.games.PlayerLevelInfo;

@ShortName("GPlayPlayer")
@BA.Events(values = {
		"onImageLoaded(URI As String, Drawable As BitmapDrawable)"
})
public class PlayerWrapper
{
	protected Player _Player;

	public PlayerWrapper()
	{
		_Player = null;
	}

	public PlayerWrapper(Player player)
	{
		_Player = player;
	}

	public boolean IsInitialized()
	{
		return _Player != null;
	}

	public static final long CURRENT_XP_UNKNOWN = Player.CURRENT_XP_UNKNOWN; 
	public static final long TIMESTAMP_UNKNOWN = Player.TIMESTAMP_UNKNOWN; 

	private static String GetImage(final BA ba, final Uri uri, ImageViewWrapper ImageView, String DbgText)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Player." + DbgText + ": uri=" + uri);
		Utils.LoadImage(ba, uri, ImageView);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 *Loads this player's banner image and fills the given ImageView.
	 *Returns the URI of the image, or an empty string if the player has none.
	 *@param Landscape Whether the image is the landscape banner image (True) or the portrait banner image (False).
	 */
	public String GetBannerImage(final BA ba, ImageViewWrapper ImageView, boolean Landscape)
	{
		Uri uri = Landscape ? _Player.getBannerImageLandscapeUri() : _Player.getBannerImagePortraitUri();
		return GetImage(ba, uri, ImageView, "GetBannerImage");
	}

	/**
	 *Loads this player's banner image and raises an onImageLoaded event when the load is complete.
	 *Returns the URI of the image, or an empty string if the player has none.
	 *@param Landscape Whether the image is the landscape banner image (True) or the portrait banner image (False).
	 */
	public String GetBannerImage2(final BA ba, boolean Landscape)
	{
		Uri uri = Landscape ? _Player.getBannerImageLandscapeUri() : _Player.getBannerImagePortraitUri();
		return GetImage(ba, uri, null, "GetBannerImage2");
	}

	/**
	 * Retrieves the display name for this player.
	 */
	public String getDisplayName()
	{
		return Utils.StringOrEmpty(_Player.getDisplayName());
	}

	/**
	 *Loads this player's hi-res profile image and fills the given ImageView.
	 *Returns the URI of the image, or an empty string if the player has none.
	 */
	public String GetHiResImage(final BA ba, ImageViewWrapper ImageView)
	{
		return GetImage(ba, _Player.getHiResImageUri(), ImageView, "GetHiResImage");
	}

	/**
	 *Loads this player's hi-res profile image and raises an onImageLoaded event when the load is complete.
	 *Returns the URI of the image, or an empty string if the player has none.
	 */
	public String GetHiResImage2(final BA ba)
	{
		return GetImage(ba, _Player.getHiResImageUri(), null, "GetHiResImage2");
	}

	/**
	 *Loads the player's icon-size profile image and fills the given ImageView.
	 *Returns the URI of the image, or an empty string if the player has none.
	 */
	public String GetIconImage(final BA ba, ImageViewWrapper ImageView)
	{
		return GetImage(ba, _Player.getIconImageUri(), ImageView, "GetIconImage");
	}

	/**
	 *Loads the player's icon-size profile image and raises an onImageLoaded event when the load is complete.
	 *Returns the URI of the image, or an empty string if the player has none.
	 */
	public String GetIconImage2(final BA ba)
	{
		return GetImage(ba, _Player.getIconImageUri(), null, "GetIconImage2");
	}

	/**
	 * Retrieves the timestamp (in ms since epoch) at which this player last played a multiplayer game with the currently signed in user. If the timestamp is not found, this function returns TIMESTAMP_UNKNOWN.
	 */
	public long getLastPlayedWithTimestamp()
	{
		return _Player.getLastPlayedWithTimestamp();
	}

	/**
	 * Retrieves the player level associated information if any exists. If no level information exists for this player, this function will return null.
	 */
	public PlayerLevelInfoWrapper getLevelInfo()
	{
		PlayerLevelInfo lvlInfo = _Player.getLevelInfo();
		if (lvlInfo == null)
			return null;
		else
			return new PlayerLevelInfoWrapper(lvlInfo);
	}

	/**
	 * Retrieves the ID of this player.
	 */
	public String getPlayerId()
	{
		return Utils.StringOrEmpty(_Player.getPlayerId());
	}

	/**
	 * Retrieves the timestamp (in ms since epoch) at which the player data was last updated locally.
	 */
	public long getRetrievedTimestamp()
	{
		return _Player.getRetrievedTimestamp();
	}

	/**
	 * Retrieves the title of the player. This is based on the player's gameplay activity in apps using Google Play Games services. Note that not all players have titles, and that a player's title may change over time.
	 */
	public String getTitle()
	{
		return Utils.StringOrEmpty(_Player.getTitle());
	}

	/**
	 * Indicates whether this player has a hi-res profile image to display.
	 */
	public boolean HasHiResImage()
	{
		return _Player.hasHiResImage();
	}

	/**
	 * Indicates whether this player has an icon-size profile image to display.
	 */
	public boolean HasIconImage()
	{
		return _Player.hasIconImage();
	}
}
