package flm.b4a.googleplay;

import java.util.ArrayList;

import android.content.Intent;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ShortName;

import com.google.android.gms.common.api.PendingResult;
import com.google.android.gms.games.Games;
import com.google.android.gms.games.Player;
import com.google.android.gms.games.Players;
import com.google.android.gms.games.Players.LoadPlayersResult;

@ShortName("GPlayPlayers")
public class PlayersWrapper
{
	protected Players _Players = null;

	public void Initialize()
	{
		_Players = Games.Players;
	}

	public boolean IsInitialized()
	{
		return _Players != null;
	}

	public static final String EXTRA_PLAYER_SEARCH_RESULTS = Players.EXTRA_PLAYER_SEARCH_RESULTS;
	public static final String RESULTMAP_PLAYERS = PendingResultWrapper.RESULTMAP_PLAYERS;
	public static final String RESULTMAP_STATUS_CODE = PendingResultWrapper.RESULTMAP_STATUS_CODE;

	/**
	 * Returns an intent that will display a screen where the user can compare themself against another player's profile. This intent must be invoked with StartActivityForResult.
	 */
	public Intent GetCompareProfileIntent(PlayerWrapper Player)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Players.GetCompareProfileIntent");
		return _Players.getCompareProfileIntent(GooglePlayConnection.getAPIclient(), Player._Player); 
	}

	/**
	 * Returns the Player object representing the currently signed in player. May be null if an error occurred while communicating with the games service.
	 */
	public PlayerWrapper getCurrentPlayer()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Players.CurrentPlayer=" + _Players.getCurrentPlayer(GooglePlayConnection.getAPIclient()));
		Player player = _Players.getCurrentPlayer(GooglePlayConnection.getAPIclient()); 
		return Utils.PlayerWrapperOrNull(player);
	}

	/**
	 * Returns the player ID for the currently signed in player. May be an empty string if an error occurred while communicating with the games service.
	 */
	public String getCurrentPlayerId()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Players.CurrentPlayerId=" + _Players.getCurrentPlayerId(GooglePlayConnection.getAPIclient()));
		return Utils.StringOrEmpty(_Players.getCurrentPlayerId(GooglePlayConnection.getAPIclient()));
	}

	/**
	 *Returns an intent that will display a screen where the user can search for people on Google+. This intent must be invoked with StartActivityForResult.
	 *If the user canceled, the result code will be ACTIVITY_RESULT_CANCELED. If the user selected any player from the search results list, the result code will be ACTIVITY_RESULT_OK, and the data intent will contain a Player object in EXTRA_PLAYER_SEARCH_RESULTS (that you can retrieve with GetPlayerFromIntentExtra).
	 */
	public Intent GetPlayerSearchIntent()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Players.GetPlayerSearchIntent");
		return _Players.getPlayerSearchIntent(GooglePlayConnection.getAPIclient());
	}

	/**
	 * Retrieves a player from the extended data (EXTRA_PLAYER_SEARCH_RESULTS) of the given intent.
	 */
	public static PlayerWrapper GetPlayerFromIntentExtra(Intent Intent)
	{
		ArrayList<Player> players = Intent.getParcelableArrayListExtra(Players.EXTRA_PLAYER_SEARCH_RESULTS);
		if (players != null && players.size() > 0)
			return Utils.PlayerWrapperOrNull(players.get(0));
		else
			return null;
	}

	/**
	 * Asynchronously loads a list of players that have connected to this game (and that the user has permission to know about).
	 * @param ForceReload If True, this call will clear any locally cached data and attempt to fetch the latest data from the server. This would commonly be used for something like a user-initiated refresh. Normally, this should be set to False to gain advantages of data caching.
	 */
	public PendingResultWrapper LoadConnectedPlayers(boolean ForceReload)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Players.LoadConnectedPlayers: ForceReload=" + ForceReload);
		PendingResult<LoadPlayersResult> PR = _Players.loadConnectedPlayers(GooglePlayConnection.getAPIclient(), ForceReload);
		return new PendingResultWrapper(PR, LoadPlayersResult.class);
	}

	/**
	 * Loads the initial page of players the currently signed-in player can invite to a multiplayer game, sorted alphabetically by name.
	 * @param PageSize The number of entries to request for this initial page. Note that if cached data already exists, the returned buffer may contain more than this size, but it is guaranteed to contain at least this many if the collection contains enough records. This must be a value between 1 and 25.
	 * @param ForceReload If True, this call will clear any locally cached data and attempt to fetch the latest data from the server. This would commonly be used for something like a user-initiated refresh. Normally, this should be set to False to gain advantages of data caching.
	 */
	public PendingResultWrapper LoadInvitablePlayers(int PageSize, boolean ForceReload)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Players.LoadInvitablePlayers: PageSize=" + PageSize + " ForceReload=" + ForceReload);
		PendingResult<LoadPlayersResult> PR = _Players.loadInvitablePlayers(GooglePlayConnection.getAPIclient(), PageSize, ForceReload);
		return new PendingResultWrapper(PR, LoadPlayersResult.class);
	}

	/**
	 * Asynchronously loads an additional page of invitable players.
	 * @param PageSize The number of additional entries to request. This must be a value between 1 and 25.
	 */
	public PendingResultWrapper LoadMoreInvitablePlayers(int PageSize)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Players.LoadMoreInvitablePlayers: PageSize=" + PageSize);
		PendingResult<LoadPlayersResult> PR = _Players.loadMoreInvitablePlayers(GooglePlayConnection.getAPIclient(), PageSize);
		return new PendingResultWrapper(PR, LoadPlayersResult.class);
	}

	/**
	 * Loads the initial page of players the currently signed-in player has played multiplayer games with recently, starting with the most recent.
	 * @param PageSize The number of entries to request for this initial page. Note that if cached data already exists, the returned buffer may contain more than this size, but it is guaranteed to contain at least this many if the collection contains enough records. This must be a value between 1 and 25.
	 * @param ForceReload If True, this call will clear any locally cached data and attempt to fetch the latest data from the server. This would commonly be used for something like a user-initiated refresh. Normally, this should be set to False to gain advantages of data caching.
	 */
	public PendingResultWrapper LoadRecentlyPlayedWithPlayers(int PageSize, boolean ForceReload)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Players.LoadRecentlyPlayedWithPlayers: PageSize=" + PageSize + " ForceReload=" + ForceReload);
		PendingResult<LoadPlayersResult> PR = _Players.loadRecentlyPlayedWithPlayers(GooglePlayConnection.getAPIclient(), PageSize, ForceReload);
		return new PendingResultWrapper(PR, LoadPlayersResult.class);
	}

	/**
	 * Asynchronously loads an additional page of players that the signed-in player has played multiplayer games with recently.
	 * @param PageSize The number of additional entries to request. This must be a value between 1 and 25.
	 */
	public PendingResultWrapper LoadMoreRecentlyPlayedWithPlayers(int PageSize)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Players.LoadMoreRecentlyPlayedWithPlayers: PageSize=" + PageSize);
		PendingResult<LoadPlayersResult> PR = _Players.loadMoreRecentlyPlayedWithPlayers(GooglePlayConnection.getAPIclient(), PageSize);
		return new PendingResultWrapper(PR, LoadPlayersResult.class);
	}

	/**
	 * Loads the profile for the requested player ID.
	 * @param PlayerId The player ID to get full profile data for.
	 */
	public PendingResultWrapper LoadPlayer(String PlayerId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Players.LoadPlayer: PlayerId=" + PlayerId);
		PendingResult<LoadPlayersResult> PR = _Players.loadPlayer(GooglePlayConnection.getAPIclient(), PlayerId);
		return new PendingResultWrapper(PR, LoadPlayersResult.class);
	}
}
