package flm.b4a.googleplay;

import java.util.ArrayList;

import com.google.android.gms.games.Games;
import com.google.android.gms.games.GamesStatusCodes;
import com.google.android.gms.games.multiplayer.Invitation;
import com.google.android.gms.games.multiplayer.Multiplayer;
import com.google.android.gms.games.multiplayer.realtime.RealTimeMultiplayer;
import com.google.android.gms.games.multiplayer.realtime.RealTimeMultiplayer.ReliableMessageSentCallback;
import com.google.android.gms.nearby.connection.Connections;

import android.content.Intent;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.collections.List;

@BA.ShortName("GPlayRealTimeMultiplayer")
@BA.Events(values = {
		"onConnectedToRoom(Room As GPlayRoom)",
		"onDisconnectedFromRoom(Room As GPlayRoom)",
		"onJoinedRoom(StatusCode As Int, Room As GPlayRoom)",
		"onLeftRoom(StatusCode As Int, RoomId As String)",
		"onP2PConnected(ParticipantId As String)",
		"onP2PDisconnected(ParticipantId As String)",
		"onPeerDeclined(Room As GPlayRoom, ParticipantIds As List)",
		"onPeerJoined(Room As GPlayRoom, ParticipantIds As List)",
		"onPeerLeft(Room As GPlayRoom, ParticipantIds As List)",
		"onPeersConnected(Room As GPlayRoom, ParticipantIds As List)",
		"onPeersDisconnected(Room As GPlayRoom, ParticipantIds As List)",
		"onRealTimeMessageSent(StatusCode As Int, TokenId As Int, RecipientParticipantId As String)",
		"onRealTimeMessageReceived(Message As GPlayRealTimeMessage)",
		"onRoomAutoMatching(Room As GPlayRoom)",
		"onRoomCreated(StatusCode As Int, Room As GPlayRoom)",
		"onRoomConnected(StatusCode As Int, Room As GPlayRoom)",
		"onRoomConnecting(Room As GPlayRoom)"
})
public class RealTimeMultiplayerWrapper
{
	protected RealTimeMultiplayer _RTMp = null;
	private RoomConfigWrapper _Config;

	public void Initialize()
	{
		_RTMp = Games.RealTimeMultiplayer;
	}

	public boolean IsInitialized()
	{
		return _RTMp != null;
	}

	public final static String EXTRA_INVITATION = Multiplayer.EXTRA_INVITATION;
	public final static String EXTRA_MAX_AUTOMATCH_PLAYERS = Multiplayer.EXTRA_MAX_AUTOMATCH_PLAYERS;
	public final static String EXTRA_MIN_AUTOMATCH_PLAYERS = Multiplayer.EXTRA_MIN_AUTOMATCH_PLAYERS;
	public final static String EXTRA_PLAYER_IDS = Games.EXTRA_PLAYER_IDS;
	public final static String EXTRA_ROOM = Multiplayer.EXTRA_ROOM;

	public static final int MAX_RELIABLE_MESSAGE_LEN = Connections.MAX_RELIABLE_MESSAGE_LEN;
	public static final int MAX_UNRELIABLE_MESSAGE_LEN = Connections.MAX_UNRELIABLE_MESSAGE_LEN;

	public static final int SEND_STATUS_OK = GamesStatusCodes.STATUS_OK;
	/** Return value indicating immediate failure. */
	public static final int SEND_STATUS_FAILED = RealTimeMultiplayer.REAL_TIME_MESSAGE_FAILED;

	public static final int STATUS_OK = GamesStatusCodes.STATUS_OK;
	/** An unspecified error occurred; no more specific information is available. */
	public static final int STATUS_INTERNAL_ERROR = GamesStatusCodes.STATUS_INTERNAL_ERROR;
	/** Indicates that the real-time room ID provided to the operation was not valid, or does not correspond to the currently active real-time room. */
	public static final int STATUS_INVALID_ROOM_ID = GamesStatusCodes.STATUS_INVALID_REAL_TIME_ROOM_ID;
	/** Failed to initialize the network connection for a real-time room. */
	public static final int STATUS_CONNECTION_FAILED = GamesStatusCodes.STATUS_REAL_TIME_CONNECTION_FAILED;
	/** The room is not currently active. */
	public static final int STATUS_INACTIVE_ROOM = GamesStatusCodes.STATUS_REAL_TIME_INACTIVE_ROOM;
	/** Failed to send message to the peer participant for a real-time room. */
	public static final int STATUS_MESSAGE_SEND_FAILED = GamesStatusCodes.STATUS_REAL_TIME_MESSAGE_SEND_FAILED;
	/** Failed to send message to the peer participant for a real-time room, since the user has not joined the room. */
	public static final int STATUS_ROOM_NOT_JOINED = GamesStatusCodes.STATUS_REAL_TIME_ROOM_NOT_JOINED;

	/**
	 * Creates a real-time room for the current game. The lifetime of the current game's connection to the room is bound to the Google API client's lifecycle. When the client disconnects, the player will leave the room and any peer-to-peer connections for this player will be torn down.
	 * @param Config The configuration parameters for the room to create.
	 */
	public void Create(RoomConfigWrapper Config)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RealTimeMp.Create: " + Config);
		_Config = Config;
		_RTMp.create(GooglePlayConnection.getAPIclient(), Config._RC);
	}

	/**
	 * Declines an invitation for a real-time room.
	 * @param InvitationId The ID of the invitation to decline.
	 */
	public void DeclineInvitation(String InvitationId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RealTimeMp.DeclineInvitation: " + InvitationId);
		_RTMp.declineInvitation(GooglePlayConnection.getAPIclient(), InvitationId);
	}

	/**
	 * Dismisses an invitation to a real-time room. Dismissing an invitation will not change the state of the room for the other participants.
	 * @param InvitationId The ID of the invitation to dismiss.
	 */
	public void DismissInvitation(String InvitationId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RealTimeMp.DismissInvitation: " + InvitationId);
		_RTMp.dismissInvitation(GooglePlayConnection.getAPIclient(), InvitationId);
	}

	/**
	 * Joins a real-time room by accepting an invitation. The lifetime of the current game's connection to the room is bound to the Google API client's lifecycle. When the client disconnects, the player will leave the room and any peer-to-peer connections for this player will be torn down.
	 * @param Config The configuration parameters for the room to join.
	 */
	public void Join(RoomConfigWrapper Config)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RealTimeMp.Join: " + Config);
		_Config = Config;
		_RTMp.join(GooglePlayConnection.getAPIclient(), Config._RC);
	}

	/**
	 * Leaves the specified room. This will disconnect the player from the room, but allow other players to continue playing the game. After this function is called, you cannot perform any further actions on the room.
	 * @param RoomId ID of the room to leave.
	 */
	public void Leave(String RoomId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RealTimeMp.Leave: " + RoomId);
		if (_Config.IsInitialized())
			_RTMp.leave(GooglePlayConnection.getAPIclient(), _Config._roomUpdateListener, RoomId);
	}

	/**
	 * Retrieves an invitation from the extended data (EXTRA_INVITATION) of the given intent.
	 */
	public static InvitationWrapper GetInvitationFromIntentExtra(Intent Intent)
	{
		Invitation inv = Intent.getParcelableExtra(Multiplayer.EXTRA_INVITATION);
		return Utils.InvitationWrapperOrNull(inv);
	}

	/**
	 *Returns an intent that will let the user see and manage any outstanding invitations. This intent must be invoked with StartActivityForResult.
	 *If the user canceled, the result code will be ACTIVITY_RESULT_CANCELED. If the user selected an invitation to accept, the result code will be ACTIVITY_RESULT_OK and the data intent will contain the selected invitation in the extras (EXTRA_INVITATION).
	 */
	public static Intent GetInvitationInboxIntent()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RealTimeMp.GetInvitationInboxIntent");
		return Games.Invitations.getInvitationInboxIntent(GooglePlayConnection.getAPIclient());
	}

	/**
	 *Returns an intent that will let the user select opponents to send an invitation for a real-time multiplayer match. This intent must be invoked with StartActivityForResult.
	 *The number of players passed in should be the desired number of additional players to select, not including the current player. So, for a game that can handle between 2 and 4 players, MinPlayers would be 1 and MaxPlayers would be 3.
	 *If the user canceled, the result will be ACTIVITY_RESULT_CANCELED. If the user selected players, the result code will be ACTIVITY_RESULT_OK, and the data intent will contain the selected player IDs in EXTRA_PLAYER_IDS and the minimum and maximum numbers of additional auto-match players in EXTRA_MIN_AUTOMATCH_PLAYERS and EXTRA_MAX_AUTOMATCH_PLAYERS respectively. The player IDs in EXTRA_PLAYER_IDS will include only the other players selected, not the current player.
	 *If the AllowAutomatch parameter is set to False, the UI will not display an option for selecting automatch players. Set this to False if your game does not support automatching.
	 *@param MinPlayers The minimum number of players to select (not including the current player).
	 *@param MaxPlayers The maximum number of players to select (not including the current player).
	 *@param AllowAutomatch Whether or not to display an option for selecting automatch players.
	 */
	public Intent GetSelectOpponentsIntent(int MinPlayers, int MaxPlayers, boolean AllowAutomatch)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("RealTimeMp.GetSelectOpponentsIntent: min=" + MinPlayers + " max=" + MaxPlayers);
		return _RTMp.getSelectOpponentsIntent(GooglePlayConnection.getAPIclient(), MinPlayers, MaxPlayers, AllowAutomatch);
	}

	/**
	 * Retrieves a list of player IDs from the extended data (EXTRA_PLAYER_IDS) of the given intent. Returns Null if the list is empty.
	 */
	public static Object GetPlayerIDsFromIntentExtra(Intent Intent)
	{
		// The return type is Object, not List, to allow Null value
		ArrayList<String> PlayerIDs = Intent.getStringArrayListExtra(EXTRA_PLAYER_IDS);
		if (PlayerIDs == null || PlayerIDs.isEmpty())
			return null;
		else
			return Utils.FromArrayListToB4AList_String(PlayerIDs);
	}

	/**
	 *Returns an intent that will display a "waiting room" screen that shows the progress of participants joining a real-time multiplayer room. This intent must be invoked with StartActivityForResult.
	 *If the necessary number of peers have connected and it's now OK to start the game, or if the user explicitly asked to start the game now, the activity result code will be ACTIVITY_RESULT_OK. If the user bailed out of the waiting room screen without taking any action, the result will be ACTIVITY_RESULT_CANCELED. If the user explicitly chose to leave the room, the result will be ACTIVITY_RESULT_LEFT_ROOM. If the room no longer exists or is otherwise invalid, the result will be ACTIVITY_RESULT_INVALID_ROOM.
	 *Regardless of what the result code was, the waiting room activity will return a data intent containing a Room object in EXTRA_ROOM that represents the current state of the room that you originally passed as a parameter here. Note that the returned room may be Null if the room no longer exists.
	 *@param Room The Room object to be displayed.
	 *@param MinParticipantsToStart The minimum number of participants that must be connected to the room (including the current player) for the "Start playing" menu item to become enabled.
	 */
	public Intent GetWaitingRoomIntent(RoomWrapper Room, int MinParticipantsToStart)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("RealTimeMp.GetWaitingRoomIntent: MinParticipantsToStart=" + MinParticipantsToStart);
		return _RTMp.getWaitingRoomIntent(GooglePlayConnection.getAPIclient(), Room._Room, MinParticipantsToStart);
	}

	private ReliableMessageSentCallback _RMSC;

	/**
	 * Sends a message to a participant in a real-time room reliably. Raises an onRealTimeMessageSent event to report the status of the send message operation. Returns the token ID for the message sent, or SEND_STATUS_FAILED if the message failed to send.
	 * @param RoomId ID of the room for which the message is being sent.
	 * @param RecipientParticipantId The participant ID to send the message to.
	 * @param Message The bytes of the message to be sent. This message must not exceed MAX_RELIABLE_MESSAGE_LEN bytes in length.
	 */
	public int SendReliableMessage(final BA ba, String RoomId, String RecipientParticipantId, byte[] Message)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RealTimeMp.SendReliableMessage: To=" + RecipientParticipantId);
		if (_RMSC == null) {
			_RMSC = new ReliableMessageSentCallback() {
				@Override
				public void onRealTimeMessageSent(int statusCode, int tokenId, String recipientParticipantId)
				{
					ba.raiseEvent2(this, true, GooglePlayConnection._evtPrefix + "_onrealtimemessagesent",
							false, new Object[] { statusCode, tokenId, recipientParticipantId });
				}};
		}
		return _RTMp.sendReliableMessage(GooglePlayConnection.getAPIclient(), _RMSC, Message, RoomId, RecipientParticipantId);
	}

	/**
	 * Sends a message to a participant in a real-time room. The message delivery is not reliable and will not report status after completion. Returns SEND_STATUS_OK on a successful attempt, or SEND_STATUS_FAILED if the message failed to send.
	 * @param RoomId ID of the room for which the message is being sent.
	 * @param RecipientParticipantId The participant ID to send the message to.
	 * @param Message The bytes of the message to be sent. This message must not exceed MAX_UNRELIABLE_MESSAGE_LEN bytes in length.
	 */
	public int SendUnreliableMessage(final BA ba, String RoomId, String RecipientParticipantId, byte[] Message)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RealTimeMp.SendUnreliableMessage: To=" + RecipientParticipantId);
		return _RTMp.sendUnreliableMessage(GooglePlayConnection.getAPIclient(), Message, RoomId, RecipientParticipantId);
	}

	/**
	 * Sends a message to one or more participants in a real-time room. The message delivery is not reliable and will not report status after completion. Returns SEND_STATUS_OK on a successful attempt, or SEND_STATUS_FAILED if the message failed to send.
	 * @param RoomId ID of the room for which the message is being sent.
	 * @param RecipientParticipantIds The list of participant IDs to send the message to.
	 * @param Message The bytes of the message to be sent. This message must not exceed MAX_UNRELIABLE_MESSAGE_LEN bytes in length.
	 */
	public int SendUnreliableMessage2(final BA ba, String RoomId, List RecipientParticipantIds, byte[] Message)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RealTimeMp.SendUnreliableMessage2");
		ArrayList<String> ParticipantIdList = Utils.FromB4AListToArrayList_String(RecipientParticipantIds);
		return _RTMp.sendUnreliableMessage(GooglePlayConnection.getAPIclient(), Message, RoomId, ParticipantIdList);
	}

	/**
	 * Sends a message to all participants in a real-time room, excluding the current player. The message delivery is not reliable and will not report status after completion. Returns SEND_STATUS_OK on a successful attempt, or SEND_STATUS_FAILED if the message failed to send.
	 * @param RoomId ID of the room for which the message is being sent.
	 * @param Message The bytes of the message to be sent. This message must not exceed MAX_UNRELIABLE_MESSAGE_LEN bytes in length.
	 */
	public int SendUnreliableMessageToOthers(final BA ba, String RoomId, byte[] Message)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RealTimeMp.SendUnreliableMessageToOthers");
		return _RTMp.sendUnreliableMessageToOthers(GooglePlayConnection.getAPIclient(), Message, RoomId);
	}
}
