package flm.b4a.googleplay;

import java.util.ArrayList;
import java.util.List;

import com.google.android.gms.games.multiplayer.realtime.RealTimeMessage;
import com.google.android.gms.games.multiplayer.realtime.RealTimeMessageReceivedListener;
import com.google.android.gms.games.multiplayer.realtime.Room;
import com.google.android.gms.games.multiplayer.realtime.RoomConfig;
import com.google.android.gms.games.multiplayer.realtime.RoomStatusUpdateListener;
import com.google.android.gms.games.multiplayer.realtime.RoomUpdateListener;

import android.os.Bundle;
import anywheresoftware.b4a.BA;

@BA.ShortName("GPlayRoomConfig")
public class RoomConfigWrapper
{
	protected RoomConfig _RC;
	protected RoomUpdateListener _roomUpdateListener = null;
	private RealTimeMessageReceivedListener _rtMessageReceivedListener = null;
	private RoomStatusUpdateListener _roomStatusUpdateListener = null;

	public final static int ROOM_VARIANT_DEFAULT = Room.ROOM_VARIANT_DEFAULT;

	private void Build(final BA ba, String InvitationId, ArrayList<String> PlayersIds, Bundle AutoMatchCriteria, int Variant)
	{
		if (_rtMessageReceivedListener == null) {
			_rtMessageReceivedListener = new RealTimeMessageReceivedListener()
			{
				@Override
				public void onRealTimeMessageReceived(RealTimeMessage message) {
					if (GooglePlayConnection.DEBUG) BA.Log("onRealTimeMessageReceived: " + message);
					RealTimeMessageWrapper rtm = null;
					if (message != null)
						rtm = new RealTimeMessageWrapper(message);
					ba.raiseEvent2(this, true, GooglePlayConnection._evtPrefix + "_onrealtimemessagereceived",
							false, new Object[] { rtm });
				}
			};
		}

		if (_roomUpdateListener == null) {
			_roomUpdateListener = new RoomUpdateListener()
			{
				@Override
				public void onJoinedRoom(int statusCode, Room room) {
					if (GooglePlayConnection.DEBUG) BA.Log("onJoinedRoom: " + statusCode + " room=" + room);
					RoomWrapper rw = null;
					if (room != null)
						rw = new RoomWrapper(room);
					ba.raiseEventFromDifferentThread(this, null, 0, GooglePlayConnection._evtPrefix + "_onjoinedroom",
							false, new Object[] { statusCode, rw });
				}

				@Override
				public void onLeftRoom(int statusCode, String roomId) {
					if (GooglePlayConnection.DEBUG) BA.Log("onLeftRoom: " + statusCode + " roomId=" + roomId);
					ba.raiseEventFromDifferentThread(this, null, 0, GooglePlayConnection._evtPrefix + "_onleftroom",
							false, new Object[] { statusCode, Utils.StringOrEmpty(roomId) });
				}

				@Override
				public void onRoomConnected(int statusCode, Room room) {
					if (GooglePlayConnection.DEBUG) BA.Log("onRoomConnected: " + statusCode + " room=" + room);
					RoomWrapper rw = null;
					if (room != null)
						rw = new RoomWrapper(room);
					ba.raiseEventFromDifferentThread(this, null, 0, GooglePlayConnection._evtPrefix + "_onroomconnected",
							false, new Object[] { statusCode, rw });
				}

				@Override
				public void onRoomCreated(int statusCode, Room room) {
					if (GooglePlayConnection.DEBUG) BA.Log("onRoomCreated: " + statusCode + " room=" + room);
					RoomWrapper rw = null;
					if (room != null)
						rw = new RoomWrapper(room);
					ba.raiseEventFromDifferentThread(this, null, 0, GooglePlayConnection._evtPrefix + "_onroomcreated",
							false, new Object[] { statusCode, rw });
				}
			};
		}

		if (_roomStatusUpdateListener == null) {
			_roomStatusUpdateListener = new RoomStatusUpdateListener()
			{
				@Override
				public void onConnectedToRoom(Room room) {
					if (GooglePlayConnection.DEBUG) BA.Log("onConnectedToRoom: " + room);
					RoomWrapper rw = null;
					if (room != null)
						rw = new RoomWrapper(room);
					ba.raiseEventFromDifferentThread(this, null, 0, GooglePlayConnection._evtPrefix + "_onconnectedtoroom",
							false, new Object[] { rw });
				}

				@Override
				public void onDisconnectedFromRoom(Room room) {
					if (GooglePlayConnection.DEBUG) BA.Log("onDisconnectedFromRoom: " + room);
					RoomWrapper rw = null;
					if (room != null)
						rw = new RoomWrapper(room);
					ba.raiseEvent2(this, true, GooglePlayConnection._evtPrefix + "_ondisconnectedfromroom",
							false, new Object[] { rw });
				}

				@Override
				public void onP2PConnected(String participantId) {
					if (GooglePlayConnection.DEBUG) BA.Log("onP2PConnected: " + participantId);
					ba.raiseEventFromDifferentThread(this, null, 0, GooglePlayConnection._evtPrefix + "_onp2pconnected",
							false, new Object[] { Utils.StringOrEmpty(participantId) });
				}

				@Override
				public void onP2PDisconnected(String participantId) {
					if (GooglePlayConnection.DEBUG) BA.Log("onP2PDisconnected: " + participantId);
					ba.raiseEvent2(this, true, GooglePlayConnection._evtPrefix + "_onp2pdisconnected",
							false, new Object[] { Utils.StringOrEmpty(participantId) });
				}

				@Override
				public void onPeerDeclined(Room room, List<String> participantIds) {
					if (GooglePlayConnection.DEBUG) BA.Log("onPeerDeclined: " + room);
					RoomWrapper rw = null;
					if (room != null)
						rw = new RoomWrapper(room);
					ba.raiseEvent2(this, true, GooglePlayConnection._evtPrefix + "_onpeerdeclined",
							false, new Object[] { rw, Utils.FromArrayListToB4AList_String((ArrayList<String>) participantIds) });
				}

				@Override
				public void onPeerInvitedToRoom(Room room, List<String> participantIds) {
					if (GooglePlayConnection.DEBUG) BA.Log("onPeerInvitedToRoom: " + room);
					RoomWrapper rw = null;
					if (room != null)
						rw = new RoomWrapper(room);
					ba.raiseEvent2(this, true, GooglePlayConnection._evtPrefix + "_onpeerinvitedtoroom",
							false, new Object[] { rw, Utils.FromArrayListToB4AList_String((ArrayList<String>) participantIds) });
				}

				@Override
				public void onPeerJoined(Room room, List<String> participantIds) {
					if (GooglePlayConnection.DEBUG) BA.Log("onPeerJoined: " + room);
					RoomWrapper rw = null;
					if (room != null)
						rw = new RoomWrapper(room);
					ba.raiseEventFromDifferentThread(this, null, 0, GooglePlayConnection._evtPrefix + "_onpeerjoined",
							false, new Object[] { rw, Utils.FromArrayListToB4AList_String((ArrayList<String>) participantIds) });
				}

				@Override
				public void onPeerLeft(Room room, List<String> participantIds) {
					if (GooglePlayConnection.DEBUG) BA.Log("onPeerLeft: " + room);
					RoomWrapper rw = null;
					if (room != null)
						rw = new RoomWrapper(room);
					ba.raiseEvent2(this, true, GooglePlayConnection._evtPrefix + "_onpeerleft",
							false, new Object[] { rw, Utils.FromArrayListToB4AList_String((ArrayList<String>) participantIds) });
				}

				@Override
				public void onPeersConnected(Room room, List<String> participantIds) {
					if (GooglePlayConnection.DEBUG) BA.Log("onPeersConnected: " + room);
					RoomWrapper rw = null;
					if (room != null)
						rw = new RoomWrapper(room);
					ba.raiseEventFromDifferentThread(this, null, 0, GooglePlayConnection._evtPrefix + "_onpeersconnected",
							false, new Object[] { rw, Utils.FromArrayListToB4AList_String((ArrayList<String>) participantIds) });
				}

				@Override
				public void onPeersDisconnected(Room room, List<String> participantIds) {
					if (GooglePlayConnection.DEBUG) BA.Log("onPeersDisconnected: " + room);
					RoomWrapper rw = null;
					if (room != null)
						rw = new RoomWrapper(room);
					ba.raiseEvent2(this, true, GooglePlayConnection._evtPrefix + "_onpeersdisconnected",
							false, new Object[] { rw, Utils.FromArrayListToB4AList_String((ArrayList<String>) participantIds) });
				}

				@Override
				public void onRoomAutoMatching(Room room) {
					if (GooglePlayConnection.DEBUG) BA.Log("onRoomAutoMatching: " + room);
					RoomWrapper rw = null;
					if (room != null)
						rw = new RoomWrapper(room);
					ba.raiseEvent2(this, true, GooglePlayConnection._evtPrefix + "_onroomautomatching",
							false, new Object[] { rw });
				}

				@Override
				public void onRoomConnecting(Room room) {
					if (GooglePlayConnection.DEBUG) BA.Log("onRoomConnecting: " + room);
					RoomWrapper rw = null;
					if (room != null)
						rw = new RoomWrapper(room);
					ba.raiseEventFromDifferentThread(this, null, 0, GooglePlayConnection._evtPrefix + "_onroomconnecting",
							false, new Object[] { rw });
				}
			};
		}

		RoomConfig.Builder rcBuilder = RoomConfig.builder(_roomUpdateListener) 
				.setMessageReceivedListener(_rtMessageReceivedListener)
				.setRoomStatusUpdateListener(_roomStatusUpdateListener);
		if (InvitationId != null)
			rcBuilder.setInvitationIdToAccept(InvitationId);
		if (PlayersIds != null)
			rcBuilder.addPlayersToInvite(PlayersIds);
		if (AutoMatchCriteria != null)
			rcBuilder.setAutoMatchCriteria(AutoMatchCriteria);
		if (Variant > 0)
			rcBuilder.setVariant(Variant);
		_RC = rcBuilder.build();
	}

	/**
	 * Creates a RoomConfig object.
	 * @param Invitees One or more player IDs to invite to the room.
	 */
	public void Initialize(final BA ba, Object Invitees)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RoomCfg_Initialize: " + Invitees);
		Build(ba, null, Utils.FromStringObjectToArrayList(Invitees), null, ROOM_VARIANT_DEFAULT);
	}

	/**
	 * Creates a RoomConfig object.
	 * @param Invitees One or more player IDs to invite to the room.
	 * @param Variant The developer-specified game variant.
	 */
	public void Initialize2(final BA ba, Object Invitees, int Variant)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RoomCfg_Initialize2: " + Invitees + " variant=" + Variant);
		Build(ba, null, Utils.FromStringObjectToArrayList(Invitees), null, Variant);
	}

	/**
	 * Creates a RoomConfig object.
	 * @param InvitationId The ID of the invitation to accept.
	 */
	public void InitializeWithInvitation(final BA ba, String InvitationId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RoomCfg_InitializeWithInvitation: " + InvitationId);
		Build(ba, InvitationId, null, null, ROOM_VARIANT_DEFAULT);
	}

	/**
	 * Creates a RoomConfig object.
	 * @param AutoMatchCriteria The criteria for auto-matching one or more players for the room.
	 */
	public void InitializeWithAutoMatch(final BA ba, Object AutoMatchCriteria)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RoomCfg_InitializeWithAutoMatch: " + AutoMatchCriteria);
		Build(ba, null, null, (Bundle) AutoMatchCriteria, ROOM_VARIANT_DEFAULT);
	}

	/**
	 * Creates a RoomConfig object.
	 * @param Invitees One or more player IDs to invite to the room.
	 * @param AutoMatchCriteria The criteria for auto-matching one or more players for the room.
	 */
	public void InitializeWithAutoMatch2(final BA ba, Object Invitees, Object AutoMatchCriteria)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RoomCfg_InitializeWithAutoMatch2: " + AutoMatchCriteria);
		Build(ba, null, Utils.FromStringObjectToArrayList(Invitees), (Bundle) AutoMatchCriteria, ROOM_VARIANT_DEFAULT);
	}

	/**
	 * Creates a RoomConfig object.
	 * @param Invitees One or more player IDs to invite to the room.
	 * @param AutoMatchCriteria The criteria for auto-matching one or more players for the room.
	 * @param Variant The developer-specified game variant.
	 */
	public void InitializeWithAutoMatch3(final BA ba, Object Invitees, Object AutoMatchCriteria, int Variant)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("RoomCfg_InitializeWithAutoMatch3: " + AutoMatchCriteria + " variant=" + Variant);
		Build(ba, null, Utils.FromStringObjectToArrayList(Invitees), (Bundle) AutoMatchCriteria, Variant);
	}

	public boolean IsInitialized()
	{
		return _RC != null;
	}

	/**
	 * Creates an auto-match criteria object to pass to InitializeWithAutoMatch.
	 * @param MinAutoMatchPlayers Minimum number of auto-matched players.
	 * @param MaxAutoMatchPlayers Maximum number of auto-matched players.
	 * @param ExclusiveBitMask Exclusive bitmasks for the automatching request. The logical AND of each pairing of automatching requests must equal zero for auto-match. If there are no exclusivity requirements for the game, this value should just be set to 0.
	 */
	public static Object CreateAutoMatchCriteria(int MinAutoMatchPlayers, int MaxAutoMatchPlayers, long ExclusiveBitMask)
	{
		return RoomConfig.createAutoMatchCriteria(MinAutoMatchPlayers, MaxAutoMatchPlayers, ExclusiveBitMask);
	}

	/**
	 * Retrieves the ID of the invitation to accept, if any.
	 */
	public String getInvitationId()
	{
		if (IsInitialized())
			return Utils.StringOrEmpty(_RC.getInvitationId());
		else
			return "";
	}

	/**
	 * Retrieves the player IDs to invite to the room.
	 */
	public String[] getInvitedPlayerIds()
	{
		if (IsInitialized())
			return _RC.getInvitedPlayerIds();
		else
			return null;
	}

	/**
	 * Retrieves the developer-controlled parameter describing the type of game to play. Must be either a positive integer or ROOM_VARIANT_DEFAULT if not desired.
	 */
	public int getVariant()
	{
		if (IsInitialized())
			return _RC.getVariant();
		else
			return ROOM_VARIANT_DEFAULT;
	}
}
