package flm.b4a.googleplay;

import com.google.android.gms.games.multiplayer.Multiplayer;
import com.google.android.gms.games.multiplayer.Participant;
import com.google.android.gms.games.multiplayer.realtime.Room;

import android.content.Intent;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.collections.List;

@BA.ShortName("GPlayRoom")
public class RoomWrapper
{
	protected Room _Room;

	public RoomWrapper()
	{
		_Room = null;
	}
	public RoomWrapper(Room room)
	{
		_Room = room;
	}

	/**
	 * Wraps the extended data (EXTRA_ROOM) retrieved from the given intent.
	 */
	public void WrapIntentExtra(Intent Intent)
	{
		_Room = Intent.getParcelableExtra(Multiplayer.EXTRA_ROOM);
	}

	public boolean IsInitialized()
	{
		return _Room != null;
	}

	public final static int PARTICIPANT_STATUS_DECLINED = Participant.STATUS_DECLINED;
	public final static int PARTICIPANT_STATUS_INVITED = Participant.STATUS_INVITED;
	public final static int PARTICIPANT_STATUS_JOINED = Participant.STATUS_JOINED;
	public final static int PARTICIPANT_STATUS_LEFT = Participant.STATUS_LEFT;
	public final static int ROOM_STATUS_ACTIVE = Room.ROOM_STATUS_ACTIVE;
	public final static int ROOM_STATUS_AUTO_MATCHING = Room.ROOM_STATUS_AUTO_MATCHING;
	public final static int ROOM_STATUS_CONNECTING = Room.ROOM_STATUS_CONNECTING;
	public final static int ROOM_STATUS_INVITING = Room.ROOM_STATUS_INVITING;
	public final static int ROOM_VARIANT_DEFAULT = Room.ROOM_VARIANT_DEFAULT;

	/**
	 * Retrieves the automatch criteria used to create or join this room, if any. May be null if the room has no automatch properties.
	 */
	public Object getAutoMatchCriteria()
	{
		return _Room.getAutoMatchCriteria();
	}

	/**
	 * Retrieves the estimated wait time for automatching to finish for players who are not automatched immediately, as measured from the time that the room entered the automatching pool.
	 *Returns -1 if the room is not automatching or no estimate could be provided.
	 */
	public int getAutoMatchWaitEstimateSeconds()
	{
		return _Room.getAutoMatchWaitEstimateSeconds();
	}

	/**
	 * Returns the server timestamp at which the room was created.
	 */
	public long getCreationTimestamp()
	{
		return _Room.getCreationTimestamp();
	}

	/**
	 * Returns the ID of the participant who created this Room. Note that not all participants will see the same value for the creator. In the case of an automatch, this value may differ for each participant.
	 */
	public String getCreatorId()
	{
		return Utils.StringOrEmpty(_Room.getCreatorId());
	}

	/**
	 * Returns the description of this room.
	 */
	public String getDescription()
	{
		return Utils.StringOrEmpty(_Room.getDescription());
	}

	/**
	 * Gets a participant in a room by its ID. Note that the participant ID must correspond to a participant in this match, or this method will throw an exception.
	 */
	public ParticipantWrapper GetParticipant(String ParticipantId)
	{
		Participant participant = _Room.getParticipant(ParticipantId); 
		if (participant == null)
			return null;
		else
			return new ParticipantWrapper(participant);	
	}

	/**
	 * Gets the participant ID for a given player. This will only return a non-empty ID if the player is actually a participant in the room and that player's identity is visible to the current player. Note that this will never be empty for the current player.
	 */
	public String GetParticipantId(String PlayerId)
	{
		return Utils.StringOrEmpty(_Room.getParticipantId(PlayerId));
	}

	/**
	 * Returns the IDs of the participants in this room. These are returned in the participant order of the room. Note that these are not stable across rooms.
	 */
	public List GetParticipantIds()
	{
		return Utils.FromArrayListToB4AList_String(_Room.getParticipantIds());
	}

	/**
	 * Retrieves the Participants in this room.
	 */
	public List GetParticipants()
	{
		return Utils.FromArrayListToB4AList_Participant(_Room.getParticipants());
	}

	/**
	 * Gets the status of a participant in a room (one of the PARTICIPANT_STATUS constants). Note that the participant ID must correspond to a participant in this room, or this method will throw an exception.
	 */
	public int GetParticipantStatus(String ParticipantId)
	{
		return _Room.getParticipantStatus(ParticipantId);
	}

	/**
	 * Returns the ID of this Room.
	 */
	public String getRoomId()
	{
		return Utils.StringOrEmpty(_Room.getRoomId());
	}

	/**
	 * Returns the current status of the room (one of the ROOM_STATUS constants).
	 */
	public int getStatus()
	{
		return _Room.getStatus();
	}

	/**
	 * Retrieves the variant specified for this room, if any. A variant is an optional developer-controlled parameter describing the type of game to play. If specified, this value will be a positive integer. If this room had no variant specified, returns ROOM_VARIANT_DEFAULT.
	 */
	public int getVariant()
	{
		return _Room.getVariant();
	}
}
