package flm.b4a.googleplay;

import anywheresoftware.b4a.BA.ShortName;

import com.google.android.gms.games.leaderboard.LeaderboardVariant;
import com.google.android.gms.games.leaderboard.ScoreSubmissionData;

@ShortName("GPlayScoreSubmissionData")
public class ScoreSubmissionDataWrapper
{
	protected ScoreSubmissionData _SSD = null;

	public ScoreSubmissionDataWrapper(ScoreSubmissionData SSD)
	{
		_SSD = SSD;
	}

	public boolean IsInitialized()
	{
		return _SSD != null;
	}

	public final static int TIME_SPAN_ALL_TIME = LeaderboardVariant.TIME_SPAN_ALL_TIME;
	public final static int TIME_SPAN_DAILY = LeaderboardVariant.TIME_SPAN_DAILY;
	public final static int TIME_SPAN_WEEKLY = LeaderboardVariant.TIME_SPAN_WEEKLY;

	/**
	 * Retrieves the ID of the leaderboard the score was submitted to.
	 */
	public String getLeaderboardId()
	{
		return Utils.StringOrEmpty(_SSD.getLeaderboardId());
	}

	/**
	 * Retrieves the ID of the player the score was submitted for.
	 */
	public String getPlayerId()
	{
		return Utils.StringOrEmpty(_SSD.getPlayerId());
	}

	/**
	 * Retrieves the score data in a display-appropriate format.
	 * @param TimeSpan Time span to retrieve result for. Valid values are TIME_SPAN_DAILY, TIME_SPAN_WEEKLY, or TIME_SPAN_ALL_TIME.
	 */
	public String GetFormattedScore(int TimeSpan)
	{
		return Utils.StringOrEmpty(_SSD.getScoreResult(TimeSpan).formattedScore);
	}

	/**
	 * Retrieves the score tag, if any.
	 * @param TimeSpan Time span to retrieve result for. Valid values are TIME_SPAN_DAILY, TIME_SPAN_WEEKLY, or TIME_SPAN_ALL_TIME.
	 */
	public String GetScoreTag(int TimeSpan)
	{
		return Utils.StringOrEmpty(_SSD.getScoreResult(TimeSpan).scoreTag);
	}

	/**
	 * Indicates whether or not this score was the player's new best score for this time span.
	 * @param TimeSpan Time span to retrieve result for. Valid values are TIME_SPAN_DAILY, TIME_SPAN_WEEKLY, or TIME_SPAN_ALL_TIME.
	 */
	public boolean IsNewBest(int TimeSpan)
	{
		return _SSD.getScoreResult(TimeSpan).newBest;
	}

	/**
	 * Retrieves the raw score value.
	 * @param TimeSpan Time span to retrieve result for. Valid values are TIME_SPAN_DAILY, TIME_SPAN_WEEKLY, or TIME_SPAN_ALL_TIME.
	 */
	public long GetRawScore(int TimeSpan)
	{
		return _SSD.getScoreResult(TimeSpan).rawScore;
	}
}
