package flm.b4a.googleplay;

import android.net.Uri;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.objects.ImageViewWrapper;

import com.google.android.gms.games.snapshot.SnapshotMetadata;

@ShortName("GPlaySnapshotMetadata")
@BA.Events(values = {
		"onImageLoaded(URI As String, Drawable As BitmapDrawable)"
})
public class SnapshotMetadataWrapper
{
	protected SnapshotMetadata _SM = null;

	public SnapshotMetadataWrapper(SnapshotMetadata SM)
	{
		_SM = SM;
	}

	public boolean IsInitialized()
	{
		return _SM != null;
	}

	public static final long PLAYED_TIME_UNKNOWN = SnapshotMetadata.PLAYED_TIME_UNKNOWN;
	public static final long PROGRESS_VALUE_UNKNOWN = SnapshotMetadata.PROGRESS_VALUE_UNKNOWN;

	/**
	 * Retrieves the aspect ratio of the cover image for this snapshot, if any. This is the ratio of width to height, so a value &gt; 1 indicates a landscape image while a value &lt; 1 indicates a portrait image. If the snapshot has no cover image, this will return 0.
	 */
	public float getCoverImageAspectRatio()
	{
		return _SM.getCoverImageAspectRatio();
	}

	/**
	 *Loads the snapshot's cover image and fills the given ImageView.
	 *Returns the URI of the image, or an empty string if the snapshot has no cover image.
	 */
	public String GetCoverImage(final BA ba, ImageViewWrapper ImageView)
	{
		Uri uri = _SM.getCoverImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("SnapshotMetadata.GetCoverImage: uri=" + uri);
		Utils.LoadImage(ba, uri, ImageView);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 *Loads the snapshot's cover image and raises an onImageLoaded event when the load is complete.
	 *Returns the URI of the image, or an empty string if the snapshot has no cover image.
	 */
	public String GetCoverImage2(final BA ba)
	{
		Uri uri = _SM.getCoverImageUri();
		if (GooglePlayConnection.DEBUG) BA.Log("SnapshotMetadata.GetCoverImage2: uri=" + uri);
		Utils.LoadImage(ba, uri, null);
		return Utils.UriStringOrEmpty(uri);
	}

	/**
	 * Retrieves the description of this snapshot.
	 */
	public String getDescription()
	{
		return Utils.StringOrEmpty(_SM.getDescription());
	}

	/**
	 * Retrieves the name of the device that wrote this snapshot, if known.
	 */
	public String getDeviceName()
	{
		return Utils.StringOrEmpty(_SM.getDeviceName());
	}

	/**
	 * Retrieves the game associated with this snapshot.
	 */
	public GameWrapper getGame()
	{
		return Utils.GameWrapperOrNull(_SM.getGame());
	}

	/**
	 * Retrieves the last time this snapshot was modified, in milliseconds since epoch.
	 */
	public long getLastModifiedTimestamp()
	{
		return _SM.getLastModifiedTimestamp();
	}

	/**
	 * Retrieves the player that owns this snapshot.
	 */
	public PlayerWrapper getOwner()
	{
		return Utils.PlayerWrapperOrNull(_SM.getOwner());
	}

	/**
	 * Retrieves the played time of this snapshot in milliseconds. This value is specified during the update operation. If not known, returns PLAYED_TIME_UNKNOWN.
	 */
	public long getPlayedTime()
	{
		return _SM.getPlayedTime();
	}

	/**
	 * Retrieves the progress value for this snapshot. If not known, returns PROGRESS_VALUE_UNKNOWN.
	 */
	public long getProgressValue()
	{
		return _SM.getProgressValue();
	}

	/**
	 *Retrieves the unique identifier of this snapshot. This value can be passed to GPlaySnapshots.Open to open the snapshot for modification.
	 *This name should be unique within the scope of the application.
	 */
	public String getUniqueName()
	{
		return Utils.StringOrEmpty(_SM.getUniqueName());
	}

	/**
	 * Indicates whether or not this snapshot has any changes pending that have not been uploaded to the server. Once all changes have been flushed to the server, this will return False.
	 */
	public boolean HasChangePending()
	{
		return _SM.hasChangePending();
	}
}
