package flm.b4a.googleplay;

import android.content.Intent;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ShortName;

import com.google.android.gms.common.api.PendingResult;
import com.google.android.gms.games.Games;
import com.google.android.gms.games.GamesStatusCodes;
import com.google.android.gms.games.snapshot.SnapshotMetadata;
import com.google.android.gms.games.snapshot.SnapshotMetadataChange;
import com.google.android.gms.games.snapshot.Snapshots;
import com.google.android.gms.games.snapshot.Snapshots.CommitSnapshotResult;
import com.google.android.gms.games.snapshot.Snapshots.DeleteSnapshotResult;
import com.google.android.gms.games.snapshot.Snapshots.LoadSnapshotsResult;
import com.google.android.gms.games.snapshot.Snapshots.OpenSnapshotResult;

@ShortName("GPlaySnapshots")
public class SnapshotsWrapper
{
	protected Snapshots _SnpSh;

	public SnapshotsWrapper()
	{
		_SnpSh = null;
	}

	public void Initialize()
	{
		_SnpSh = Games.Snapshots;
	}

	public boolean IsInitialized()
	{
		return _SnpSh != null;
	}

	public static final int DISPLAY_LIMIT_NONE = Snapshots.DISPLAY_LIMIT_NONE;
	public static final String EXTRA_SNAPSHOT_METADATA = Snapshots.EXTRA_SNAPSHOT_METADATA;
	public static final String EXTRA_SNAPSHOT_NEW = Snapshots.EXTRA_SNAPSHOT_NEW;
	public static final String RESULTMAP_SNAPSHOT = PendingResultWrapper.RESULTMAP_SNAPSHOT;
	public static final String RESULTMAP_SNAPSHOT_CONFLICT_ID = PendingResultWrapper.RESULTMAP_SNAPSHOT_CONFLICT_ID;
	public static final String RESULTMAP_SNAPSHOT_CONFLICTING_WITH = PendingResultWrapper.RESULTMAP_SNAPSHOT_CONFLICTING_WITH;
	public static final String RESULTMAP_SNAPSHOT_ID = PendingResultWrapper.RESULTMAP_SNAPSHOT_ID;
	public static final String RESULTMAP_SNAPSHOT_METADATA = PendingResultWrapper.RESULTMAP_SNAPSHOT_METADATA;
	public static final String RESULTMAP_SNAPSHOTS = PendingResultWrapper.RESULTMAP_SNAPSHOTS;
	public static final String RESULTMAP_STATUS_CODE = PendingResultWrapper.RESULTMAP_STATUS_CODE;
	public static final Object SMC_EMPTY_CHANGE = null;
	public static final int STATUS_OK = GamesStatusCodes.STATUS_OK;
	public static final int STATUS_SNAPSHOT_COMMIT_FAILED = GamesStatusCodes.STATUS_SNAPSHOT_COMMIT_FAILED;
	public static final int STATUS_SNAPSHOT_CONFLICT = GamesStatusCodes.STATUS_SNAPSHOT_CONFLICT;
	public static final int STATUS_SNAPSHOT_CONTENTS_UNAVAILABLE = GamesStatusCodes.STATUS_SNAPSHOT_CONTENTS_UNAVAILABLE;
	public static final int STATUS_SNAPSHOT_CREATION_FAILED = GamesStatusCodes.STATUS_SNAPSHOT_CREATION_FAILED;
	public static final int STATUS_SNAPSHOT_FOLDER_UNAVAILABLE = GamesStatusCodes.STATUS_SNAPSHOT_FOLDER_UNAVAILABLE;
	public static final int STATUS_SNAPSHOT_NOT_FOUND = GamesStatusCodes.STATUS_SNAPSHOT_NOT_FOUND;

	/** In the case of a conflict, the snapshot with the highest progress value will be used. */
	public final static int RESOLUTION_POLICY_HIGHEST_PROGRESS = Snapshots.RESOLUTION_POLICY_HIGHEST_PROGRESS;
	/** In the case of a conflict, the last known good version of this snapshot will be used. */
	public final static int RESOLUTION_POLICY_LAST_KNOWN_GOOD = Snapshots.RESOLUTION_POLICY_LAST_KNOWN_GOOD;
	/** In the case of a conflict, the snapshot with the longest played time will be used. */
	public final static int RESOLUTION_POLICY_LONGEST_PLAYTIME = Snapshots.RESOLUTION_POLICY_LONGEST_PLAYTIME;
	/** In the case of a conflict, the result will be returned to the app for resolution. */
	public final static int RESOLUTION_POLICY_MANUAL = Snapshots.RESOLUTION_POLICY_MANUAL;
	/** In the case of a conflict, the most recently modified version of this snapshot will be used. */
	public final static int RESOLUTION_POLICY_MOST_RECENTLY_MODIFIED = Snapshots.RESOLUTION_POLICY_MOST_RECENTLY_MODIFIED;

	/**
	 *Commits any modifications made to the snapshot. This will cause the changes to be synced to the server in the background.
	 *Calling this function with a snapshot that has already been committed or that was not opened via Open will throw an exception.
	 *Note that the total size of the contents of snapshot may not exceed the size provided by MaxDataSize.
	 *@param Snapshot The snapshot to commit the data for.
	 *@param MetadataChange The set of changes to apply to the metadata for the snapshot. Use SMC_EMPTY_CHANGE to preserve the existing metadata.
	 */
	public PendingResultWrapper CommitAndClose(SnapshotWrapper Snapshot, SnapshotMetadataChangeWrapper MetadataChange)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Snapshots.CommitAndClose");
		SnapshotMetadataChange SMC;
		if (MetadataChange == null)
			SMC = SnapshotMetadataChange.EMPTY_CHANGE;
		else
			SMC = MetadataChange._SMC;
		PendingResult<CommitSnapshotResult> PR = _SnpSh.commitAndClose(GooglePlayConnection.getAPIclient(), Snapshot._SnSh, SMC);
		return new PendingResultWrapper(PR, CommitSnapshotResult.class);
	}

	/**
	 * Deletes the specified snapshot. This will delete the data of the snapshot locally and on the server. 
	 * @param Metadata The metadata of the snapshot to delete.
	 */
	public PendingResultWrapper Delete(SnapshotMetadataWrapper Metadata)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Snapshots.Delete");
		PendingResult<DeleteSnapshotResult> PR = _SnpSh.delete(GooglePlayConnection.getAPIclient(), Metadata._SM);
		return new PendingResultWrapper(PR, DeleteSnapshotResult.class);
	}

	/**
	 *Discards the contents of the snapshot and close the contents. This will discard all changes made to the file, and close the snapshot to future changes until it is re-opened. The file will not be modified on the server.
	 *Calling this function with a snapshot that has already been committed or that was not opened via Open will throw an exception.
	 *@param Snapshot The snapshot to discard the data for.
	 */
	public void DiscardAndClose(SnapshotWrapper Snapshot)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Snapshots.DiscardAndClose");
		_SnpSh.discardAndClose(GooglePlayConnection.getAPIclient(), Snapshot._SnSh);
	}

	/**
	 *Gets the maximum data size per snapshot cover image in bytes. Guaranteed to be at least 800 KB. May increase in the future.
	 *If the service cannot be reached for some reason, this will return -1.
	 */
	public int getMaxCoverImageSize()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Snapshots.MaxCoverImageSize=" + _SnpSh.getMaxCoverImageSize(GooglePlayConnection.getAPIclient()));
		return _SnpSh.getMaxCoverImageSize(GooglePlayConnection.getAPIclient());
	}

	/**
	 *Gets the maximum data size per snapshot in bytes. Guaranteed to be at least 3 MB. May increase in the future.
	 *If the service cannot be reached for some reason, this will return -1.
	 */
	public int getMaxDataSize()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Snapshots.MaxDataSize=" + _SnpSh.getMaxDataSize(GooglePlayConnection.getAPIclient()));
		return _SnpSh.getMaxDataSize(GooglePlayConnection.getAPIclient());
	}

	/**
	 *Returns an intent that will let the user select a snapshot. This intent must be invoked with StartActivityForResult.
	 *If the user canceled without selecting a snapshot, the result code will be ACTIVITY_RESULT_CANCELED. If the user selected a snapshot from the list, the result code will be ACTIVITY_RESULT_OK and the data intent will contain the metadata of the selected snapshot in EXTRA_SNAPSHOT_METADATA. If the user pressed the Create button, the result code will be ACTIVITY_RESULT_OK and the data intent will contain a True boolean value in EXTRA_SNAPSHOT_NEW.
	 *Note that if you have modified an open snapshot, the changes will not appear in this UI until you call CommitAndClose on the snapshot.
	 *@param Title The title to display in the action bar of the returned Activity.
	 *@param AllowAddButton Whether or not to display a "create new snapshot" option in the selection UI.
	 *@param AllowDelete Whether or not to provide a delete overflow menu option for each snapshot in the selection UI.
	 *@param MaxSnapshots The maximum number of snapshots to display in the UI. Use DISPLAY_LIMIT_NONE to display all snapshots.
	 */
	public Intent GetSelectSnapshotIntent(String Title, boolean AllowAddButton, boolean AllowDelete, int MaxSnapshots)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Snapshots.GetSelectSnapshotIntent: " + Title);
		return _SnpSh.getSelectSnapshotIntent(GooglePlayConnection.getAPIclient(), Title, AllowAddButton, AllowDelete, MaxSnapshots);
	}

	/**
	 * Retrieves the metadata of a snapshot from the extended data (EXTRA_SNAPSHOT_METADATA) of the given intent.
	 */
	public SnapshotMetadataWrapper GetSnapshotMetadataFromIntentExtra(Intent Intent)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Snapshots.GetSnapshotMetadataFromIntentExtra");
		SnapshotMetadata SM = Intent.getParcelableExtra(Snapshots.EXTRA_SNAPSHOT_METADATA);
		return Utils.MetadataWrapperOrNull(SM);
	}

	/**
	 * Asynchronously loads the list of snapshot metadata for the currently signed-in player.
	 * @param ForceReload If True, this call will clear any locally cached data and attempt to fetch the latest data from the server. This would commonly be used for something like a user-initiated refresh. Normally, this should be set to False to gain advantages of data caching.
	 */
	public PendingResultWrapper Load(boolean ForceReload)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Snapshots.Load");
		PendingResult<LoadSnapshotsResult> PR = _SnpSh.load(GooglePlayConnection.getAPIclient(), ForceReload);
		return new PendingResultWrapper(PR, LoadSnapshotsResult.class);
	}

	/**
	 * Opens a snapshot with the given name.
	 * @param FileName The name of the snapshot file to open. Must be between 1 and 100 non-URL-reserved characters (a-z, A-Z, 0-9, or the symbols "-", ".", "_", or "~").
	 * @param CreateIfNotFound If True, the snapshot will be created if one cannot be found.
	 */
	public PendingResultWrapper OpenWithName(String FileName, boolean CreateIfNotFound)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Snapshots.OpenWithName: FileName=" + FileName);
		PendingResult<OpenSnapshotResult> PR = _SnpSh.open(GooglePlayConnection.getAPIclient(), FileName, CreateIfNotFound);
		return new PendingResultWrapper(PR, OpenSnapshotResult.class);
	}

	/**
	 * Opens a snapshot with the given name.
	 * @param FileName The name of the snapshot file to open. Must be between 1 and 100 non-URL-reserved characters (a-z, A-Z, 0-9, or the symbols "-", ".", "_", or "~").
	 * @param CreateIfNotFound If True, the snapshot will be created if one cannot be found.
	 * @param ConflictPolicy The conflict resolution policy to use for this snapshot (one of the RESOLUTION_POLICY constants).
	 */
	public PendingResultWrapper OpenWithName2(String FileName, boolean CreateIfNotFound, int ConflictPolicy)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Snapshots.OpenWithName2: FileName=" + FileName);
		PendingResult<OpenSnapshotResult> PR = _SnpSh.open(GooglePlayConnection.getAPIclient(), FileName, CreateIfNotFound, ConflictPolicy);
		return new PendingResultWrapper(PR, OpenSnapshotResult.class);
	}

	/**
	 * Opens a snapshot with the given metadata (usually returned from Load or GetSnapshotMetadataFromIntentExtra). To succeed, the snapshot must exist; i.e. this call will fail if the snapshot was deleted between Load and Open.
	 * @param Metadata The metadata of the existing snapshot to load.
	 */
	public PendingResultWrapper Open(SnapshotMetadataWrapper Metadata)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Snapshots.Open: UniqueName=" + Metadata.getUniqueName());
		PendingResult<OpenSnapshotResult> PR = _SnpSh.open(GooglePlayConnection.getAPIclient(), Metadata._SM);
		return new PendingResultWrapper(PR, OpenSnapshotResult.class);
	}

	/**
	 * Opens a snapshot with the given metadata (usually returned from Load or GetSnapshotMetadataFromIntentExtra). To succeed, the snapshot must exist; i.e. this call will fail if the snapshot was deleted between Load and Open.
	 * @param Metadata The metadata of the existing snapshot to load.
	 * @param ConflictPolicy The conflict resolution policy to use for this snapshot (one of the RESOLUTION_POLICY constants).
	 */
	public PendingResultWrapper Open2(SnapshotMetadataWrapper Metadata, int ConflictPolicy)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Snapshots.Open2: UniqueName=" + Metadata.getUniqueName());
		PendingResult<OpenSnapshotResult> PR = _SnpSh.open(GooglePlayConnection.getAPIclient(), Metadata._SM, ConflictPolicy);
		return new PendingResultWrapper(PR, OpenSnapshotResult.class);
	}

	/**
	 *Resolves a conflict using the data from the provided snapshot. This will replace the data on the server with the specified snapshot. Note that it is possible for this operation to result in a conflict itself, in which case resolution should be repeated.
	 *Note that the total size of the contents of snapshot may not exceed the size provided by MaxDataSize.
	 *Calling this function with a snapshot that has already been committed or that was not opened via Open will throw an exception.
	 *@param ConflictId The ID of the conflict to resolve.
	 *@param Snapshot The snapshot to use to resolve the conflict.
	 */
	public PendingResultWrapper ResolveConflict(String ConflictId, SnapshotWrapper Snapshot)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("Snapshots.ResolveConflict: ConflictId=" + ConflictId);
		PendingResult<OpenSnapshotResult> PR = _SnpSh.resolveConflict(GooglePlayConnection.getAPIclient(), ConflictId, Snapshot._SnSh);
		return new PendingResultWrapper(PR, OpenSnapshotResult.class);
	}
}
