package flm.b4a.googleplay;

import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMatch;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMatchConfig;

import android.os.Bundle;
import anywheresoftware.b4a.BA;

@BA.ShortName("GPlayTurnBasedMatchConfig")
public class TurnBasedMatchConfigWrapper
{
	protected TurnBasedMatchConfig _TBMC;

	public final static int MATCH_VARIANT_DEFAULT = TurnBasedMatch.MATCH_VARIANT_DEFAULT;

	/**
	 * Creates a TurnBasedMatchConfig object.
	 * @param Invitees One or more player IDs to invite to the match. Can be a string array, a List or the extended data (EXTRA_PLAYER_IDS) of an intent.
	 */
	public void Initialize(Object Invitees)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMCfg.Initialize: " + Invitees);
		_TBMC = TurnBasedMatchConfig.builder()
				.addInvitedPlayers(Utils.FromStringObjectToArrayList(Invitees))
				.build();
	}

	/**
	 * Creates a TurnBasedMatchConfig object with a Variant parameter set to MATCH_VARIANT_DEFAULT.
	 * @param Invitees One or more player IDs to invite to the match. Can be a string array, a List or the extended data (EXTRA_PLAYER_IDS) of an intent.
	 * @param AutoMatchCriteria The criteria for auto-matching one or more players for the match.
	 */
	public void InitializeWithAutoMatch(Object Invitees, Object AutoMatchCriteria)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("TBMCfg.InitializeWithAutoMatch: " + Invitees + " criteria=" + AutoMatchCriteria.toString());
		if (Invitees == null)
			_TBMC = TurnBasedMatchConfig.builder()
			.setAutoMatchCriteria((Bundle) AutoMatchCriteria)
			.build();
		else
			_TBMC = TurnBasedMatchConfig.builder()
			.addInvitedPlayers(Utils.FromStringObjectToArrayList(Invitees))
			.setAutoMatchCriteria((Bundle) AutoMatchCriteria)
			.build();
	}

	/**
	 * Creates a TurnBasedMatchConfig object.
	 * @param Invitees One or more player IDs to invite to the match. Can be a string array, a List or an object returned by Intent.GetExtra(EXTRA_PLAYERS).
	 * @param AutoMatchCriteria The criteria for auto-matching one or more players for the match.
	 * @param Variant Developer-controlled parameter describing the type of game to play. Must be either a positive integer, or MATCH_VARIANT_DEFAULT if not desired. Note that variants must match exactly. Thus, if you do not specify a variant, only other matches created with MATCH_VARIANT_DEFAULT will be considered potential auto-matches.
	 */
	public void InitializeWithAutoMatch2(Object Invitees, Object AutoMatchCriteria, int Variant)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("TBMCfg.InitializeWithAutoMatch2: " + Invitees + " criteria=" + AutoMatchCriteria.toString() + " variant=" + Variant);
		if (Invitees == null)
			_TBMC = TurnBasedMatchConfig.builder()
			.setAutoMatchCriteria((Bundle) AutoMatchCriteria)
			.setVariant(Variant)
			.build();
		else
			_TBMC = TurnBasedMatchConfig.builder()
			.addInvitedPlayers(Utils.FromStringObjectToArrayList(Invitees))
			.setAutoMatchCriteria((Bundle) AutoMatchCriteria)
			.setVariant(Variant)
			.build();
	}

	public boolean IsInitialized()
	{
		return _TBMC != null;
	}

	/**
	 * Gets the player IDs to invite to the match.
	 */
	public String[] getInvitedPlayerIds()
	{
		return _TBMC.getInvitedPlayerIds();
	}

	/**
	 * Retrieves the developer-specified match variant.
	 */
	public int getVariant()
	{
		return _TBMC.getVariant();
	}

	/**
	 * Creates an auto-match criteria object to pass to InitializeWithAutoMatch.
	 * @param MinAutoMatchPlayers Min number of auto-matched players.
	 * @param MaxAutoMatchPlayers Max number of auto-matched players.
	 * @param ExclusiveBitMask Exclusive bitmask for exclusive roles for the player. The exclusive bitmask of each pairing of players must equal zero for auto-match.
	 */
	public static Object CreateAutoMatchCriteria(int MinAutoMatchPlayers, int MaxAutoMatchPlayers, long ExclusiveBitMask)
	{
		return TurnBasedMatchConfig.createAutoMatchCriteria(MinAutoMatchPlayers, MaxAutoMatchPlayers, ExclusiveBitMask);
	}
}
