package flm.b4a.googleplay;

import com.google.android.gms.games.multiplayer.Participant;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMatch;

import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.collections.List;

@BA.ShortName("GPlayTurnBasedMatch")
public class TurnBasedMatchWrapper
{
	protected TurnBasedMatch _TBM;

	public TurnBasedMatchWrapper()
	{
		_TBM = null;
	}

	public TurnBasedMatchWrapper(TurnBasedMatch match)
	{
		_TBM = match;
	}

	public boolean IsInitialized()
	{
		return _TBM != null;
	}

	public final static int[] MATCH_TURN_STATUS_ALL = TurnBasedMatch.MATCH_TURN_STATUS_ALL;
	public final static int MATCH_STATUS_ACTIVE = TurnBasedMatch.MATCH_STATUS_ACTIVE;
	public final static int MATCH_STATUS_AUTO_MATCHING = TurnBasedMatch.MATCH_STATUS_AUTO_MATCHING;
	public final static int MATCH_STATUS_CANCELED = TurnBasedMatch.MATCH_STATUS_CANCELED;
	public final static int MATCH_STATUS_COMPLETE = TurnBasedMatch.MATCH_STATUS_COMPLETE;
	public final static int MATCH_STATUS_EXPIRED = TurnBasedMatch.MATCH_STATUS_EXPIRED;
	public final static int MATCH_TURN_STATUS_COMPLETE = TurnBasedMatch.MATCH_TURN_STATUS_COMPLETE;
	public final static int MATCH_TURN_STATUS_INVITED = TurnBasedMatch.MATCH_TURN_STATUS_INVITED;
	public final static int MATCH_TURN_STATUS_MY_TURN = TurnBasedMatch.MATCH_TURN_STATUS_MY_TURN;
	public final static int MATCH_TURN_STATUS_THEIR_TURN = TurnBasedMatch.MATCH_TURN_STATUS_THEIR_TURN;
	public final static int MATCH_VARIANT_DEFAULT = TurnBasedMatch.MATCH_VARIANT_DEFAULT;
	public final static int PARTICIPANT_STATUS_DECLINED = Participant.STATUS_DECLINED;
	public final static int PARTICIPANT_STATUS_FINISHED = Participant.STATUS_FINISHED;
	public final static int PARTICIPANT_STATUS_INVITED = Participant.STATUS_INVITED;
	public final static int PARTICIPANT_STATUS_JOINED = Participant.STATUS_JOINED;
	public final static int PARTICIPANT_STATUS_LEFT = Participant.STATUS_LEFT;
	public final static int PARTICIPANT_STATUS_NOT_INVITED_YET = Participant.STATUS_NOT_INVITED_YET;
	public final static int PARTICIPANT_STATUS_UNRESPONSIVE = Participant.STATUS_UNRESPONSIVE;

	/**
	 * Returns whether or not this match can be rematched. This will return True when the match has complete and has not already been rematched.
	 */
	public boolean CanRematch()
	{
		return _TBM.canRematch();
	}

	/**
	 * Retrieves the automatch criteria used to create or join this match, if any. May be null if the match has no automatch properties.
	 */
	public Object getAutoMatchCriteria()
	{
		return _TBM.getAutoMatchCriteria();
	}

	/**
	 * Returns the maximum number of available automatch slots for this match. If automatch criteria were not specified during match creation, or if all slots have been filled, this will return 0.
	 */
	public int getAvailableAutoMatchSlots()
	{
		return _TBM.getAvailableAutoMatchSlots();
	}

	/**
	 * Returns the server timestamp at which the match was created.
	 */
	public long getCreationTimestamp()
	{
		return _TBM.getCreationTimestamp();
	}

	/**
	 * Returns the ID of the participant who created this match.
	 */
	public String getCreatorId()
	{
		return Utils.StringOrEmpty(_TBM.getCreatorId());
	}

	/**
	 * Returns a byte array representing the current (game-specific) match state data.
	 */
	public byte[] getData()
	{
		return _TBM.getData();
	}

	/**
	 * Returns the description of this match.
	 */
	public String getDescription()
	{
		return Utils.StringOrEmpty(_TBM.getDescription());
	}

	/**
	 * Returns the participant representing the primary opponent in the match.
	 */
	public ParticipantWrapper getDescriptionParticipant()
	{
		return Utils.ParticipantWrapperOrNull(_TBM.getDescriptionParticipant());
	}

	/**
	 * Gets the ID of another participant in the match that can be used when describing the participants the user is playing with. For example, in a four player match this might be used to state "Martha (and 2 others)".
	 */
	public String getDescriptionParticipantId()
	{
		return Utils.StringOrEmpty(_TBM.getDescriptionParticipantId());
	}

	/**
	 * Returns the Game object that owns this match. Note that this should not be cached separately from the match itself, since the data underlying this object may change.
	 */
	public GameWrapper getGame()
	{
		return Utils.GameWrapperOrNull(_TBM.getGame());
	}

	/**
	 * Returns the timestamp at which the match was last modified.
	 */
	public long getLastUpdatedTimestamp()
	{
		return _TBM.getLastUpdatedTimestamp();
	}

	/**
	 * Returns the ID of the participant who updated the match most recently.
	 */
	public String getLastUpdaterId()
	{
		return Utils.StringOrEmpty(_TBM.getLastUpdaterId());
	}

	/**
	 * Retrieves the ID of this match.
	 */
	public String getMatchId()
	{
		return Utils.StringOrEmpty(_TBM.getMatchId());
	}

	/**
	 * Returns the match sequence number for this match. This number starts at 1, and increases every time a rematch is created.
	 */
	public int getMatchNumber()
	{
		return _TBM.getMatchNumber();
	}

	/**
	 * Gets a participant in a match by its ID. Note that the participant ID must correspond to a participant in this match, or this method will throw an exception.
	 */
	public ParticipantWrapper GetParticipant(String ParticipantId)
	{
		return Utils.ParticipantWrapperOrNull(_TBM.getParticipant(ParticipantId));
	}

	/**
	 * Gets the participant ID for a given player. This will only return a non-empty ID if the player is actually a participant in the match and that player's identity is visible to the current player. Note that this will never be empty for the current player.
	 */
	public String GetParticipantId(String PlayerId)
	{
		return Utils.StringOrEmpty(_TBM.getParticipantId(PlayerId));
	}

	/**
	 * Returns the IDs of the participants in this match. These are returned in the participant order of the match. Note that these are not stable across matches.
	 */
	public List GetParticipantIds()
	{
		return Utils.FromArrayListToB4AList_String(_TBM.getParticipantIds());
	}

	/**
	 * Retrieves the participants in this match.
	 */
	public List GetParticipants()
	{
		return Utils.FromArrayListToB4AList_Participant(_TBM.getParticipants());
	}

	/**
	 * Gets the status of a participant in a match (one of the PARTICIPANT_STATUS constants). Note that the participant ID must correspond to a participant in this match, or this method will throw an exception.
	 */
	public int GetParticipantStatus(String ParticipantId)
	{
		return _TBM.getParticipantStatus(ParticipantId);
	}

	/**
	 * Returns the ID of the participant that is considered pending. If no participant is considered pending (i.e., the match is over, etc), this function will return an empty string.
	 */
	public String getPendingParticipantId()
	{
		return Utils.StringOrEmpty(_TBM.getPendingParticipantId());
	}

	/**
	 * Returns the match data from the previous match, if available. Note that this is only provided on the first turn of a rematched match.
	 */
	public byte[] getPreviousMatchData()
	{
		return _TBM.getPreviousMatchData();
	}

	/**
	 * Returns the match ID of the rematch that was created from this match, if any. This will only be non-empty if a rematch has been created.
	 */
	public String getRematchId()
	{
		return Utils.StringOrEmpty(_TBM.getRematchId());
	}

	/**
	 * Returns the current status of the match (one of the MATCH_STATUS constants).
	 */
	public int getStatus()
	{
		return _TBM.getStatus();
	}

	/**
	 * Returns the turn status of the match for the current player (one of the MATCH_TURN_STATUS constants). 
	 */
	public int getTurnStatus()
	{
		return _TBM.getTurnStatus();
	}

	/**
	 * Retrieves the variant specified for this match, if any. A variant is an optional developer-controlled parameter describing the type of game to play, ranging from 1-1023 (inclusive). If this match had no variant specified, returns MATCH_VARIANT_DEFAULT.
	 */
	public int getVariant()
	{
		return _TBM.getVariant();
	}

	/**
	 * Returns the current version of the match.
	 */
	public int getVersion()
	{
		return _TBM.getVersion();
	}

	/**
	 * Returns whether or not this match has been locally modified. If this is True, the local device has match state which has not successfully synced to the server yet. In this state, further mutations to the match will fail with a status of STATUS_MATCH_ERROR_LOCALLY_MODIFIED.
	 */
	public boolean IsLocallyModified()
	{
		return _TBM.isLocallyModified();
	}
}
