package flm.b4a.googleplay;

import java.util.ArrayList;

import com.google.android.gms.common.api.PendingResult;
import com.google.android.gms.games.Games;
import com.google.android.gms.games.GamesStatusCodes;
import com.google.android.gms.games.multiplayer.Multiplayer;
import com.google.android.gms.games.multiplayer.turnbased.OnTurnBasedMatchUpdateReceivedListener;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMatch;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMultiplayer;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMultiplayer.CancelMatchResult;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMultiplayer.InitiateMatchResult;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMultiplayer.LeaveMatchResult;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMultiplayer.LoadMatchResult;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMultiplayer.LoadMatchesResult;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMultiplayer.UpdateMatchResult;

import android.content.Intent;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.collections.List;

@BA.ShortName("GPlayTurnBasedMultiplayer")
@BA.Events(values = {
		"onTurnBasedMatchReceived(Match As GPlayTurnBasedMatch)",
		"onTurnBasedMatchRemoved(MatchID As String)",
})
public class TurnBasedMultiplayerWrapper
{
	protected TurnBasedMultiplayer _TBMP = null;

	public void Initialize()
	{
		_TBMP = Games.TurnBasedMultiplayer;
	}

	public boolean IsInitialized()
	{
		return _TBMP != null;
	}

	public final static String EXTRA_MAX_AUTOMATCH_PLAYERS = Multiplayer.EXTRA_MAX_AUTOMATCH_PLAYERS;
	public final static String EXTRA_MIN_AUTOMATCH_PLAYERS = Multiplayer.EXTRA_MIN_AUTOMATCH_PLAYERS;
	public final static String EXTRA_PLAYER_IDS = Games.EXTRA_PLAYER_IDS;
	public final static String EXTRA_TURN_BASED_MATCH = Multiplayer.EXTRA_TURN_BASED_MATCH;
	public static final String RESULTMAP_MATCH = PendingResultWrapper.RESULTMAP_MATCH;
	public static final String RESULTMAP_MATCHES = PendingResultWrapper.RESULTMAP_MATCHES;
	public static final String RESULTMAP_MATCH_ID = PendingResultWrapper.RESULTMAP_MATCH_ID;
	public static final String RESULTMAP_STATUS_CODE = PendingResultWrapper.RESULTMAP_STATUS_CODE;

	public final static int STATUS_OK = GamesStatusCodes.STATUS_OK;
	public final static int STATUS_CLIENT_RECONNECT_REQUIRED = GamesStatusCodes.STATUS_CLIENT_RECONNECT_REQUIRED;
	public final static int STATUS_INTERNAL_ERROR = GamesStatusCodes.STATUS_INTERNAL_ERROR;

	/** Indicates that the specified match has already had a rematch created. */
	public final static int STATUS_MATCH_ERROR_ALREADY_REMATCHED = GamesStatusCodes.STATUS_MATCH_ERROR_ALREADY_REMATCHED;
	/** Indicates that the match is not currently active. */
	public final static int STATUS_MATCH_ERROR_INACTIVE_MATCH = GamesStatusCodes.STATUS_MATCH_ERROR_INACTIVE_MATCH;
	/** Indicates that the match is not in the correct state to perform the specified action. */
	public final static int STATUS_MATCH_ERROR_INVALID_MATCH_STATE = GamesStatusCodes.STATUS_MATCH_ERROR_INVALID_MATCH_STATE;
	/** Indicates that one or more participants in this match are not in valid states. */
	public final static int STATUS_MATCH_ERROR_INVALID_PARTICIPANT_STATE = GamesStatusCodes.STATUS_MATCH_ERROR_INVALID_PARTICIPANT_STATE;
	/** Indicates that the specified match has already been modified locally. */
	public final static int STATUS_MATCH_ERROR_LOCALLY_MODIFIED = GamesStatusCodes.STATUS_MATCH_ERROR_LOCALLY_MODIFIED;
	/** Indicates that the match data is out of date. */
	public final static int STATUS_MATCH_ERROR_OUT_OF_DATE_VERSION = GamesStatusCodes.STATUS_MATCH_ERROR_OUT_OF_DATE_VERSION;
	/** Indicates that the specified match cannot be found. */
	public final static int STATUS_MATCH_NOT_FOUND = GamesStatusCodes.STATUS_MATCH_NOT_FOUND;

	/** Indicates that this game does not support multiplayer. */
	public final static int STATUS_MULTIPLAYER_DISABLED = GamesStatusCodes.STATUS_MULTIPLAYER_DISABLED;
	/** Indicates that the user is not allowed to create a new multiplayer game at this time. */
	public final static int STATUS_MULTIPLAYER_ERROR_CREATION_NOT_ALLOWED = GamesStatusCodes.STATUS_MULTIPLAYER_ERROR_CREATION_NOT_ALLOWED;
	/** Indicates that the match is not the right type to perform this action on. */
	public final static int STATUS_MULTIPLAYER_ERROR_INVALID_MULTIPLAYER_TYPE = GamesStatusCodes.STATUS_MULTIPLAYER_ERROR_INVALID_MULTIPLAYER_TYPE;
	/** Indicates that this multiplayer operation is not valid, and the server rejected it. */
	public final static int STATUS_MULTIPLAYER_ERROR_INVALID_OPERATION = GamesStatusCodes.STATUS_MULTIPLAYER_ERROR_INVALID_OPERATION;
	/** Indicates that the user attempted to invite another user who was not authorized to see the game.  */
	public final static int STATUS_MULTIPLAYER_ERROR_NOT_TRUSTED_TESTER = GamesStatusCodes.STATUS_MULTIPLAYER_ERROR_NOT_TRUSTED_TESTER;

	/** Indicates that a network error occurred while attempting to retrieve fresh data, and no data was available locally. */
	public final static int STATUS_NETWORK_ERROR_NO_DATA = GamesStatusCodes.STATUS_NETWORK_ERROR_NO_DATA;
	/** Indicates that a network error occurred while attempting to modify data, but the data was successfully modified locally and will be updated on the network the next time the device is able to sync. */
	public final static int STATUS_NETWORK_ERROR_OPERATION_DEFERRED = GamesStatusCodes.STATUS_NETWORK_ERROR_OPERATION_DEFERRED;
	/** Indicates that a network error occurred while attempting to perform an operation that requires network access. */
	public final static int STATUS_NETWORK_ERROR_OPERATION_FAILED = GamesStatusCodes.STATUS_NETWORK_ERROR_OPERATION_FAILED;
	/** Indicates that a network error occurred while attempting to retrieve fresh data, but some locally cached data was available. */
	public final static int STATUS_NETWORK_ERROR_STALE_DATA = GamesStatusCodes.STATUS_NETWORK_ERROR_STALE_DATA;

	/**
	 * Accepts an invitation for a turn-based match. This changes the current player's participant status to STATUS_JOINED.
	 * @param InvitationId The ID of the invitation to be accepted.
	 */
	public PendingResultWrapper AcceptInvitation(String InvitationId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.AcceptInvitation: " + InvitationId);
		PendingResult<InitiateMatchResult> PR = _TBMP.acceptInvitation(GooglePlayConnection.getAPIclient(), InvitationId);
		return new PendingResultWrapper(PR, InitiateMatchResult.class);
	}

	/**
	 * Creates a new turn-based match for the current game. If the provided TurnBasedMatchConfig includes automatch parameters, the server will attempt to find any previously created matches that satisfy these parameters and join the current player into the previous match. If no suitable match can be found, a new match will be created.
	 * @param Config The configuration parameters for the match to create.
	 */
	public PendingResultWrapper CreateMatch(TurnBasedMatchConfigWrapper Config)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.CreateMatch: " + Config);
		PendingResult<InitiateMatchResult> PR = _TBMP.createMatch(GooglePlayConnection.getAPIclient(), Config._TBMC);
		return new PendingResultWrapper(PR, InitiateMatchResult.class);
	}

	/**
	 * Cancels a turn-based match. Once this call succeeds, the match will be removed from local storage. Note that this will cancel the match completely, forcing it to end for all players involved. See LeaveMatch for a different alternative.
	 * @param MatchId ID of the match to cancel.
	 */
	public PendingResultWrapper CancelMatch(String MatchId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.CancelMatch: " + MatchId);
		PendingResult<CancelMatchResult> PR = _TBMP.cancelMatch(GooglePlayConnection.getAPIclient(), MatchId);
		return new PendingResultWrapper(PR, CancelMatchResult.class);
	}

	/**
	 * Declines an invitation for a turn-based match. Note that this will cancel the match for the other participants and remove the match from the caller's local device.
	 * @param InvitationId The ID of the invitation to decline.
	 */
	public void DeclineInvitation(String InvitationId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.DeclineInvitation: " + InvitationId);
		_TBMP.declineInvitation(GooglePlayConnection.getAPIclient(), InvitationId);
	}

	/**
	 * Dismisses an invitation to a turn-based match. Dismissing an invitation will not change the state of the match for the other participants.
	 * @param InvitationId The ID of the invitation to dismiss.
	 */
	public void DismissInvitation(String InvitationId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.DismissInvitation: " + InvitationId);
		_TBMP.dismissInvitation(GooglePlayConnection.getAPIclient(), InvitationId);
	}

	/**
	 * Deletes a match from the server and local storage. Dismissing a match will not change the state of the match for the other participants, but dismissed matches will never be shown to the dismissing player again.
	 * @param MatchId The ID of the match to dismiss.
	 */
	public void DismissMatch(String MatchId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.DismissMatch: " + MatchId);
		_TBMP.dismissMatch(GooglePlayConnection.getAPIclient(), MatchId);
	}

	/**
	 * Indicates that a participant is finished with a match. This will not change the data of the match, but it will transition the match into state MATCH_STATUS_COMPLETE if the match is not already in that state. This function is most commonly used to report that a participant has finished any post-processing steps the game might have and has seen their results in the match.
	 * @param MatchId ID of the match to finish.
	 */
	public PendingResultWrapper FinishMatch(String MatchId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.FinishMatch: matchId=" + MatchId);
		PendingResult<UpdateMatchResult> PR = _TBMP.finishMatch(GooglePlayConnection.getAPIclient(), MatchId);
		return new PendingResultWrapper(PR, UpdateMatchResult.class);
	}

	/**
	 *Marks a match as finished. This should be called when the match is over and all participants have results to be reported (if appropriate). Note that the last client to update a match is responsible for calling FinishMatch on that match.
	 *On the last turn of the match, the client should call this function instead of TakeTurn.
	 *@param MatchId ID of the match to finish.
	 *@param MatchData Data representing the new state of the match after this update. Limited to a maximum of MaxMatchDataSize bytes.
	 *@param Results List of ParticipantResult objects for this match. The client which calls FinishMatch is responsible for reporting the results for all appropriate participants in the match. Not every participant is required to have a result, but providing results for participants who are not in the match is an error.
	 */
	public PendingResultWrapper FinishMatch2(String MatchId, byte[] MatchData, List Results)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("TBMulti.FinishMatch2: matchId=" + MatchId + " results=" + Results + " data=" + MatchData);
		PendingResult<UpdateMatchResult> PR = _TBMP.finishMatch(GooglePlayConnection.getAPIclient(), MatchId, MatchData, Utils.FromB4AListToArrayList_ParticipantResult(Results));
		return new PendingResultWrapper(PR, UpdateMatchResult.class);
	}

	/**
	 *Returns an intent that will let the user see and manage any outstanding invitations and matches. This intent must be invoked with StartActivityForResult.
	 *If the user canceled, the result code will be ACTIVITY_RESULT_CANCELED. If the user selected an invitation or a match to accept, the result code will be ACTIVITY_RESULT_OK and the data intent will contain the selected invitation/match in the extras (EXTRA_TURN_BASED_MATCH).
	 */
	public Intent GetInboxIntent()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.GetInboxIntent");
		return _TBMP.getInboxIntent(GooglePlayConnection.getAPIclient());
	}

	/**
	 * Retrieves a match from the extended data (EXTRA_TURN_BASED_MATCH) of the given intent.
	 */
	public static TurnBasedMatchWrapper GetMatchFromIntentExtra(Intent Intent)
	{
		TurnBasedMatch match = Intent.getParcelableExtra(Multiplayer.EXTRA_TURN_BASED_MATCH);
		return Utils.MatchWrapperOrNull(match);
	}

	/**
	 *Gets the maximum data size per match in bytes. Guaranteed to be at least 128 KB. May increase in the future.
	 *If the service cannot be reached for some reason, this will return -1.
	 */
	public int getMaxMatchDataSize()
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("TBMulti.MaxMatchDataSize=" + _TBMP.getMaxMatchDataSize(GooglePlayConnection.getAPIclient()));
		return _TBMP.getMaxMatchDataSize(GooglePlayConnection.getAPIclient());
	}

	/**
	 *Returns an intent that will let the user select opponents to send an invitation to for a turn based multiplayer match. This intent must be invoked with StartActivityForResult.
	 *The number of players passed in should be the desired number of additional players to select, not including the current player. So, for a game that can handle between 2 and 4 players, MinPlayers would be 1 and MaxPlayers would be 3.
	 *If the user canceled, the result will be ACTIVITY_RESULT_CANCELED. If the user selected players, the result code will be ACTIVITY_RESULT_OK, and the data intent will contain the selected player IDs in EXTRA_PLAYER_IDS and the minimum and maximum numbers of additional auto-match players in EXTRA_MIN_AUTOMATCH_PLAYERS and EXTRA_MAX_AUTOMATCH_PLAYERS respectively. The player IDs in EXTRA_PLAYER_IDS will include only the other players selected, not the current player.
	 *If the AllowAutomatch parameter is set to False, the UI will not display an option for selecting automatch players. Set this to False if your game does not support automatching.
	 *@param MinPlayers The minimum number of players to select (not including the current player).
	 *@param MaxPlayers The maximum number of players to select (not including the current player).
	 *@param AllowAutomatch Whether or not to display an option for selecting automatch players.
	 */
	public Intent GetSelectOpponentsIntent(int MinPlayers, int MaxPlayers, boolean AllowAutomatch)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("TBMulti.GetSelectOpponentsIntent: min=" + MinPlayers + " max=" + MaxPlayers);
		return _TBMP.getSelectOpponentsIntent(GooglePlayConnection.getAPIclient(), MinPlayers, MaxPlayers, AllowAutomatch);
	}

	/**
	 * Retrieves a list of player IDs from the extended data (EXTRA_PLAYER_IDS) of the given intent. Returns Null if the list is empty.
	 */
	public static Object GetPlayerIDsFromIntentExtra(Intent Intent)
	{
		// The return type is Object, not List, to allow Null value
		ArrayList<String> PlayerIDs = Intent.getStringArrayListExtra(EXTRA_PLAYER_IDS);
		if (PlayerIDs == null || PlayerIDs.isEmpty())
			return null;
		else
			return Utils.FromArrayListToB4AList_String(PlayerIDs);
	}

	/**
	 * Leaves the specified match when it is not the current player's turn. If this takes the match to fewer than two participants, the match will be canceled. Call LeaveMatchDuringTurn instead to leave during the current player's turn.
	 * @param MatchId ID of the match to leave.
	 */
	public PendingResultWrapper LeaveMatch(String MatchId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.LeaveMatch: " + MatchId);
		PendingResult<LeaveMatchResult> PR = _TBMP.leaveMatch(GooglePlayConnection.getAPIclient(), MatchId);
		return new PendingResultWrapper(PR, LeaveMatchResult.class);
	}

	/**
	 * Leaves the specified match during the current player's turn. If this takes the match to fewer than two participants, the match will be canceled. The provided PendingParticipantId will be used to determine which participant should act next. If no pending participant is provided and the match has available auto-match slots, the match will wait for additional players to be found. If there are no auto-match slots available for this match, a pending participant ID is required. Call LeaveMatch instead to leave when it is not the current player's turn.
	 * @param MatchId ID of the match to leave.
	 * @param PendingParticipantId ID of the participant who will be set to pending after this update succeeds, or an empty string to wait for additional automatched players (if possible).
	 */
	public PendingResultWrapper LeaveMatchDuringTurn(String MatchId, String PendingParticipantId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.LeaveMatchDuringTurn: " + MatchId + " " + PendingParticipantId);
		PendingResult<LeaveMatchResult> PR;
		if (PendingParticipantId.length() > 0)
			PR = _TBMP.leaveMatchDuringTurn(GooglePlayConnection.getAPIclient(), MatchId, PendingParticipantId);
		else
			PR = _TBMP.leaveMatchDuringTurn(GooglePlayConnection.getAPIclient(), MatchId, null);
		return new PendingResultWrapper(PR, LeaveMatchResult.class);
	}

	/**
	 * Loads a specified turn-based match.
	 * @param MatchId The ID of the match to retrieve.
	 */
	public PendingResultWrapper LoadMatch(String MatchId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.LoadMatch: " + MatchId);
		PendingResult<LoadMatchResult> PR = _TBMP.loadMatch(GooglePlayConnection.getAPIclient(), MatchId);
		return new PendingResultWrapper(PR, LoadMatchResult.class);
	}

	public final static int MATCH_TURN_STATUS_COMPLETE = TurnBasedMatch.MATCH_TURN_STATUS_COMPLETE;
	public final static int MATCH_TURN_STATUS_INVITED = TurnBasedMatch.MATCH_TURN_STATUS_INVITED;
	public final static int MATCH_TURN_STATUS_MY_TURN = TurnBasedMatch.MATCH_TURN_STATUS_MY_TURN;
	public final static int MATCH_TURN_STATUS_THEIR_TURN = TurnBasedMatch.MATCH_TURN_STATUS_THEIR_TURN;

	/**
	 *Asynchronously loads turn-based matches for the current game. Matches with any specified turn status codes will be returned.
	 *Valid turn status values are MATCH_TURN_STATUS_INVITED, MATCH_TURN_STATUS_MY_TURN, MATCH_TURN_STATUS_THEIR_TURN, or MATCH_TURN_STATUS_COMPLETE. Note that if your game implements both turn-based and real-time multiplayer, requesting MATCH_TURN_STATUS_INVITED will return invitations for both turn-based matches and real-time matches. 
	 *@param MatchTurnStatuses Array of turn statuses to request.
	 */
	public PendingResultWrapper LoadMatchesByStatus(int[] MatchTurnStatuses)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.LoadMatchesByStatus: " + MatchTurnStatuses);
		PendingResult<LoadMatchesResult> PR = _TBMP.loadMatchesByStatus(GooglePlayConnection.getAPIclient(), Multiplayer.SORT_ORDER_MOST_RECENT_FIRST, MatchTurnStatuses);
		return new PendingResultWrapper(PR, LoadMatchesResult.class);
	}

	private OnTurnBasedMatchUpdateReceivedListener _onTurnBasedMatchUpdateReceivedListener = null;
	private class MatchUpdateListener implements OnTurnBasedMatchUpdateReceivedListener
	{
		private BA _ba;
		private String _evtPrefix;

		public MatchUpdateListener(BA ba)
		{
			_ba = ba;
			_evtPrefix = GooglePlayConnection._evtPrefix; 
		}

		@Override
		public void onTurnBasedMatchReceived(TurnBasedMatch match) {
			if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.onTurnBasedMatchReceived: " + match);
			_ba.raiseEvent2(this, true, _evtPrefix + "_onturnbasedmatchreceived",
					false, new Object[] { Utils.MatchWrapperOrNull(match) });
		}

		@Override
		public void onTurnBasedMatchRemoved(String matchId) {
			if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.onTurnBasedMatchRemoved: " + matchId);
			_ba.raiseEvent2(this, true, _evtPrefix + "_onturnbasedmatchremoved",
					false, new Object[] { Utils.StringOrEmpty(matchId) });
		}
	}

	/**
	 *Registers a listener to intercept incoming match updates for the currently signed-in user. If a listener is registered by this function, the incoming match update will not generate a status bar notification as long as this client remains connected.
	 *Note that only one match update listener may be active at a time. Calling this function while another match update listener was previously registered will replace the original listener with the new one.
	 *The listener will raise the onTurnBasedMatchReceived and onTurnBasedMatchRemoved events.
	 */
	public void RegisterMatchUpdateListener(BA ba)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.RegisterMatchUpdateListener");
		if (_onTurnBasedMatchUpdateReceivedListener == null) {
			_onTurnBasedMatchUpdateReceivedListener = new MatchUpdateListener(ba);
		}
		_TBMP.registerMatchUpdateListener(GooglePlayConnection.getAPIclient(), _onTurnBasedMatchUpdateReceivedListener);
	}

	/**
	 * Creates a rematch of a previously completed turn-based match. The new match will have the same participants as the previous match. Note that only one rematch may be created from any single completed match, and only by a player that has already called FinishMatch on the match.
	 * @param MatchId The ID of the previous match to re-create.
	 */
	public PendingResultWrapper Rematch(String MatchId)
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.Rematch");
		PendingResult<InitiateMatchResult> PR = _TBMP.rematch(GooglePlayConnection.getAPIclient(), MatchId);
		return new PendingResultWrapper(PR, InitiateMatchResult.class);
	}

	/**
	 *Updates a match with new turn data. The participant that is passed in as the pending participant will be notified that it is their turn to take action. If no pending participant is provided and the match has available auto-match slots, the match will wait for additional players to be found. If there are no auto-match slots available for this match, a pending participant ID is required.
	 *For the final turn of the match, there is no need to call this function. Instead, call FinishMatch directly.
	 * @param MatchId ID of the match to update.
	 * @param MatchData Data representing the new state of the match after this update. Limited to a maximum of MaxMatchDataSize bytes.
	 * @param PendingParticipantId ID of the participant who will be set to pending after this update succeeds, or an empty string to wait for additional automatched players (if possible).
	 */
	public PendingResultWrapper TakeTurn(String MatchId, byte[] MatchData, String PendingParticipantId)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("TBMulti.TakeTurn: matchId=" + MatchId + " pending participant=" + PendingParticipantId + " data=" + MatchData);
		final PendingResult<UpdateMatchResult> PR;
		if (PendingParticipantId.length() == 0)
			PR = _TBMP.takeTurn(GooglePlayConnection.getAPIclient(), MatchId, MatchData, null);
		else
			PR = _TBMP.takeTurn(GooglePlayConnection.getAPIclient(), MatchId, MatchData, PendingParticipantId);
		return new PendingResultWrapper(PR, UpdateMatchResult.class);
	}

	/**
	 *Updates a match with new turn data. The participant that is passed in as the pending participant will be notified that it is their turn to take action. If no pending participant is provided and the match has available auto-match slots, the match will wait for additional players to be found. If there are no auto-match slots available for this match, a pending participant ID is required.
	 *Note that players will not receive invitations for matches until this function is called. An invitation will be sent to a player the first time they are set as the pending participant of a match.
	 *For the final turn of the match, there is no need to call this function. Instead, call FinishMatch directly.
	 * @param MatchId ID of the match to update.
	 * @param MatchData Data representing the new state of the match after this update. Limited to a maximum of MaxMatchDataSize bytes.
	 * @param PendingParticipantId ID of the participant who will be set to pending after this update succeeds, or an empty string to wait for additional automatched players (if possible).
	 * @param Results List of ParticipantResult objects for this match. Note that the results reported here should be final - if results reported later conflict with these values, the returned value will indicate a conflicted result by returning MATCH_RESULT_DISAGREED. This is most useful for cases where a participant knows their results early. For example, a single elimination game where participants are eliminated as the game continues might wish to specify results for the eliminated participants here.
	 */
	public PendingResultWrapper TakeTurn2(String MatchId, byte[] MatchData, String PendingParticipantId, List Results)
	{
		if (GooglePlayConnection.DEBUG)
			BA.Log("TBMulti.TakeTurn2: matchId=" + MatchId + " pending participant=" + PendingParticipantId + " results=" + Results + " data=" + MatchData);
		final PendingResult<UpdateMatchResult> PR;
		if (PendingParticipantId.length() == 0)
			PR = _TBMP.takeTurn(GooglePlayConnection.getAPIclient(), MatchId, MatchData, null, Utils.FromB4AListToArrayList_ParticipantResult(Results));
		else
			PR = _TBMP.takeTurn(GooglePlayConnection.getAPIclient(), MatchId, MatchData, PendingParticipantId, Utils.FromB4AListToArrayList_ParticipantResult(Results));
		return new PendingResultWrapper(PR, UpdateMatchResult.class);
	}

	/**
	 * Unregisters this client's match update listener, if any. Any new match updates will generate status bar notifications as normal.
	 */
	public void UnregisterMatchUpdateListener()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("TBMulti.UnregisterMatchUpdateListener");
		_TBMP.unregisterMatchUpdateListener(GooglePlayConnection.getAPIclient());
		_onTurnBasedMatchUpdateReceivedListener = null;
	}
}
