package flm.b4a.googleplay;

import java.util.ArrayList;

import com.google.android.gms.common.images.ImageManager;
import com.google.android.gms.common.images.ImageManager.OnImageLoadedListener;
import com.google.android.gms.games.Game;
import com.google.android.gms.games.Games;
import com.google.android.gms.games.GamesActivityResultCodes;
import com.google.android.gms.games.Player;
import com.google.android.gms.games.PlayerBuffer;
import com.google.android.gms.games.PlayerLevel;
import com.google.android.gms.games.achievement.AchievementBuffer;
import com.google.android.gms.games.leaderboard.Leaderboard;
import com.google.android.gms.games.leaderboard.LeaderboardBuffer;
import com.google.android.gms.games.leaderboard.LeaderboardScore;
import com.google.android.gms.games.leaderboard.LeaderboardScoreBuffer;
import com.google.android.gms.games.leaderboard.LeaderboardVariant;
import com.google.android.gms.games.leaderboard.ScoreSubmissionData;
import com.google.android.gms.games.multiplayer.Invitation;
import com.google.android.gms.games.multiplayer.InvitationBuffer;
import com.google.android.gms.games.multiplayer.Participant;
import com.google.android.gms.games.multiplayer.ParticipantResult;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMatch;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMatchBuffer;
import com.google.android.gms.games.request.GameRequest;
import com.google.android.gms.games.snapshot.Snapshot;
import com.google.android.gms.games.snapshot.SnapshotContents;
import com.google.android.gms.games.snapshot.SnapshotMetadata;
import com.google.android.gms.games.snapshot.SnapshotMetadataBuffer;
import com.google.android.gms.games.stats.PlayerStats;

import android.app.Activity;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.IOnActivityResult;
import anywheresoftware.b4a.objects.ImageViewWrapper;
import anywheresoftware.b4a.objects.collections.List;
import anywheresoftware.b4a.objects.collections.Map;
import anywheresoftware.b4a.objects.drawable.BitmapDrawable;

@BA.ShortName("GPlayUtils")
@BA.Events(values = { "onActivityResult(ID As String, ResultCode As Int, ResultIntent As Intent)" })
public class Utils // This class is a collection of static methods mainly used in other classes
{
	/**
	 * Gets the application ID linked to this client instance.
	 */
	public static String getAppId()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("AppId=" + Games.getAppId(GooglePlayConnection.getAPIclient()));
		return Utils.StringOrEmpty(Games.getAppId(GooglePlayConnection.getAPIclient()));
	}

	/**
	 *Gets the name of the currently selected account. This is the account the user has chosen to use for Google Play Games.
	 *Note that the Google API client must be connected to call this function, and your app must have AddPermission(android.permission.GET_ACCOUNTS) declared in your manifest in order to use this function.
	 */
	public static String getCurrentAccountName()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("CurrentAccountName=" + Games.getCurrentAccountName(GooglePlayConnection.getAPIclient()));
		return Utils.StringOrEmpty(Games.getCurrentAccountName(GooglePlayConnection.getAPIclient()));
	}

	/**
	 *Gets an intent to show the Settings screen that allows the user to configure games-related features for the current game. This intent must be invoked with StartActivityForResult.
	 *An ACTIVITY_RESULT_RECONNECT_REQUIRED may be returned as the result code of the onActivityResult event if the Google API client ends up in an inconsistent state.
	 *Most applications will not need to call this directly, since the Settings UI is already reachable from most other Games UI screens (achievements, leaderboards, etc.) via a menu item.
	 */
	public static Intent GetSettingsIntent()
	{
		if (GooglePlayConnection.DEBUG) BA.Log("GetSettingsIntent");
		return Games.getSettingsIntent(GooglePlayConnection.getAPIclient());
	}

	private static IOnActivityResult iOnActivityResult;
	public final static int ACTIVITY_RESULT_OK = Activity.RESULT_OK;
	public final static int ACTIVITY_RESULT_CANCELED = Activity.RESULT_CANCELED;
	public final static int ACTIVITY_RESULT_RECONNECT_REQUIRED = GamesActivityResultCodes.RESULT_RECONNECT_REQUIRED;
	public final static int ACTIVITY_RESULT_INVALID_ROOM = GamesActivityResultCodes.RESULT_INVALID_ROOM;
	public final static int ACTIVITY_RESULT_LEFT_ROOM = GamesActivityResultCodes.RESULT_LEFT_ROOM;

	/** Launches an activity for which you would like a result when it finished. When this activity exits, an onActivityResult event is raised.
	 * @param ID An unique identifier helping you identify which activity exited. */
	public static void StartActivityForResult(final BA ba, Intent i, final String ID)
	{
		iOnActivityResult = new IOnActivityResult() {
			@Override
			public void ResultArrived(int resultCode, Intent resultIntent) {
				ba.raiseEvent(this, GooglePlayConnection._evtPrefix + "_onactivityresult",
						new Object[] { ID, resultCode, resultIntent });
			}
		};
		ba.startActivityForResult(iOnActivityResult, i);
	}

	@SuppressWarnings("unchecked")
	protected static ArrayList<String> FromStringObjectToArrayList(Object objStr)
	{
		if (objStr == null)
			return null;
		ArrayList<String> arrStrings;
		if (objStr instanceof ArrayList)
			arrStrings = (ArrayList<String>) objStr;
		else if (objStr instanceof List)
			arrStrings = Utils.FromB4AListToArrayList_String((List) objStr);
		else
			arrStrings = Utils.FromStringsToArrayList((String[]) objStr);
		return arrStrings;
	}

	protected static ArrayList<String> FromStringsToArrayList(String[] Strings)
	{
		ArrayList<String> arrStrings = new ArrayList<String>();
		for (String str: Strings) {
			arrStrings.add(str);
		}
		return arrStrings;
	}

	protected static ArrayList<String> FromB4AListToArrayList_String(List list)
	{
		ArrayList<String> arrList = new ArrayList<String>();
		for (int i = 0; i < list.getSize(); i++) {
			arrList.add((String) list.Get(i));
		}
		return arrList;
	}

	protected static ArrayList<ParticipantResult> FromB4AListToArrayList_ParticipantResult(List list)
	{
		ArrayList<ParticipantResult> arrList = new ArrayList<ParticipantResult>();
		for (int i = 0; i < list.getSize(); i++) {
			ParticipantResultWrapper prw = (ParticipantResultWrapper) list.Get(i);
			arrList.add(prw._pResult);
		}
		return arrList;
	}

	protected static List FromListToB4AList_Player(java.util.List<Player> lstPlayer)
	{
		List list = new List();
		list.Initialize();
		for (Player pl: lstPlayer) {
			list.Add(pl);
		}
		return list;
	}

	protected static List FromArrayListToB4AList_String(ArrayList<String> arrList)
	{
		List list = new List();
		list.Initialize();
		for (String str: arrList) {
			list.Add(StringOrEmpty(str));
		}
		return list;
	}

	protected static List FromArrayListToB4AList_Request(ArrayList<GameRequest> arrList)
	{
		List list = new List();
		list.Initialize();
		for (GameRequest rq: arrList) {
			list.Add(new GameRequestWrapper(rq));
		}
		return list;
	}

	protected static List FromArrayListToB4AList_LeaderboardVariant(ArrayList<LeaderboardVariant> arrList)
	{
		List list = new List();
		list.Initialize();
		for (LeaderboardVariant lbv: arrList) {
			list.Add(new LeaderboardVariantWrapper(lbv));
		}
		return list;
	}

	protected static List FromArrayListToB4AList_Participant(ArrayList<Participant> arrList)
	{
		List list = new List();
		list.Initialize();
		for (Participant par: arrList) {
			list.Add(new ParticipantWrapper(par));
		}
		return list;
	}

	protected static AchievementWrapper[] FromBufferToArray_Achievement(AchievementBuffer Buffer)
	{
		if (Buffer == null)
			return null;
		int Count = Buffer.getCount();
		AchievementWrapper[] AW = new AchievementWrapper[Count];
		for (int i = 0; i < Count; i++) {
			AW[i] = new AchievementWrapper(Buffer.get(i));
		}
		return AW;
	}

	protected static InvitationWrapper[] FromBufferToArray_Invitation(InvitationBuffer Buffer)
	{
		if (Buffer == null)
			return null;
		int Count = Buffer.getCount();
		InvitationWrapper[] IW = new InvitationWrapper[Count];
		for (int i = 0; i < Count; i++) {
			IW[i] = new InvitationWrapper(Buffer.get(i));
		}
		return IW;
	}

	protected static LeaderboardWrapper[] FromBufferToArray_Leaderboard(LeaderboardBuffer Buffer)
	{
		if (Buffer == null)
			return null;
		int Count = Buffer.getCount();
		LeaderboardWrapper[] LW = new LeaderboardWrapper[Count];
		for (int i = 0; i < Count; i++) {
			LW[i] = new LeaderboardWrapper(Buffer.get(i));
		}
		return LW;
	}

	protected static LeaderboardScoreWrapper[] FromBufferToArray_LeaderboardScore(LeaderboardScoreBuffer Buffer)
	{
		if (Buffer == null)
			return null;
		int Count = Buffer.getCount();
		LeaderboardScoreWrapper[] LSW = new LeaderboardScoreWrapper[Count];
		for (int i = 0; i < Count; i++) {
			LSW[i] = new LeaderboardScoreWrapper(Buffer.get(i));
		}
		return LSW;
	}

	protected static PlayerWrapper[] FromBufferToArray_Player(PlayerBuffer Buffer)
	{
		if (Buffer == null)
			return null;
		int Count = Buffer.getCount();
		PlayerWrapper[] PW = new PlayerWrapper[Count];
		for (int i = 0; i < Count; i++) {
			PW[i] = new PlayerWrapper(Buffer.get(i));
		}
		return PW;
	}

	protected static SnapshotMetadataWrapper[] FromBufferToArray_SnapshotMetadata(SnapshotMetadataBuffer Buffer)
	{
		if (Buffer == null)
			return null;
		int Count = Buffer.getCount();
		SnapshotMetadataWrapper[] SMW = new SnapshotMetadataWrapper[Count];
		for (int i = 0; i < Count; i++) {
			SMW[i] = new SnapshotMetadataWrapper(Buffer.get(i));
		}
		return SMW;
	}

	protected static TurnBasedMatchWrapper[] FromBufferToArray_TurnBasedMatch(TurnBasedMatchBuffer Buffer)
	{
		if (Buffer == null)
			return null;
		int Count = Buffer.getCount();
		TurnBasedMatchWrapper[] TBM = new TurnBasedMatchWrapper[Count];
		for (int i = 0; i < Count; i++) {
			TBM[i] = new TurnBasedMatchWrapper(Buffer.get(i));
		}
		return TBM;
	}

	protected static String StringOrEmpty(String str)
	{
		if (str == null)
			return "";
		else
			return str;
	}

	protected static GameWrapper GameWrapperOrNull(Game game)
	{
		if (game == null)
			return null;
		else
			return new GameWrapper(game);
	}

	protected static InvitationWrapper InvitationWrapperOrNull(Invitation invitation)
	{
		if (invitation == null)
			return null;
		else
			return new InvitationWrapper(invitation);
	}

	protected static LeaderboardWrapper LeaderboardWrapperOrNull(Leaderboard leaderboard)
	{
		if (leaderboard == null)
			return null;
		else
			return new LeaderboardWrapper(leaderboard);
	}

	protected static LeaderboardScoreWrapper LeaderboardScoreWrapperOrNull(LeaderboardScore score)
	{
		if (score == null)
			return null;
		else
			return new LeaderboardScoreWrapper(score);
	}

	protected static TurnBasedMatchWrapper MatchWrapperOrNull(TurnBasedMatch Match)
	{
		if (Match == null)
			return null;
		else
			return new TurnBasedMatchWrapper(Match);
	}

	protected static ParticipantWrapper ParticipantWrapperOrNull(Participant participant)
	{
		if (participant == null)
			return null;
		else
			return new ParticipantWrapper(participant);
	}

	protected static PlayerWrapper PlayerWrapperOrNull(Player player)
	{
		if (player == null)
			return null;
		else
			return new PlayerWrapper(player);
	}

	protected static PlayerLevelWrapper PlayerLevelWrapperOrNull(PlayerLevel level)
	{
		if (level == null)
			return null;
		else
			return new PlayerLevelWrapper(level);
	}

	protected static PlayerStatsWrapper PlayerStatsWrapperOrNull(PlayerStats playerStats)
	{
		if (playerStats == null)
			return null;
		else
			return new PlayerStatsWrapper(playerStats);
	}

	protected static ScoreSubmissionDataWrapper ScoreSubmissionDataWrapperOrNull(ScoreSubmissionData SSD)
	{
		if (SSD == null)
			return null;
		else
			return new ScoreSubmissionDataWrapper(SSD);
	}

	protected static SnapshotWrapper SnapshotWrapperOrNull(Snapshot snapshot)
	{
		if (snapshot == null)
			return null;
		else
			return new SnapshotWrapper(snapshot);
	}

	protected static SnapshotContentsWrapper ContentsWrapperOrNull(SnapshotContents Contents)
	{
		if (Contents == null)
			return null;
		else
			return new SnapshotContentsWrapper(Contents);
	}

	protected static SnapshotMetadataWrapper MetadataWrapperOrNull(SnapshotMetadata metadata)
	{
		if (metadata == null)
			return null;
		else
			return new SnapshotMetadataWrapper(metadata);
	}

	protected static String UriStringOrEmpty(Uri uri)
	{
		if (uri == null)
			return "";
		else
			return uri.toString();
	}

	private static ImageManager _ImgMgr;
	private static OnImageLoadedListener _ImgLoadedListener;
	private static class ImageLoadedListener implements OnImageLoadedListener
	{
		private BA _ba;
		private String _evt;

		public ImageLoadedListener(BA ba)
		{
			_ba = ba;
			_evt = GooglePlayConnection._evtPrefix + "_onimageloaded";
		}

		@Override
		public void onImageLoaded(Uri uri, Drawable drawable, boolean isRequestedDrawable)
		{
			if (GooglePlayConnection.DEBUG)
				BA.Log("onImageLoaded: uri=" + uri + " drawable=" + drawable + " isrequested=" + isRequestedDrawable);
			BitmapDrawable bmpDrawable = new BitmapDrawable();
			bmpDrawable.setObject((android.graphics.drawable.BitmapDrawable) drawable);
			_ba.raiseEvent(this, _evt, new Object[] { UriStringOrEmpty(uri), bmpDrawable });
		}
	}

	protected static synchronized void LoadImage(BA ba, Uri Uri, ImageViewWrapper IV)
	{
		if (_ImgMgr == null)
			_ImgMgr = ImageManager.create(ba.context);
		if (IV != null) {
			_ImgMgr.loadImage(IV.getObject(), Uri);
			return;
		}
		if (_ImgLoadedListener == null) {
			_ImgLoadedListener = new ImageLoadedListener(ba);
		}
		_ImgMgr.loadImage(_ImgLoadedListener, Uri);
	}

	/** Retrieves the status code from the map returned by a GPlayPendingResult. */
	public int GetStatusCode(Map Result)
	{
		return (int) Result.Get(PendingResultWrapper.RESULTMAP_STATUS_CODE);
	}
}
